/*
   Licensed Materials - Property of IBM
   5724-M22, 5724-M23, 5724-M24, 5724-L01, 5724-R31
   Copyright IBM Corporation 2007, 2008. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bspace.BSpacePortletMenu");

dojo.require("dijit._Widget");
dojo.require("dijit._base.popup");
dojo.require("dijit.Menu");
dojo.require("dojox.layout.ContentPane");

dojo.declare("com.ibm.bspace.BSpacePortletMenu", dijit._Widget, {
	isContainer: true,
	
	constructor: function() {
		this.min = false;
		this.max = false;
		this.originalColumnWidth=null;
		
		this.messages = null;
		this.widgetWidth = null;
		this.widgetHeight = null;
		this.widgetBodyHeight = null;
		this.widgetBodyWidth = null;
		this.isRTL = false;
		this.menuNoActionsText=null;
		this.loadingImage=null;
		
		this.menuPortletURL=null;
		
		this.widgetId = null;		
		this.expandedTwistyID = "expandedTwistyID";
		
		// internal data
		this.bidi=null;
	},
			   
	postCreate: function() {
		var node = this.domNode;
		while (node && !node.dir)
			node = node.parentNode;
		if (node && node.dir && (node.dir.toLowerCase() == 'rtl'))
			this.bidi = true;
    if (this.bidi) {
			this.domNode.align = 'right';
		} else {
			this.domNode.align = 'left';
		}

		this.createMenu(this.widgetId);
		
		var columnNode = this.findContainingDiv(this.widgetId);			
		this.originalColumnWidth = columnNode.width;		
	},

	postMixInProperties: function()	{
    dojo.requireLocalization("com.ibm.bspace", "BSpaceResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bspace", "BSpaceResources");
	},
		
	createMenu: function(id) {
		var menuA = document.createElement('a');
		menuA.id="menu_"+id;
					
		this.connect(menuA, "onkeypress", dojo.hitch(this, function(){
			dojo.global.accessibleShowMenu(event,id, this.menuNoActionsText, this.isRTL, this.menuPortletURL, this.loadingImage);
		}));
									
		this.connect(menuA, "onclick", dojo.hitch(this, function(){
			dojo.global.showPortletMenu(id, this.menuNoActionsText, this.isRTL, this.menuPortletURL, null, this.loadingImage);
		}));
					
		this.domNode.appendChild(menuA);
					
    var img = document.createElement('img');
    img.id="menu_"+id+"_img"
    img.src = "img/controlsCommon/ctrl_arrow.gif";
    img.onmouseover = function() {if(!this.clicked) this.src = "img/controlsCommon/ctrl_arrow_hov.gif";};
    img.onmouseout = function() {if(!this.clicked) this.src = "img/controlsCommon/ctrl_arrow.gif";};
		      
    menuA.appendChild(img);
		
		// register dropdown menu													
		javascriptEventController.register("menu_" + id);
		javascriptEventController.register("mouseoverTable_" + id, null);
		      		        
    var imgS = document.createElement('IMG');
    imgS.src = 'img/controlsCommon/ctrl_widget_sep.gif';
    imgS.style.paddingLeft = '3px';
    this.domNode.appendChild(imgS);
					    
    // minimize icon
    var imgMin = document.createElement('img');
    imgMin.id = 'minicon_'+id;
    imgMin.src = "img/controlsCommon/ctrl_min.gif";
    imgMin.onmouseover = function() { this.src = "img/controlsCommon/ctrl_min_hov.gif" };
    imgMin.onmouseout = function() { this.src = "img/controlsCommon/ctrl_min.gif" };
    imgMin.style.cursor ='pointer';
    imgMin.title = this.messages.Minimize;
    this.domNode.appendChild(imgMin);
    if (this.min) imgMin.style.display = 'none';

		this.connect(imgMin, "onclick", dojo.hitch(this, function(){
			this.doMinIcon(id);
		}));
			                      
		// restore icon
		var imgRestore = document.createElement('img');
		imgRestore.id = 'restoreicon_'+id;
		imgRestore.src = "img/controlsCommon/ctrl_rest.gif";
		imgRestore.onmouseover = function() { this.src = "img/controlsCommon/ctrl_rest_hov.gif" };
		imgRestore.onmouseout = function() { this.src = "img/controlsCommon/ctrl_rest.gif" };
		imgRestore.style.cursor ='pointer';
		imgRestore.title = this.messages.Restore;
		if (!this.max && !this.min) imgRestore.style.display = 'none';
		this.connect(imgRestore, "onclick", dojo.hitch(this, function(){
			this.doRestoreIcon(id);
		}));

		this.domNode.appendChild(imgRestore);
			            
		// maximize icon
		var imgMax = document.createElement('img');
		imgMax.id = 'maxicon_'+id;
		imgMax.src = "img/controlsCommon/ctrl_max.gif";
		imgMax.onmouseover = function() { this.src = "img/controlsCommon/ctrl_max_hov.gif" };
		imgMax.onmouseout = function() { this.src = "img/controlsCommon/ctrl_max.gif" };
		imgMax.style.cursor ='pointer';
		imgMax.title = this.messages.Maximize;
		if (this.max) imgMax.style.display = 'none';
		this.connect(imgMax, "onclick", dojo.hitch(this, function(){
			this.doMaxIcon(id);
		}));
		this.domNode.appendChild(imgMax);
	},
		
	doMinIcon: function(widgetId) {
		if (iWidgetContainer) {
			var iWidget = iWidgetContainer.getWidgetById(widgetId);
			if (iWidget) {
				if (this.max) {
					var payload = new Object();
					payload.newWidth = this.widgetBodyWidth;
					payload.newHeight = this.widgetBodyHeight;
	
					try { 
						iWidget.iScope.iContext.iEvents.fireEvent("onSizeChanged",null, payload);
					} catch (error) {
						console.debug(error);
					}

					var columnNode = this.findContainingDiv(widgetId);					
					columnNode.width = this.originalColumnWidth;
							
					this.showOtherWidgets(true, widgetId);
				}	else {
					var portletBodyNode = dojo.byId("wpsPortletBody_" + this.widgetId);
					this.widgetBodyWidth = portletBodyNode.offsetWidth;
					this.widgetBodyHeight = portletBodyNode.offsetHeight;
				}
						
				dojo.byId("wpsPortletBody_"+widgetId).style.display="none";
						
				dojo.byId("minicon_"+widgetId).style.display="none";
				dojo.byId("restoreicon_"+widgetId).style.display="inline";
				dojo.byId("maxicon_"+widgetId).style.display="inline";
						
				this.min=true;
				this.max=false;
			}		
		}
	},
				
	doRestoreIcon: function(widgetId) {
		if (iWidgetContainer) {
			var iWidget = iWidgetContainer.getWidgetById(widgetId);
			if (iWidget) {
				var mainContentNode = dojo.byId("mainContent");

				var maxicon = dojo.byId("maxicon_"+widgetId);
				var minicon = dojo.byId("minicon_"+widgetId);
						
				dojo.byId("wpsPortletBody_"+widgetId).style.display="";

				if (this.widgetBodyWidth) {
					var payload = new Object();
					payload.newWidth = this.widgetBodyWidth;
					payload.newHeight = this.widgetBodyHeight;
	
					try { 
						iWidget.iScope.iContext.iEvents.fireEvent("onSizeChanged",null, payload);
					} catch (error) {
						console.debug(error);
					}
			
					//show all other widgets on the page
					this.showOtherWidgets(true, widgetId);

					var columnNode = this.findContainingDiv(widgetId);			
					columnNode.width = this.originalColumnWidth;
				}
						
				if (minicon.style.display=="none")
					minicon.style.display="inline";
							
				if (maxicon.style.display=="none")
					maxicon.style.display="inline";

				dojo.byId("restoreicon_"+widgetId).style.display="none";

				this.min = false;
				this.max = false;
			}		
		}
	},
	
	findContainingDiv: function(widgetId) {
		var node = dojo.byId("mouseoverTable_" + widgetId);
		return this._findParent(node);
	},
	
	findContainingDivById: function(widgetId) {
		var node = dojo.byId(widgetId);
		return this._findParent(node);
	},
	
	_findParent: function(node) {
		var parent = node.parentNode;
		
		while(parent!=null) {	
			parent = parent.parentNode;
			if (!!(parent.id) && parent.id.indexOf("pageLayoutColumn_")!=-1)
				break;
		}	
			
		return parent;
	},

	doMaxIcon:function(widgetId) {
			var mainContentNode = dojo.byId("mainContent");
			var maxWidth = mainContentNode.offsetWidth;
			var maxHeight = mainContentNode.offsetHeight;

			var portletNode = dojo.byId("mouseoverTable_" + widgetId);
			this.widgetWidth = portletNode.offsetWidth;
			this.widgetHeight = portletNode.offsetHeight;

			var portletBodyNode = dojo.byId("wpsPortletBody_" + widgetId);
			if (!this.min)
			{
				this.widgetBodyWidth = portletBodyNode.offsetWidth;
				this.widgetBodyHeight = portletBodyNode.offsetHeight;
			}
						
			var columnNode = this.findContainingDiv(widgetId);			
						
			var iconDivNode = dojo.byId("BSpaceWidgetAction_"+widgetId);
			var barHeight = iconDivNode.offsetHeight;			

			if (iWidgetContainer)
			{
					var iWidget = iWidgetContainer.getWidgetById(widgetId);
					if (iWidget)
					{
						//hide all other widgets on the page
						this.showOtherWidgets(false, widgetId);


						var payload = new Object();
						payload.newWidth = maxWidth - 60;
						payload.newHeight = maxHeight-barHeight;
						
						// defect 90102
						var availHeight = screen.height - 450;
						if (payload.newHeight > availHeight) {
							payload.newHeight = availHeight;
						}
						// end of fix for defect 90102
						
							try { 
								iWidget.iScope.iContext.iEvents.fireEvent("onSizeChanged",null, payload);
							} catch (error) {
								console.debug(error);
							}
						
						dojo.byId("wpsPortletBody_"+widgetId).style.display="";
						dojo.byId("minicon_"+widgetId).style.display="inline";
						dojo.byId("restoreicon_"+widgetId).style.display="inline";
						dojo.byId("maxicon_"+widgetId).style.display="none";
						columnNode.width="100%";
						
						this.min=false;
						this.max=true;
					}		
			}
	},
				
	showOtherWidgets: function(bShow, widgetId) {
		var mainContentNode = dojo.byId("mainContent");

		var display="none";
		if (bShow)
			display="";
			
		var self = this;
					
		dojo.query(".wpsPortlet", mainContentNode).forEach(function(node){
			if (node.id) {
				var pos1 = node.id.indexOf("mouseoverTable_");
				var pos2 = node.id.indexOf(widgetId);
				if (pos1!=-1 && pos2==-1)	{								

					// defect 90102
					var tdObj = self.findContainingDivById(node.id);
					if (bShow) {
						var org_width = tdObj.getAttribute("width_org");
						if (org_width) {
							tdObj.width = org_width;
							tdObj.removeAttribute("width_org");
						}
					} else {
						var width = tdObj.width;
						var width_org_att = tdObj.getAttribute("width_org");
						if (!width_org_att && width && width.indexOf("0") != 0) {
							tdObj.setAttribute("width_org", width);
						}
						tdObj.removeAttribute("width");
					}
					// end of fix for defect 90102

					dojo.style(node, "display", display); 
					
				}
			}
		}); 	
	}
})
