/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2007 - All Rights reserved.  **/

if(!dojo._hasResource["ibm.portal.portlet.portlet"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.portal.portlet.portlet"] = true;
dojo.provide("ibm.portal.portlet.portlet")

dojo.require("dojox.data.dom");

dojo.require("ibm.portal.xml.xpath");
dojo.require("ibm.portal.xml.xslt");
dojo.require("com.ibm.portal.state");

ibm.portal.portlet._SafeToExecute = false;

if ( window.addEventListener ) {
    window.addEventListener( "load", function () { ibm.portal.portlet._SafeToExecute = true; }, false );
}
else if ( window.attachEvent ) {
    window.attachEvent( "onload", function () { ibm.portal.portlet._SafeToExecute = true; } );
}

dojo.declare("ibm.portal.portlet.PortletWindow", null, {

    STATUS_UNDEFINED: 0,
    STATUS_OK: 1,
    STATUS_ERROR: 2,
    
    constructor: function (/*String*/ windowid) {
        /**
         * summary:
        *     Constructor
        *
        * windowid: The window id
        */

        if (windowid==null) return; // special constructor for static initializers

        this.windowID = windowid;
        var element = document.getElementById("com.ibm.wps.web2.portlet.preferences."+this.windowID);
        this.preferenceEditID = element.getAttribute("editid");
        this.preferenceConfigID = element.getAttribute("configid");
        this.preferenceEditDefaultsID = element.getAttribute("editdefaultsid");
        this.pageID = element.getAttribute("pageid");

        this.attributes = new Array();
        this._queuedFuncs = new Array();
        this.portletState = new ibm.portal.portlet.PortletState(windowid);
        
        this.isCSA = false; 
        try {
            this.isCSA = (typeof(document.isCSA) != "undefined" );
        }
        catch (e) {}
        
        //Certain functions aren't safe to be executed until the page has loaded. This will execute any functions
        //which have been queued at the page load event.
        var me = this;
        function executeQueued() {
            for ( var i = 0; i < me._queuedFuncs.length; i++ ) {
                me._queuedFuncs[i]();
            }   
        }
        
        if ( window.addEventListener ) {
            window.addEventListener( "load", function () { if ( !ibm.portal.portlet._SafeToExecute) { ibm.portal.portlet._SafeToExecute = true; } executeQueued(); }, false );
        }
        else if ( window.attachEvent ) {
            window.attachEvent( "onload", function () { if ( !ibm.portal.portlet._SafeToExecute) { ibm.portal.portlet._SafeToExecute = true; } executeQueued(); } );
        }
    },
    
    reportError: function(/*ibm.portal.portlet.Error*/ error) {
    /**
     * summary:
     *     Reports an error to the framework for display purposes
     */
        var code;
        if (error.getErrorCode()==ibm.portal.portlet.Error.ERROR) code = "error";
        else if (error.getErrorCode()==ibm.portal.portlet.Error.INFO) code = "info";
        else if (error.getErrorCode()==ibm.portal.portlet.Error.WARN) code = "warning";
        var statusMessage = { "_type" : code,
                              "_message" : error.getMessage(), 
                              "_details" : error.getDescription() };
        if ( this.isCSA )
        {
            dojo.publish("/portal/status",[{message: statusMessage}]);
        }
        else {
            if ( typeof( console ) != "undefined") {
                if (error.getErrorCode()==ibm.portal.portlet.Error.ERROR) 
                    console.error(statusMessage._message+"\n"+statusMessage._details);
                else if (error.getErrorCode()==ibm.portal.portlet.Error.INFO) 
                    console.info(statusMessage._message+"\n"+statusMessage._details);
                else if (error.getErrorCode()==ibm.portal.portlet.Error.WARN) 
                    console.warn(statusMessage._message+"\n"+statusMessage._details);
            }
            else {
                alert(statusMessage._type.toUpperCase()+"\nMessage: "+statusMessage._message+"\nDetails: "+statusMessage._details);
            }
        }
    },
    getAttribute: function(/*String*/ name) {
    /**
     * summary:
     *     Gets the attribute for the given name scoped by the portlet window
     */
        return this.attributes[name]; // Object, maybe null
    },

    setAttribute: function(/*String*/ name, /*Object*/ value) {
    /**
     * summary:
     *     Sets the attribute for the given name scoped by the portlet window
     */
         var ret = this.attributes[name];
         this.attributes[name] = value;
         return ret; // Object, maybe null
    },

    removeAttribute: function(/*String*/ name) {
    /**
     * summary:
     *     Removes the attribute for the given name scoped by the portlet window
     *     
     */
        this.attributes[name] = null;
    },

    clearAttributes: function() {
    /**
     * summary:
     *     Removes all attributes scoped by the portlet window
     *     
     */
         this.attributes = new Array();
    },

    getPortletState: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the portlet state bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet state.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
        var returnObject = this.portletState;
        var pointer2this = this;
        var result = null;
        if (callbackfn != null) {
            // invoke the callback function
            callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_OK, returnObject);
        } else {
            result = { "portletWindow" : pointer2this, 
                       "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                       "returnObject" : returnObject };
        }
        return result;
    },

    setPortletState: function(/*PortletState*/state, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the portlet state bean (holding the navigational state) scoped
     *     by the window id. The behavior of this function varies depending on
     *     whether or not the surrounding theme is capable of dealing with
     *     client-side state changes. If not, this function will trigger a full
     *     page refresh. Otherwise the provided callback function decides how to
     *     react on the state change.
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes storing the portlet state.
     *     The callback function defines how to update the portlet's markup
     *     to reflect the new navigational state. If no callback function is
     *     provided (or null is passed in) the method executes synchronously,
     *     and in addition refreshes the portlet (default behavior of reacting
     *     to a navigational state change.
     */
        this.portletState = state;
        
        if ( this.isCSA ) {
            // if no callback function is provided, execute the default behavior:
            // refresh the portlet by sending a render request to the server
            if (callbackfn == null) {
                var navState = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                var url = navState.newPortletRenderURL( this.windowID );
                var service = new com.ibm.portal.services.PortletFragmentService();
                service.getPortletInfo( "lm:oid:" + this.windowID + "@oid:"+ this.pageID, url );
            }
        } else {
            var stateManager = new com.ibm.portal.state.StateManager(ibmPortalConfig["contentHandlerURI"]);
            stateManager.reset( state.portletAccessor.stateDOM );
            // serialize the state into a portal URL
            var serializationMgr = stateManager.getSerializationManager();
            var serializationResult = serializationMgr.serialize(stateManager.getState());
            
            // return the URL
            var serializedState = serializationResult["returnObject"];
            
            var url = serializedState;
            window.location.href = url;
        }
        // dispatch
        return this.getPortletState(callbackfn);
    },
    
    _queueUp: function ( /*Function*/funcToQ ) {
        // summary: Certain functions are not safe to execute before the page loads. Any function
        //      passed to this function will be queued up and executed only after the page loads.
        // funcToQ: the function to queue
        this._queuedFuncs.push( funcToQ );
    },
    
    _throwInappropriateRequestError: function ( /*String*/funcName ) {
        throw new Error( "Cannot execute a synchronous call before the page loads! Please use an onload handler to execute this call to \"" + funcName + "\"." );
        return null;
    },
    
    getPortletPreferences: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the portlet preferences bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
        if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.getPortletPreferences( callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "getPortletPreferences" );
            }
        }
        
        var portletMode = this.getPortletState().returnObject.getPortletMode();
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var tempurl = document.getElementById("com.ibm.wps.web2.portlet.root."+this.windowID).innerHTML;
         var idx = tempurl.indexOf("--portletwindowid--");
         var _url = tempurl.replace(/--portletwindowid--/g, this.windowID);
         
         if ( _url.indexOf( "?" ) < 0 ) {
            _url = _url + "?"; 
         } 
         
         _url += "&verb=download&levels=-all&rep=compact&preferences=aggregated";

         this.requestedPreferenceID = "pm:oid:"+this.preferenceEditID; //+"@oid:"+this.pageID;
         if (portletMode==ibm.portal.portlet.PortletMode.CONFIG) 
             this.requestedPreferenceID = "pm:oid:"+this.preferenceConfigID;
         else if (portletMode==ibm.portal.portlet.PortletMode.EDIT_DEFAULTS) 
             this.requestedPreferenceID = "pm:oid:"+this.preferenceEditDefaultsID;
         
         var pointer2this = this;
         var pointer2return = null;
         
         dojo.xhrGet({
             url: _url,
             handleAs: "xml",
             headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
             sync: (callbackfn)?false:true,
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if(type == "load") {
                      var responseXML = response;
                      
                      //IE doesn't populate the responseXML property here because of the content-type of the response. So we need to create the XML
                      //document from text to be able to execute XPath queries.
                      if ( !responseXML || ( typeof( dojox.data.dom.innerXML( response )) == "undefined" ) ) {
                        responseXML = dojox.data.dom.createDocument( ioArgs.xhr.responseText );
                      }
                      
                      var returnObject = 
                          new ibm.portal.portlet.PortletPreferences(pointer2this.windowID,
                                                                        pointer2this.requestedPreferenceID,
                                                                        responseXML );
                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,returnObject);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : returnObject };
                      }
                  }else if(type == "error"){
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    setPortletPreferences: function(/*PortletPreferences*/preferences, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the portlet preferences bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes storing the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.setPortletPreferences( preferences, callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "setPortletPreferences" );
            }
        }
         
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var tempurl = document.getElementById("com.ibm.wps.web2.portlet.root."+this.windowID).innerHTML;
         var idx = tempurl.indexOf("--portletwindowid--");
         var _url = tempurl.replace(/--portletwindowid--/g, this.windowID);
         if ( _url.indexOf( "?" ) < 0 ) {
             _url += "?verb=download";
         }
         else {
             _url += "&verb=download";
         }

         var requestedPreferenceID = preferences.requestedPreferenceID;
         // strip down to necessary data
         var expr = "/atom:feed/atom:entry[atom:id='"+requestedPreferenceID+"']";
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, preferences.xmlData, preferences.ns);
         var entry;
         if (nodes && nodes.length>0) {
             entry = nodes[0];
         }
         else {
             return null; // TODO
         }
         var parentNode = entry.parentNode;
         expr = "/atom:feed/atom:entry";
         nodes = ibm.portal.xml.xpath.evaluateXPath(expr, preferences.xmlData, preferences.ns);
         for (var i=0; i<nodes.length; i++) {
             var node = nodes[i];
             if (node!=entry) {
                 parentNode.removeChild(node);
             }
         }
         
         //alert (dojox.data.dom.innerXML(preferences.xmlData));

         var pointer2this = this;
         var pointer2return = null;


         dojo.rawXhrPut({
             url: _url,
             sync: (callbackfn)?false:true,
             putData: dojox.data.dom.innerXML(preferences.xmlData),
             contentType: "application/xml",
             handleAs: "xml",
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if(type == "load") {
                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,preferences);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : preferences };
                      }
                  }else if(type == "error") {
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    getUserProfile: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the current user profile
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.getUserProfile( callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "getUserProfile" );
            }
        }
         
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var _url = document.getElementById("com.ibm.wps.web2.portlet.user."+this.windowID).innerHTML;

         var pointer2this = this;
         var pointer2return = null;
         dojo.xhrGet({
             url: _url,
             headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
             sync: (callbackfn)?false:true,
             handleAs: "xml",
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if (type == "load") {
                      var responseXML = response;
                      
                      // IE doesn't populate the responseXML property here because of the content-type of the response.
                      // We need to create the XML document from text to be able to execute XPath queries.
                      if ( !responseXML || ( typeof( dojox.data.dom.innerXML( response )) == "undefined" ) ) {
                          responseXML = dojox.data.dom.createDocument( ioArgs.xhr.responseText );
                      }

                      var returnObject = 
                          new ibm.portal.portlet.UserProfile(pointer2this.windowID, responseXML );

                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,returnObject);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : returnObject };
                      }
                  } else if(type == "error"){
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    setUserProfile: function(/*userProfile*/userProfile, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the current user profile
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         //var serializer = new XMLSerializer();
         //resultStr = serializer.serializeToString(userProfile.xmlData);
         //alert(resultStr);
        if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.setUserProfile( userProfile, callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "setUserProfile" );
            }
        }

         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var _url = document.getElementById("com.ibm.wps.web2.portlet.user."+this.windowID).innerHTML;

         var pointer2this = this;
         var pointer2return = null;
         dojo.rawXhrPost({
             url: _url,
             sync: (callbackfn)?false:true,
             postData: dojox.data.dom.innerXML(userProfile.xmlData),
             contentType: "application/xml",
             handleAs: "xml",
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if(type == "load") {
                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,userProfile);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : userProfile };
                      }
                  } else if(type == "error"){
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },
    newXMLPortletRequest: function() {
        // summary: Creates a new XMLPortletRequest object with the current page ID and portlet window ID.
        // returns: a new XMLPortletRequest
        return new ibm.portal.portlet.XMLPortletRequest( this.pageID, this.windowID ); // ibm.portal.portlet.XMLPortletRequest
    }
});


/**
 * summary:
 * The <CODE>PortletPreferences</CODE> interface allows the portlet to store
 * configuration data. It is not the
 * purpose of this interface to replace general purpose databases.
 * 
 * description:
 * There are two different types of preferences:
 * <ul>
 * <li>modifiable preferences - these preferences can be changed by the
 *     portlet in any standard portlet mode (<code>EDIT, HELP, VIEW</code>).
 *     Per default every preference is modifiable.
 * <li>read-only preferences - these preferences cannot be changed by the
 *     portlet in any standard portlet mode, but may be changed by administrative modes.
 *     Preferences are read-only, if the are defined in the
 *     deployment descriptor with <code>read-only</code> set to <code>true</code>,
 *     or if the portlet container restricts write access.
 * </ul>
 * <p>
 * Changes are persisted when the <code>store</code> method is called.
 */

dojo.declare("ibm.portal.portlet.PortletPreferences", null, {
    constructor: function (/*String*/ windowid, 
                           /*String*/ requestedpreferenceid,
                           /*XMLDocument */ data) {
        /** 
        * summary:
        *     Objects of this type should never be called directly but always created
        *     through the PortletManagers method getPortletPreferences.
        *
        * windowid: The window id
        */

        this.windowID = windowid;
        this.requestedPreferenceID = requestedpreferenceid;
        this.xmlData = data;
        this.xsltURL = dojo.moduleUrl( "ibm", "portal/portlet/");
        
        this.ns = {
            "xsl" : "http://www.w3.org/1999/XSL/Transform",
            "thr" : "http://purl.org/syndication/thread/1.0",
            "atom" : "http://www.w3.org/2005/Atom",
            "xhtml" : "http://www.w3.org/1999/xhtml",
            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
            "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
        };

        // performance optimization
        this.internal_reset();
    },

    getMap: function (){
    /**
     * summary: 
     *     Returns a <code>Map</code> of the preferences.
     *     If no preferences exist this method returns an empty <code>Map</code>.
     */
        //ibm.portal.debug.entry( "PortletPreferences.internal_applyXSLT" , [ feed.toString(), dom, args ] );

        if (this.result_getMap) {
            return this.result_getMap;
        }

        //load the xslt
        var xslDoc = ibm.portal.xml.xslt.loadXsl(this.xsltURL + "PortletPreferencesMap.xsl");

        //apply the xslt
        if (xslDoc.documentElement == null) {alert("xslDoc is null");}
        var results = ibm.portal.xml.xslt.transform(this.xmlData, xslDoc, null, {"selectionid":this.requestedPreferenceID}, true);
        if (results == null) {
            this.result_getNames = null;
            return null;
        }

        var prefs = eval( results );
        // "portletPreferences = { \"preferences\": [ { \"name\":\"parallel\" , \"values\": [ \"false\" ] , \"read-only\": \"false\" }, { \"name\":\"test\" , \"values\": [ \"newValue\" ] , \"read-only\": \"false\" } ] };"

        if (prefs)            
            prefs = prefs.preferences;

        this.result_getMap = prefs;

        return this.result_getMap; /* object[] {name: string, values: string[], readonly: boolean}
     *             a <code>Map</code> containing preference names as 
     *             keys and preference values as map values, or an empty <code>Map</code>
     *             if no preference exist.
     */
    },

    getNames: function (){
    /**
     * summary:
     *     Returns all of the keys that have an associated value,
     *     or an empty <code>Enumeration</code> if no keys are available.
     */
         //ibm.portal.debug.entry( "PortletPreferences.internal_applyXSLT" , [ feed.toString(), dom, args ] );
         
         if (this.result_getNames) {
             return this.result_getNames;
         }
         
         //load the xslt
         var xslDoc = ibm.portal.xml.xslt.loadXsl(this.xsltURL + "PortletPreferencesNames.xsl");

         //apply the xslt
         if (xslDoc.documentElement == null) {alert("xslDoc is null");}
         var results = ibm.portal.xml.xslt.transform(this.xmlData, xslDoc, null, {"selectionid":this.requestedPreferenceID}, true);
         if (results == null) {
             this.result_getNames = null;
             return null;
         }

         var prefsNames = eval( results );
         // "portletPreferences = { \"preferences\": [ { \"name\":\"parallel\" , \"values\": [ \"false\" ] , \"read-only\": \"false\" }, { \"name\":\"test\" , \"values\": [ \"newValue\" ] , \"read-only\": \"false\" } ] };"

         if (prefsNames)
             prefsNames = prefsNames.names;

         this.result_getNames = prefsNames;

         return this.result_getNames; /* String[] - an Enumeration of the keys that have an associated value,
         *         or an empty <code>Enumeration</code> if no keys are
        *         available.
        */
    },

    getValue: function( /*String*/ key, /*String*/ def ){
    /**
     * summary:
     *     Returns the first String value associated with the specified key of this preference.
     * description:
     *     If there is one or more preference values associated with the given key 
     *     it returns the first associated value.
     *     If there are no preference values associated with the given key, or the 
     *     backing preference database is unavailable, it returns the given 
     *     default value.
     *
     * key: String - key for which the associated value is to be returned
     * def: String - the value to be returned in the event that there is no 
     *            value available associated with this <code>key</code>.
     */
         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']/base:value";
         // /atom:feed/atom:entry[atom:id='pm:oid:5_J12SV3D00O6JD02ECR217F00E0@oid:6_J12SV3D00O6JD02ECR217F00M6']/atom:content/model:portlet-entity/model:portletpreferences[@name='parallel']
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         var result;
         if (nodes && nodes.length>0) {
             result = nodes[0].getAttribute("value");
         }
         else {
             result = def;
         }

         return result; /* String - the value associated with <code>key</code>, or <code>def</code>
         *         if no value is associated with <code>key</code>, or the backing
         *         store is inaccessible.
         */
    },

    getValues: function( /*String*/ key, /*String[]*/ def ){
    /**
     * summary:
     *     Returns the String array value associated with the specified key in this preference.
     * description:
     *     Returns the specified default if there is no value
     *     associated with the key, or if the backing store is inaccessible.
     *
     *     If the implementation supports <i>stored defaults</i> and such a
     *     default exists and is accessible, it is used in favor of the
     *     specified default.
     *
     * key: String - key for which associated value is to be returned.
     * def: String[] - the value to be returned in the event that this
     *        preference node has no value associated with <code>key</code>
     *        or the associated value cannot be interpreted as a String array,
     *        or the backing store is inaccessible.
     */

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']/base:value";
         // /atom:feed/atom:entry[atom:id='pm:oid:5_J12SV3D00O6JD02ECR217F00E0@oid:6_J12SV3D00O6JD02ECR217F00M6']/atom:content/model:portlet-entity/model:portletpreferences[@name='parallel']
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);
         var result;
         if (nodes && nodes.length>0) {
             result = new Array();
             for (var i=0; i<nodes.length; i++) {
                 result[i] = nodes[i].getAttribute("value");
             }
         }
         else {
             result = def;
         }
        
         return result; /* String[] - the String array value associated with
         *         <code>key</code>, or <code>def</code> if the
         *         associated value does not exist.
         */
    },

    isReadOnly: function( /*String*/ key ){
    /**
     * summary:
     *     Returns true, if the value of this key cannot be modified by the user.
     * description:
     *     Modifiable preferences can be changed by the
     *     portlet in any standard portlet mode (<code>EDIT, HELP, VIEW</code>). 
     *     Per default every preference is modifiable.
     * 
     *     Read-only preferences cannot be changed by the
     *     portlet in any standard portlet mode, but inside of custom modes
     *     it may be allowed changing them.
     *     Preferences are read-only, if they are defined in the 
     *     deployment descriptor with <code>read-only</code> set to <code>true</code>,
     *     or if the portlet container restricts write access.
     *
     * key: String - key for which the associated value is to be returned
     */

         var id = this.requestedPreferenceID;
         var expr = "/atom:feed/atom:entry[atom:id='"+id+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         
         var result = false;
         if (nodes && nodes.length>0) {
             var temp = nodes[0].getAttribute("read-only");
             if (temp != null) {
                 // convert to a boolean
                 if ( temp == "true" ) {
                    result = true;
                 }
             }
         }
         return result; // boolean - false, if the value of this key can be changed, or if the key is not known
    },

    reset: function ( /*String*/ key ){
    /**
     * summary:
     *     Resets or removes the value associated with the specified key.
     * description:
     *     If this implementation supports stored defaults, and there is such
     *     a default for the specified preference, the given key will be 
     *     reset to the stored default.
     *
     *     If there is no default available the key will be removed.
     *
     * key: String - key to reset
     */
         this.internal_reset();

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);
         if (nodes && nodes.length>0) {
             var parent = nodes[0].parentNode;
             parent.removeChild(nodes[0]);
         }

    },

    setValue: function ( /*String*/ key, /*String*/ value ){
    /**
     * summary:
     *     Associates the specified String value with the specified key in this preference.
     * description:
     *     The key cannot be <code>null</code>, but <code>null</code> values
     *     for the value parameter are allowed.
     *
     * key: String - key with which the specified value is to be associated.
     * value: String - value to be associated with the specified key.
     */
         var values = new Array();
         values[0] = value;
         this.setValues(key,values);
    },

    setValues: function( /*String*/ key, /*String[]*/ values ){
    /**
     * summary:
     *     Associates the specified String array value with the specified key in this preference.
     * description:
     *     The key cannot be <code>null</code>, but <code>null</code> values
     *     in the values parameter are allowed.
     *
     * key: String - key with which the  value is to be associated
     * values: String[] - values to be associated with key
     */
         this.internal_reset();

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);

         var prefNode = null;
         if (nodes && nodes.length>0) {
             prefNode = nodes[0];
             for (var i=prefNode.childNodes.length-1; i>=0; i--) {
                 prefNode.removeChild(prefNode.childNodes[i]);
             }
         }
         else {
             var expr2 = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*";
             var nodes2 = ibm.portal.xml.xpath.evaluateXPath(expr2, this.xmlData, this.ns);
             if (dojo.isIE) {
                 prefNode = this.xmlData.createNode(1, "model:portletpreferences", this.ns.model); // 1 = NODE_ELEMENT
             }
             else {
                 prefNode = this.xmlData.createElementNS(this.ns.model,"model:portletpreferences");
             }
             prefNode.setAttribute("name",key);
             prefNode.setAttribute("read-only","false");
             nodes2[0].appendChild(prefNode);
         }
         for (var i=0; i<values.length; i++) {
             var newNode;
             if (dojo.isIE) {
                 newNode = this.xmlData.createNode(1, "base:value", this.ns.base); // 1 = NODE_ELEMENT
                 var attribute = this.xmlData.createNode(2, "xsi:type",this.ns.xsi ); // 1 = NODE_ATTRIBUTE
                 attribute.nodeValue = "String";
                 newNode.setAttributeNode(attribute);
             }
             else {
                 newNode = this.xmlData.createElementNS(this.ns.base,"base:value");
                 newNode.setAttributeNS(this.ns.xsi,"xsi:type","String");
             }
             newNode.setAttribute("value",values[i]);
             prefNode.appendChild(newNode);
         }
    },

    internal_reset: function() {
         this.result_getMap = null;
         this.result_getNames = null;
    },

    clone:function()
    {
        var resultStr = dojox.data.dom.innerXML(this.xmlData);
        var xmlDataCopy = dojox.data.dom.createDocument( resultStr ); 

        return new ibm.portal.portlet.PortletPreferences(this.windowID, 
                                                             this.requestedPreferenceID,
                                                             xmlDataCopy);
    }


});















dojo.declare("ibm.portal.portlet.PortletMode", null, {
    VIEW: "view",
    EDIT: "edit",
    EDIT_DEFAULTS: "edit_defaults",
    HELP: "help",
    CONFIG: "config"    
});

dojo.declare("ibm.portal.portlet.WindowState", null, {
    NORMAL: "normal",
    MINIMIZED: "minimized",
    MAXIMIZED: "maximized"
});

dojo.declare("ibm.portal.portlet.PortletState", null, {
    constructor: function (/*String or XMLDocument*/ windowid, /*String?*/portletWindowId ) {
        /**
         * summary:
         *     Objects of this type should never be called directly but always created
         *     through the PortletManagers method getPortletState.
         *
         * windowid: The window id
         */
        var stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
        if ( dojo.isString( windowid ) ) {
            var stateDOM = this._getExistingState( windowid, stateManager.getSerializationManager() );
            stateManager.reset( stateDOM );
        }
        else {
            stateManager.reset( windowid );
            windowid = portletWindowId; 
        }
        
        // get a state accessor that allows for reading and writing portlet state
        this.portletAccessor = stateManager.newPortletAccessor(windowid);
        // also get the Parameters object for performance reasons
        this.renderParameters = this.portletAccessor.getRenderParameters();
    },
    _isCSA: function () {
        var retVal = false;
        try {
            retVal = (typeof(document.isCSA )!= "undefined" );
        }
        catch (e) {
            //do nothing here... something was not defined so we aren't in CSA
        }
        
        return retVal;
    },
    _getExistingState: function ( /*String*/windowid, serializationMgr ) {
        var retVal = null;
        if ( this._isCSA() ) {
            // CSA has access to the current state
            retVal = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState().stateDOM;
        } else if (serializationMgr != null) {
            // get the current state from the server
            var result = serializationMgr.deserialize( location.href );
            retVal = result.returnObject;
        } else {
            // fall back to an "empty" state
            retVal = dojox.data.dom.createDocument();
        }
        return retVal;
    },
    getPortletMode:function() {
    /**
     * summary:
     *     Returns the portlet mode.
     */
        return this.portletAccessor.getPortletMode(); /*ibm.portal.portlet.PortletMode*/
    },

    setPortletMode:function(/*ibm.portal.portlet.PortletMode*/ portletMode) {
    /**
     * summary:
     *     Sets the portlet mode.
     */
        this.portletAccessor.setPortletMode(portletMode);
        return portletMode;
    },

    getWindowState:function() {
    /**
     * summary:
     *     Returns the window state.
     */
        return this.portletAccessor.getWindowState(); /*ibm.portal.portlet.WindowState*/
    },
    
    setWindowState:function(/*ibm.portal.portlet.WindowState*/windowState) {
    /**
     * summary:
     *     Sets the window state.
     */
        this.portletAccessor.setWindowState(windowState);
        return windowState;
    },

    getParameterNames: function() {
    /**
     * summary:
     *     Returns a String[] containing the names of all parameters that have an
     *     associated value. Returns an empty array if no parameters are available.
     */
        return this.renderParameters.getNames(); // String[]
    },

    getParameterValue: function(/*String*/ name) {
    /**
     * summary:
     *     Returns the String value associated with the render parameter with the given name.
     *     
     * description:
     *     If there are one or more render parameter values associated with the given name 
     *     it returns the first associated value.
     *
     * name: String - Name of the render parameter for which the associated value is to
     *     be returned. Must not be NULL.
     */
        return this.renderParameters.getValue(name); // String
    },

    getParameterValues: function(/*String*/ name) {
    /**
     * summary:
     *     Returns the String[] representing the values that are associated with
     *     the render parameter with the given name.
     *
     * name: String - Name of the render parameter for which the associated values
     *     are to be returned. Must not be NULL.
     */
        return this.renderParameters.getValues(name); // String[]
    },

    getParameterMap: function() {
    /**
     * summary:
     *     Returns an unmodfiable map of render parameters.
     */
        return this.renderParameters.getMap(); /* object[] {name: string, values: string[]} */
    },

    setParameterValue: function(/*String*/ name, /*String*/ value) {
    /**
     * summary:
     *     Sets a simple String value for the render parameter with the given name. Overrides
     *     the value of the render parameter if it already exists.
     *
     * name: String - The name of the render parameter. Must not be NULL.
     * value: String - The value of the render parameter that should be set. Must not be NULL. 
     */
        this.renderParameters.setValue(name, value);
        return value; // String
    },
    
    setParameterValues: function(/*String*/ name, /*String[]*/ values) {
    /**
     * summary:
     *     Sets a String[] representing the value of the render parameter with the given name.
     *     Overrides the value of the render parameter if it already exists.
     *
     * name: String - The name of the render parameter. Must not be NULL.
     * value: String[] - The value of the render parameter that should be set. Must not be NULL. 
     */
        this.renderParameters.setValues(name, values);
        return values; // String[]
    },
    
    setParameterMap:function(/*object[] {name: string, values: string[]}*/map, /*boolean*/replace) {
    /**
     * summary:
     *     Adds all render parameters that are contained in the given map. Overrides the values
     *     of parameters that already exist. Returns the resulting map.
     *
     * map: Object[] {name: String, values: String[]} - Two dimensional object array
     *     representing the map of parameters that should be added. Must not be NULL.
     * replace: boolean - Optional argument that indicates whether the given map should
     *     completely replace the existing parameter map.
     */
        // check if we need to delete the existing parameters
        if (replace == true) {
            this.renderParameters.clear();
        }
        this.renderParameters.putAll(map);
        return this.renderParameters.getMap(); /* object[] {name: string, values: string[]} */
    },
    
    removeParameter:function(/*String*/name) {
    /**
     * summary:
     *     Removes the render parameter with the given name.
     *
     * name: String - Name of the render parameter that should be removed. Must not be NULL.
     */
        this.renderParameters.remove(name);
    }
    
});







dojo.require( "com.ibm.portal.services.PortletFragmentService" );

dojo.declare("ibm.portal.portlet.XMLPortletRequest", null, {
     // summary: Provides XMLHttpRequest functionality for addressing portlets.
     // description: Wrappers and exposes all standard properties on an XMLHttpRequest, as well
     //         as, exposing an additional property called onportletstateready. This new property
     //         can be assigned a function that will be called when the portlet state is ready. If
     //         a handler is set for onportletstateready, the portlet state will not be updated 
     //         globally. The XMLPortletRequest will only work for URLs which point to the portlet
     //         fragment feed.
     // 
     // onportletstateready: Function - handler for intercepting global portlet state updates, the 
     //         PortletState object is the lone parameter passed into this function when called.
     // onreadystatechange: Function - handler for the readyState property changing
     // readyState: int - indicates the current state of the request
     // responseText: String - the text of the response body (only available when readyState == 4)
     // responseXML: XMLDocument - the xml document in the response body
     // status: int - the HTTP status code received in the response
     // statusText: String - the String corresponding to the HTTP status code
     // 
    onreadystatechange: null,     /*Function*/
    readyState: 0,            /* int */
    responseText: null,           /* String */
    responseXML: null,            /* Document */
    status: null,                 /* int */
    statusText: null,             /* String */
    onportletstateready: null,     /* Function(ibm.portal.portlet.PortletState) */
    _location: null,              /* String - stores the location of the request */
    
    constructor: function ( /*String*/page, /*String*/window ) {
        // summary: Constructor for the XMLPortletRequest object. 
        // description: A correctly initialized instance of this object can be obtained from the portlet window object.  The only
        //      case where this constructor should be called directly is when sending an XPR to a different portlet than the
        //      one represented by the portletWindow object. 
        // page: the page's ObjectID
        // window: the porltet window's ObjectID
        this.pageID = page;
        this.windowID = window;
    },
    _getXHR: function () {
         // summary: Creates the XMLHttpRequest object if necessary and returns it
         // returns: the XMLHttpRequest instance associated with this XMLPortletRequest
        if ( !this._xhr ) { 
            this._xhr = this._createXHR();
        }
        
        return this._xhr; //XMLHttpRequest
    },
    _createXHR: function () {
        // summary: Cross-browser creation of XMLHttpRequest or it's equivalent.
        // returns: a new instance of an XMLHttpRequest or it's equivalent
        var retVal = null;
        
        if ( typeof( XMLHttpRequest ) != "undefined" ) {
            retVal = new XMLHttpRequest();  
        }
        else {
            retVal = new ActiveXObject( "Microsoft.XMLHTTP" );
        }
        
        return retVal; //XMLHttpRequest
    },
    _onreadystatechangehandler: function () {
        // summary: Wrappers any user-set onreadystatechange handlers and processes the 
        //      response from the Portlet Fragment feed.
        // description: Intercepts the underlying XMLHttpRequest readyState changes and
        //      extracts the necessary information from the Portlet Fragment feed to 
        //      make the XMLPortletRequest object ready for consumption by the developer.
        var xhr = this._getXHR();
        
        //Copy over common properties that may have changed.
        this.readyState = xhr.readyState;
        
        //Extract the response portlet state and any dependent portlets
        if ( this.readyState == 4 ) {
            //Can't copy these common properties over until readyState is 4 because of IE
            this.responseText = xhr.responseText;
            this.responseXML = xhr.responseXML;
            this.status = xhr.status;
            this.statusText = xhr.statusText;
            //Use our service object for convenience here. We're just using to parse the
            //ATOM Feed response, not for actually retrieving any data because we need
            //direct access to the XMLHttpRequest object in this use case.
            var service = new com.ibm.portal.services.PortletFragmentService();
            this.responseText = service.readMarkup( xhr.responseXML );
            //Will need to set this based on the atom:content type attribute, but since we
            //aren't supporting any other markup types but HTML out of the box, this should
            //never be set.
            this.responseXML = null;
            
            this._handleDependentPortlets( service.readDependentPortlets( xhr.responseXML ) );
            var doGlobalStateUpdate = true;
            
            if ( this.onportletstateready != null ) {
                var xmlState = service.readPortletState( xhr.responseXML );
                var service = new com.ibm.portal.services.PortletFragmentService();  
                var windowId = service.readWindowID( xhr.responseXML );
                var portletState = new ibm.portal.portlet.PortletState( xmlState, windowId );
                
                //allow developer to handle portlet state
                doGlobalStateUpdate = this.onportletstateready( portletState );
            }
            
            if ( doGlobalStateUpdate && this._isCSA() )
            {
                //update global state
                var portletInfo = service.createPortletInfo( xhr.responseXML );
                service._fireGlobalPortletStateChange( portletInfo );
            }   
            
        }
        
        //Finally call the user's onreadystate change handler.
        if ( this.onreadystatechange != null ) {
            this.onreadystatechange();
        }       
    },
    _handleDependentPortlets: function ( /*Array*/portlets ) {
        // summary: Trigger reloads for all dependent portlets.
        // description: Reloads any portlets whose content has changed as a result of
        //      this XMLPortletRequest. This generally affects C2A portlets. If we are 
        //      not in Client-Side Aggregation, this will trigger a full page refresh.
        if ( this._isCSA() ) {
            var portletService = new com.ibm.portal.services.PortletFragmentService();
            for ( var i = 0; i < portlets.length; i++ ) {
                var portletid = portlets[i].uri;
                portletid = portletid.replace( /fragment:pm:/g, "lm:" );
                com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.getFragment( portletid ).setLoading();
                portletService.getPortletInfo( portletid, portlets[i].link );
            }   
        }
        else {
            if ( portlets.length > 0 ) {
                //Until we have Differential Page Rendering on the server side, we have no 
                //choice but to do a full page refresh, but we must point to the page
                //so the action isn't submitted twice and all portlets are rendered with fresh
                //parameters 
                
                window.location.href = this._newPageURL();
            }   
        }
    },
    _isCSA: function () {
        var retVal = false;
        try {
            retVal = (typeof(document.isCSA )!= "undefined" );
        }
        catch (e) {
            //do nothing here... something was not defined so we aren't in CSA
        }
        
        return retVal;
    },
    _flag: function ( /*String*/ uriToFlag ) {
        // summary: Flags a URL with the appropriate url parameters to retrieve a portlet fragment.
        var id = "lm:oid:" + this.windowID + "@oid:" + this.pageID;
        var service = new com.ibm.portal.services.PortletFragmentService();
        return service._flagPortletUrl( uriToFlag, id );
    },
    _newPageURL: function ( ) {
        // summary: Creates a new page url based on the portlet window id and page id.
        // description: Creates a new page url to the page containing the portlet that 
        // originated this XMLPortletRequest object.
        
        // get a copy of the state which contains the state of our portlet only    
        var stateManager = new com.ibm.portal.state.StateManager(ibmPortalConfig["contentHandlerURI"]);
        var stateDOM = dojox.data.dom.createDocument();
        stateManager.reset( stateDOM );
            
        var portletState = stateManager.newPortletAccessor( this.windowID ).getPortletState();
        
        var selectionAcc = stateManager.newSelectionAccessor(portletState);
        selectionAcc.setPageSelection(this.pageID);
        
        // serialize the state into a portal URL
        var serializationMgr = stateManager.getSerializationManager();
        var serializationResult = serializationMgr.serialize(portletState);
        
        // return the URL
        var serializedState = serializationResult["returnObject"];
        
        var url = serializedState; 
        
        return url; // String
    },
    open: function( /*String*/ method, /*String*/ uri ) {
        // @see XMLHttpRequest
        this.open( method, uri, false );  
    },
    open: function( /*String*/ method, /*String*/ uri,  /*boolean*/ async ) {
        // @see XMLHttpRequest
        var xhr = this._getXHR();
        var me = this;
        this._location = uri;
        xhr.onreadystatechange = function () { me._onreadystatechangehandler(); };
        xhr.open( method, this._flag(uri), async );
    },
    setRequestHeader: function( /*String*/ header, /*String*/ value ) {
        // @see XMLHttpRequest
        this._getXHR().setRequestHeader( header, value );
    },
    send: function( /*String or Document*/ data ) {
        // @see XMLHttpRequest
        this._getXHR().send( data );   
    },
    abort: function( ) {
        // @see XMLHttpRequest
        this._getXHR().abort();
    },
    getAllResponseHeaders: function( ) {
        // @see XMLHttpRequest
        return this._getXHR().getAllResponseHeaders();
    },
    getResponseHeader: function( /*String*/ header ) {
        // @see XMLHttpRequest
        return this._getXHR().getResponseHeader( header );
    }

});

















dojo.declare("ibm.portal.portlet.UserProfile", null, {

    constructor: function (/*String*/ windowid,
                           /*XMLDocument */ data) {
    /**
     * summary:
     *     Objects of this type should never be called directly but always created
     *     through the PortletWindows method getUser.
     *
     * windowid: The window id
     */
        this.windowID = windowid;
        this.xmlData = data;

        this.ns = {
            "xsl" : "http://www.w3.org/1999/XSL/Transform",
            "atom" : "http://www.w3.org/2005/Atom",
            "xhtml" : "http://www.w3.org/1999/xhtml",
            "xsi" : "http://www.w3.org/2001/XMLSchema-instance",
            "um" : "http://www.ibm.com/xmlns/prod/websphere/um.xsd"
        };
    },

    getAttribute:function(/*String*/ name) {
    /**
     * summary:
     *     Returns the value of the given user attribute name as defined by the P3P Spec.
     * description:
     *     The portlet must define the user information attribute it is interested in
     *     inside the user-attribute section of the deployment descriptor. 
     *     If an attribute is not supported by the current runtime system or if it is supported
     *     but not defined for a particular user, then this method will return null
     * 
     * name: String - the name of the P3P attribute to return
     */
         var expr = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']/um:attributeValue";
         //var expr = "/um:profile[@type='user']/um:attribute[@name='"+name+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         

         var result = null;
         if (nodes && nodes.length>0) {
             if (nodes[0].textContent) {
                 result = nodes[0].textContent;
             }
             else {
                 result = nodes[0].text;
             }
         }

         return result; /* String */

    },

    setAttribute:function(/*String*/ name, /*String*/ value) {
         var expr = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']/um:attributeValue";
         //var expr = "/um:profile[@type='user']/um:attribute[@name='"+name+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         var previousText = null;
         if (nodes && nodes.length>0) {
             if (nodes[0].textContent) {
                 previousText = nodes[0].textContent;
                 nodes[0].textContent = value;
             }
             else {
                 previousText = nodes[0].text;
                 nodes[0].text = value;
             }
         }
         else {
             // TODO             
             var expr1 = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']";
             var nodes1 = ibm.portal.xml.xpath.evaluateXPath(expr1, this.xmlData, this.ns);
             var attrNode = null;
             if (nodes1 && nodes1.length>0) {
                 attrNode = nodes1[0];
             }
             else {
                 var expr2 = "/atom:entry/atom:content/um:profile[@type='user']";
                 var nodes2 = ibm.portal.xml.xpath.evaluateXPath(expr2, this.xmlData, this.ns);
                 
                 if (dojo.isIE) {
                     attrNode = this.xmlData.createNode(1, "um:attribute", this.ns.um); // 1 = NODE_ELEMENT
                 }
                 else {
                     attrNode = this.xmlData.createElementNS(this.ns.um, "um:attribute");
                 }
                 attrNode.setAttribute("type","xs:string");
                 attrNode.setAttribute("multiValued","false");
                 attrNode.setAttribute("name",name);
                 nodes2[0].appendChild(attrNode);
             }
             var newNode;
             if (dojo.isIE) {
                 newNode = this.xmlData.createNode(1, "um:attributeValue", this.ns.um); // 1 = NODE_ELEMENT
                 newNode.text = value;
             }
             else {
                 newNode = this.xmlData.createElementNS(this.ns.um, "um:attributeValue");
                 newNode.textContent = value;
             }
             attrNode.appendChild(newNode);
         }

         return previousText;
    },

    clone:function()
    {
        var resultStr = dojox.data.dom.innerXML(this.xmlData);
        var xmlDataCopy = dojox.data.dom.createDocument( resultStr ); 

        return new ibm.portal.portlet.UserProfile(this.windowID,
                                                      xmlDataCopy);
    }
    
});







dojo.declare("ibm.portal.portlet.Error", null, {

    INFO: 0,
    WARN: 1,
    ERROR: 2,
    
    constructor: function (/*String*/ errorCode, /*String*/ message, /*String*/ description) {
        this.errorCode = errorCode;
        this.message = message;
        this.description = description;
    },
    
    getErrorCode: function() {
    /**
     * summary:
     *     Returns the error code
     */
        return this.errorCode;
    },
    
    getMessage: function() {
    /**
     * summary:
     *     Returns the message
     */
        return this.message;
    },
    
    getDescription: function() {
    /**
     * summary:
     *     Returns the description
     */
        return this.description;
    }

});


{ // create static initializers
var com_ibm_portal_portlet_portletwindow = new ibm.portal.portlet.PortletWindow();
ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED = com_ibm_portal_portlet_portletwindow.STATUS_UNDEFINED;
ibm.portal.portlet.PortletWindow.STATUS_OK = com_ibm_portal_portlet_portletwindow.STATUS_OK;
ibm.portal.portlet.PortletWindow.STATUS_ERROR = com_ibm_portal_portlet_portletwindow.STATUS_ERROR;
com_ibm_portal_portlet_portletwindow = null;
var com_ibm_portal_portlet_portletmode = new ibm.portal.portlet.PortletMode();
ibm.portal.portlet.PortletMode.VIEW = com_ibm_portal_portlet_portletmode.VIEW;
ibm.portal.portlet.PortletMode.EDIT = com_ibm_portal_portlet_portletmode.EDIT;
ibm.portal.portlet.PortletMode.EDIT_DEFAULTS = com_ibm_portal_portlet_portletmode.EDIT_DEFAULTS;
ibm.portal.portlet.PortletMode.HELP = com_ibm_portal_portlet_portletmode.HELP;
ibm.portal.portlet.PortletMode.CONFIG = com_ibm_portal_portlet_portletmode.CONFIG;
com_ibm_portal_portlet_portletmode = null;
var com_ibm_portal_portlet_windowstate = new ibm.portal.portlet.WindowState();
ibm.portal.portlet.WindowState.NORMAL = com_ibm_portal_portlet_windowstate.NORMAL;
ibm.portal.portlet.WindowState.MINIMIZED = com_ibm_portal_portlet_windowstate.MINIMIZED;
ibm.portal.portlet.WindowState.MAXIMIZED = com_ibm_portal_portlet_windowstate.MAXIMIZED;
com_ibm_portal_portlet_windowstate = null;
var com_ibm_portal_portlet_error = new ibm.portal.portlet.Error();
ibm.portal.portlet.Error.INFO = com_ibm_portal_portlet_error.INFO;
ibm.portal.portlet.Error.WARN = com_ibm_portal_portlet_error.WARN;
ibm.portal.portlet.Error.ERROR = com_ibm_portal_portlet_error.ERROR;
com_ibm_portal_portlet_error = null;
}


}
