dojo.require("com.ibm.portal.xpath");
dojo.provide("ibm.portal.search.widgets.common");

dojo.declare(
	"ibm.portal.search.scope.XmlWriter", null,
	{
		//constants
		SPACE: " ",
		EQUALS: "=",
		QUOTES: "\"",
		SLASH: "/",
		START_TAG: "<",
		END_TAG: ">",
		SCOPE_ELEMENT: "scope",
		TITLE_ELEMENT: "title",
		DESCRIPTION_ELEMENT: "description",
		QUERY_TEXT_ELEMENT: "querytext",
		SCOPE_PROPERTY_ELEMENT: "scopeproperty",
		KEY_ATTRIBUTE: "key",
		VALUE_ATTRIBUTE: "value",
		ID_ATTRIBUTE: "id",
		SELECTED_SCOPE: "SELECTED_SCOPE",
		SCOPEELEMENT_ELEMENT: "scopeelement",
		LOCATION_ELEMENT: "location",
		FEATURE_ELEMENT: "feature",
		FEATURE_VALUE_ELEMENT: "feature_value",
		FEATURE_ID: "feature_id",
		SOURCE_ATTRIBUTE: "source",
		PATH_ATTRIBUTE: "path",
		
		writeAttribute: function(key, value, buffer) {
		    buffer.push(this.SPACE);
		    buffer.push(key);
		    buffer.push(this.EQUALS);
		    buffer.push(this.QUOTES);
		    buffer.push(value);
		    buffer.push(this.QUOTES);
		},
		startElement: function(elementName, attributes, attributeValues, close, buffer)  {
		    buffer.push(this.START_TAG);
		    buffer.push(elementName);
		    if (attributes !== null) {
		        if (attributes.constructor == Array) {
		            for (i = 0; i < attributes.length; i++) {
		                this.writeAttribute(attributes[i], attributeValues[i], buffer);
		            }
		        } else {
		            this.writeAttribute(attributes, attributeValues, buffer);
		        }
		    }
		    if (close) {
		        buffer.push(this.SLASH);
		    }
		    buffer.push(this.END_TAG);
		},
		writeNoAttributeElement: function(elementName, content, buffer)  {
		    this.startElement(elementName, null, null, false, buffer);
		    buffer.push(content);
		    this.endElement(elementName, buffer);
		},
		endElement: function(elementName, buffer) {
		    buffer.push(this.START_TAG);
		    buffer.push(this.SLASH);
		    buffer.push(elementName);
		    buffer.push(this.END_TAG);
		},
		writeScope: function(/*ibm.portal.search.scope.Scope*/ scope) {
			 var buffer = [];
		    // start scope tag
		    this.startElement(this.SCOPE_ELEMENT, this.ID_ATTRIBUTE, scope.id, false, buffer);
		    
		    //write title, description and  query text
		    this.writeNoAttributeElement(this.TITLE_ELEMENT, scope.label, buffer);
		    this.writeNoAttributeElement(this.DESCRIPTION_ELEMENT, scope.description, buffer);
		    if (scope.query != null) {
		        this.writeNoAttributeElement(this.QUERY_TEXT_ELEMENT, scope.query, buffer);
		    }
		    
		    //write elements
		    for (elementIndex = 0; elementIndex < scope.scopeElements.length; elementIndex++) {
		        this.writeScopeElement(scope.scopeElements[elementIndex], buffer);
		    }
		    //write properties
		    for (property in scope.properties) {
		        this.startElement(this.SCOPE_PROPERTY_ELEMENT, 
		            [this.KEY_ATTRIBUTE, this.VALUE_ATTRIBUTE], 
		            [property, scope.properties[property]], true, buffer);
		    }
		    //end scope
		    this.endElement(this.SCOPE_ELEMENT, buffer);
		    //transform the buffer to a string
		    return buffer.join("");
		},
		writeScopeElement: function(/*ibm.portal.search.scope.ScopeElement*/ scopeElement, buffer) {
			this.startElement(this.SCOPEELEMENT_ELEMENT, null, null, false, buffer);
		
		    //write locations
		    for (index = 0; index < scopeElement.locations.length; index++) {
		        var sourcePathPair = scopeElement.locations[index];
		        var source = sourcePathPair[0];
		        var path = sourcePathPair[1];
		        if (path === null) {
		            this.startElement(this.LOCATION_ELEMENT, 
		                this.SOURCE_ATTRIBUTE, source, true, buffer);
		        } else {
		            this.startElement(this.LOCATION_ELEMENT, 
		                [this.SOURCE_ATTRIBUTE, this.PATH_ATTRIBUTE],
		                 sourcePathPair, true, buffer);
		        }
		    }
		    
		    //write features
		    for (id in scopeElement.features) {
		        this.startElement(this.FEATURE_ELEMENT, 
		                this.FEATURE_ID, id, false, buffer);
		        
		        var value = scopeElement.features[id] ;
		        if (dojo.isArray(value)) {
		            for (i = 0; i < value.length; i++) {
		                this.writeNoAttributeElement(this.FEATURE_VALUE_ELEMENT, value[i], buffer);
		            }
		        } else {
		            this.writeNoAttributeElement(this.FEATURE_VALUE_ELEMENT, value, buffer);
		        }
		        this.endElement(this.FEATURE_ELEMENT, buffer);
		    }
		    
		    this.endElement(this.SCOPEELEMENT_ELEMENT, buffer);
		}
});

dojo.declare(
	"ibm.portal.search.scope.Scope", null,
	{
	//summary:
	//	a Javascript object reflecting a SIAPI scope
	
	//id: String
	//	the id of the scope. It is not required for contextual scopes.
	id: null,
	
	//label: String
	//	name of the scope. Will be displayed in the search menu.
	label: null,
	
	//description: String
	//	description of the scope. Will be displayed as tooltip in the search menu.
	description: null,
	
	//icon: String
	//	URL of an icon representing this scope. Will be displayed in the search menu.
	icon: null,
	
	//params:
	//	should include values for the label, description, icon, and possibly id attributes
	constructor: function(/*String*/ name, /*String*/ description, /*String*/ icon, /*String?*/ id) {
		this.label = name;
		this.description = description;
		this.icon = icon;
		this.id = (id != null)? id : name;
	},
	
	//summary: 
	//	Adds a scope element to the scope
	addElement: function (/*ibm.portal.search.scope.ScopeElement*/scopeElement) {
		if (this.scopeElements == null) {
			this.scopeElements = [];
		}
    	this.scopeElements.push(scopeElement);
	},
	
	//summary: 
	//	Adds the property to the scope
	addProperty: function (/*String*/ key, /*String*/ value) {
		if (this.properties == null) {
			this.properties = {};
		}
    	this.properties[key] = value;
	},
	
	//summary: 
	//	Adds the query text to the scope
	addQuery: function (/*String*/ query) {
		this.query = query;
	},
	
	//summary:
	//	checks whether this is a contextual scope
	isContextual: function() {
		return (this.scopeElements != null);
	},
	
	//summary:
	//	extracts a string represenation of the scope which can be sent to a search service
	getValue: function () {
		if (this.isContextual()) {
			var writer = new ibm.portal.search.scope.XmlWriter();
			return writer.writeScope(this);
		} else {
			return this.id;
		}
	}
});

dojo.declare(
	"ibm.portal.search.scope.ScopeElement", null,
	{
	constructor: function() {
    	this.locations = [];
	    this.features = {};
	},
	//summary:
	//	Adds a location to the scope element. path parameter is optional.
	addLocation: function(/*String*/ source, /*String*/ path) {
    	this.locations.push([source, path]);
	},
	//summary:
	//	Adds the feature to the scope element.
	//value: can be either a single value, or an array of values (each of them a string)
	//	
	addFeature: function(/*String*/ id, /*String or String array*/value) {
	    if (dojo.isArray(value)) {
	        this.features[id] = [];
	        for (i = 0; i < value.length; i++)  {
	              this.features[id].push(value[i]);
	        }       
	    } else {
	        this.features[id] = value;
	    }
	}
});

//a topic which indicates a search scope was selected
ibm.portal.search.scope.SCOPE_SELECTED_TOPIC = 'ibm.portal.search.SCOPE_SELECTED_TOPIC';
	
//a topic which is used for adding a new contextual search scope
ibm.portal.search.scope.ADD_SCOPE_TOPIC = 'ibm.portal.search.ADD_SCOPE_TOPIC';

//a topic which is used for submitting a new scoped query to the search center
ibm.portal.search.scope.SUBMIT_SCOPE_TOPIC = 'ibm.portal.search.SUBMIT_SCOPE_TOPIC';

//a topic which indicates a new search query was submitted
ibm.portal.search.NEW_QUERY_TOPIC = 'ibm.portal.search.NEW_QUERY_TOPIC';

//this constant value represents the currently selected scope
ibm.portal.search.scope.SELECTED_SCOPE = 'ibm.portal.search.scope.SELECTED_SCOPE';

//Summary:
// Appends the given scope to the search menu in the theme as a contextual scope.
ibm.portal.search.scope.addContextualScope = function(/*ibm.portal.search.scope.Scope*/ scope, 
	/*Boolean?*/ isDefault) {
	dojo.publish(ibm.portal.search.scope.ADD_SCOPE_TOPIC, [scope, isDefault]);
}

//Summary:
//	Performs search for the supplied query text upon the supplied scope. 
//	The call is synchronous and the caller is directed to the Search Center page.
ibm.portal.search.scope.submit = function(/*ibm.portal.search.scope.Scope*/ scope, /*String?*/ query) {
	dojo.publish(ibm.portal.search.scope.SUBMIT_SCOPE_TOPIC, [scope, query]);
}

dojo.declare(
	"ibm.portal.search.Query", null,
	{
	//summary:
	//	a Javascript object reflecting a SIAPI query
	
	//text: String
	//	the actual text of the query
	text: null,

	//language: String
	//	the language of the query
	language: null,
	
	//scope: String
	//	the value of the query scope 
	scope: null,
	
	//advancedQueries: Array
	//	Advanced query parts which should be submitted but not displayed
	advancedQueries: null,
	
	//returns the full text of the query, including advanced parts
	getFullQueryText: function() {
		var queryText = this.text;
		if (this.advancedQueries.length > 0) {
			return queryText + " " + this.advancedQueries.join(" ");
		}
		return queryText;
	},
	
	//add an advanced part to the query string
	addAdvancedQuery: function(/*String*/advancedQuery) {
		this.advancedQueries.push(advancedQuery);
	},
	
	//params:
	//	should include values for the text, language, and scope attributes
	constructor: function(params) {
		this.advancedQueries = [];
		dojo.mixin(this, params);
	}
});

//define the ibm.portal.search.util namespace
if (ibm.portal.search.util == undefined) {
	ibm.portal.search.util = {};
}

//summary:
//	returns the URL of the search service for the supplied action and parameters
ibm.portal.search.util.getServiceUrl = function(root, action, params) {
	var urlBuffer = [root];
	urlBuffer.push(action);
	urlBuffer.push('?');
	urlBuffer.push(dojo.objectToQuery(params));
	return urlBuffer.join('');
}

//summary:
//	returns the first word in the current path
ibm.portal.search.util.getContextRoot = function() {
	var end = window.location.pathname.indexOf("/", 1);
	return window.location.pathname.substring(0, end);
}

//summary:
//	opens a new window (smaller than the current one). useful for displaying help
ibm.portal.search.util.openSideWindow = function(/*String*/newWindowUrl) {
	var newWindowAtts = 'width=800,height=600,toolbar=yes,location=no,directories=no,status=yes,menubar=no,scrollbars=yes,resizable=yes,';
	var browserSpecificAtts = dojo.isIE? 'left=15,top=15': 'screenX=15,screenY=15';
	window.open(newWindowUrl, '', newWindowAtts + browserSpecificAtts);
}

//summary:
//	returns the user locale, which is set as the language of the html document
ibm.portal.search.util.getLocale = function() {
	return document.documentElement.lang;
}

ibm.portal.search.util.xmlNamespaces = {
	'atom': 'http://www.w3.org/2005/Atom',
	'opensearch': 'http://a9.com/-/spec/opensearch/1.1/',
	'xsl': 'http://www.w3.org/1999/XSL/Transform',
	'xhtml': 'http://www.w3.org/1999/xhtml'
}

//summary:
//	get the value of an xml node
ibm.portal.search.util.getXpathValue = function(/*DomNode*/ xmlDoc, /*String*/ nodePath) {
	var nodes =	com.ibm.portal.xpath.evaluateXPath(nodePath, 
		xmlDoc, ibm.portal.search.util.xmlNamespaces);
	
	if (nodes == null || nodes.length == 0) {
		return null;
	}
	var node = nodes[0];
	if (node.firstChild == null) {
		return node.data;
	}
	return node.firstChild.data;
}
