/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2007 - All Rights reserved.  **/
dojo.provide("ibm.ibmClientModel");
if(!dojo._hasResource["dojox.data.dom"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.dom"] = true;
dojo.provide("dojox.data.dom");

//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

//FIXME:  Remove this file when possible.
//This file contains internal/helper APIs as holders until the true DOM apis of Dojo 0.9 are finalized.
//Therefore, these should not be generally used, they are present only for the use by XmlStore and the
//wires project until proper dojo replacements are available.  When such exist, XmlStore and the like
//will be ported off these and this file will be deleted.
dojo.experimental("dojox.data.dom");

dojox.data.dom.createDocument = function(/*string?*/ str, /*string?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object.
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	var _document = dojo.doc;

	if(!mimetype){ mimetype = "text/xml"; }
	if(str && (typeof dojo.global["DOMParser"]) !== "undefined"){
		var parser = new DOMParser();
		return parser.parseFromString(str, mimetype);	//	DOMDocument
	}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
		var prefixes = [ "MSXML2", "Microsoft", "MSXML", "MSXML3" ];
		for(var i = 0; i<prefixes.length; i++){
			try{
				var doc = new ActiveXObject(prefixes[i]+".XMLDOM");
				if(str){
					if(doc){
						doc.async = false;
						doc.loadXML(str);
						return doc;	//	DOMDocument
					}else{
						console.log("loadXML didn't work?");
					}
				}else{
					if(doc){ 
						return doc; //DOMDocument
					}
				}
			}catch(e){ /* squelch */ };
		}
	}else if((_document.implementation)&&
		(_document.implementation.createDocument)){
		if(str){
			if(_document.createElement){
				// FIXME: this may change all tags to uppercase!
				var tmp = _document.createElement("xml");
				tmp.innerHTML = str;
				var xmlDoc = _document.implementation.createDocument("foo", "", null);
				for(var i = 0; i < tmp.childNodes.length; i++) {
					xmlDoc.importNode(tmp.childNodes.item(i), true);
				}
				return xmlDoc;	//	DOMDocument
			}
		}else{
			return _document.implementation.createDocument("", "", null); // DOMDocument
		}
	}
	return null;	//	DOMDocument
}

dojox.data.dom.textContent = function(/*Node*/node, /*string?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child 
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	if(arguments.length>1){
		var _document = node.ownerDocument || dojo.doc;  //Preference is to get the node owning doc first or it may fail
		dojox.data.dom.replaceChildren(node, _document.createTextNode(text));
		return text;	//	string
	} else {
		if(node.textContent !== undefined){ //FF 1.5
			return node.textContent;	//	string
		}
		var _result = "";
		if(node == null){
			return _result; //empty string.
		}
		for(var i = 0; i < node.childNodes.length; i++){
			switch(node.childNodes[i].nodeType){
				case 1: // ELEMENT_NODE
				case 5: // ENTITY_REFERENCE_NODE
					_result += dojox.data.dom.textContent(node.childNodes[i]);
					break;
				case 3: // TEXT_NODE
				case 2: // ATTRIBUTE_NODE
				case 4: // CDATA_SECTION_NODE
					_result += node.childNodes[i].nodeValue;
					break;
				default:
					break;
			}
		}
		return _result;	//	string
	}
}

dojox.data.dom.replaceChildren = function(/*Element*/node, /*Node || array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	var nodes = [];
	
	if(dojo.isIE){
		for(var i=0;i<node.childNodes.length;i++){
			nodes.push(node.childNodes[i]);
		}
	}

	dojox.data.dom.removeChildren(node);
	for(var i=0;i<nodes.length;i++){
		dojo._destroyElement(nodes[i]);
	}

	if(!dojo.isArray(newChildren)){
		node.appendChild(newChildren);
	}else{
		for(var i=0;i<newChildren.length;i++){
			node.appendChild(newChildren[i]);
		}
	}
}

dojox.data.dom.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo._destroyElement on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	var count = node.childNodes.length;
	while(node.hasChildNodes()){
		node.removeChild(node.firstChild);
	}
	return count; // int
}


dojox.data.dom.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	if(node.innerXML){
		return node.innerXML;	//	string
	}else if (node.xml){
		return node.xml;		//	string
	}else if(typeof XMLSerializer != "undefined"){
		return (new XMLSerializer()).serializeToString(node);	//	string
	}
}


}

/* Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */

dojo.provide( "com.ibm.portal.xpath" );

// generic impl

com.ibm.portal.xpath.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
    if (typeof ActiveXObject != "undefined") {
        return com.ibm.portal.xpath.ie.evaluateXPath(xpathExpr, doc, namespaces);
    }
    else {
        return com.ibm.portal.xpath.gecko.evaluateXPath(xpathExpr, doc, namespaces);
    }
}

// IE Implementation

dojo.provide( "com.ibm.portal.xpath.ie" );

com.ibm.portal.xpath.ie.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        if (namespaces) {        
            var ns = "";
            for (var prop in namespaces) {
                ns += "xmlns:"+prop+"='"+namespaces[prop]+"' ";
            }
            if (doc.ownerDocument) {
                doc.ownerDocument.setProperty("SelectionNamespaces", ns);
            } else {
                doc.setProperty("SelectionNamespaces", ns);
            }
        }
        var result = doc.selectNodes(xpathExpr);

        var thisResult;
        var resultSet = new Array();
        var len = 0;
        for (var i=0; i<result.length; i++) {
            thisResult = result[i];
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        }
        return resultSet;
}


// GECKO Implementation

dojo.provide( "com.ibm.portal.xpath.gecko" );

com.ibm.portal.xpath.gecko.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        var result;
        try {
        	var rootDoc = doc;
        	if(!rootDoc.evaluate) rootDoc = doc.ownerDocument;
            result = rootDoc.evaluate( xpathExpr, doc, function(prefix) { return namespaces[prefix] || null;}, XPathResult.ANY_TYPE, null );
        } 
        catch (exc)
        {
            //Callers should catch this and can substitute their own error message
            throw new Error("Error with xpath expression" + exc);
        }
    
        //alert(result.resultType);
        var thisResult;
        var resultSet = new Array();
        var len = 0;
        do {
            thisResult = result.iterateNext();
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        } while ( thisResult );
        return resultSet;
}


/* Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */

dojo.provide( "ibm.portal.xml.xpath" );
dojo.require( "com.ibm.portal.xpath" );

// generic impl

ibm.portal.xml.xpath.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
    return com.ibm.portal.xpath.evaluateXPath(xpathExpr, doc, namespaces);
}

// IE Implementation

dojo.provide( "ibm.portal.xml.xpath.ie" );

ibm.portal.xml.xpath.ie.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        return com.ibm.portal.xpath.ie.evaluateXPath(xpathExpr, doc, namespaces);
}


// GECKO Implementation

dojo.provide( "ibm.portal.xml.xpath.gecko" );

ibm.portal.xml.xpath.gecko.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        return com.ibm.portal.xpath.gecko.evaluateXPath(xpathExpr, doc, namespaces);
}


/* ***************************************************************** */
/*                                                                                                                                    */
/* IBM Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */
dojo.provide( "com.ibm.portal.xslt" );

dojo.require( "dojox.data.dom" );

dojo.declare( "com.ibm.portal.xslt.TransformerFactory",
    null,
    {
        constructor: function() {
            // create our internal XSL cache
            this._xsltMap = new Array();
        },
        /**
         * summary:
         *		Creates a new com.ibm.portal.xslt.Transformer object.
         */
        newTransformer: function(/*String*/ xsltUrl) {
             ibm.portal.debug.entry( "newTransformer", [ xsltUrl ] );
             var trf = this._getCached(xsltUrl);
             if(trf == null) {
             	trf = new com.ibm.portal.xslt.Transformer(xsltUrl);
             	this._xsltMap.push({url: xsltUrl, transformer: trf});
             }
             return trf;
        },
        _getCached: function(xsltUrl) {
        	var xslt = null;
             // try to get the XSL object from our internal cache
             for (i=0; i<this._xsltMap.length; i++) {
                 var entry = this._xsltMap[i];
                 if (xsltUrl == entry.url) {
                     // alert("cache hit");
                     xslt = entry.transformer;
                     break;
                 }
             }
             return xslt;
        }
    }
);

dojo.declare( "com.ibm.portal.xslt.Transformer",
    null,
    {
        /**
         * summary:
         *		Creates a new transformer for cross-browser XSLT operations
         */
        constructor: function(/* String */ xsltUrl) {
            this._xslt = com.ibm.portal.xslt.loadXsl(xsltUrl);
        },
        /**
         * summary:
         *		Transforms the given xmlDoc and puts the result into the passed
         *		DOM node
         *
         * xmlDoc:
         * 		XML Document to transform
         * xsltParams:
         *		Set of configuration parameters to pass to the transformation
         *		engine.  Object map in this format: {param1Name: "param1Value", param2Name: "param2Value"}
         * node:
         * 		DOM node to put the transform results in
         * doc:
         * 		HTML document to update
         */ 
        transformToRegion: function(xmlDoc, xsltParams, node, doc) {
			if (dojo.isIE) {
				var resultStr = com.ibm.portal.xslt.transform(xmlDoc, this._xslt, null, xsltParams, true);
				node.innerHTML = dojo.string.trim(resultStr);
			} else {
				/*
				var resultsDoc = this.transformToDocument( xmlDoc, xsltParams);
				dojox.data.dom.replaceChildren(node, resultsDoc.documentElement);
				*/
				var frag = com.ibm.portal.xslt.gecko._transformToFragment(xmlDoc, this._xslt, null, xsltParams, doc);
				node.innerHTML = "";
				node.appendChild(frag);
			}
        },
        /** 
         * summary:
         *		Transforms the given xmlDoc and returns a new XML document or String containing
         *		the result
         * xmlDoc:
         * 		XML Document to transform
         * xsltParams:
         *		Set of configuration parameters to pass to the transformation
         *		engine.  Object map in this format: {param1Name: "param1Value", param2Name: "param2Value"}
         * bReturnAsString:
         * 		Boolean indicating whether a XML document or String should be returned.
         */
        transformToDocument: function(xmlDoc, xsltParams, bReturnAsString) {
             var results = com.ibm.portal.xslt.transform(xmlDoc, this._xslt, null, xsltParams, bReturnAsString);
             return results;
        }
    }
);

// global transformer factory
com.ibm.portal.xslt.TRANSFORMER_FACTORY = new com.ibm.portal.xslt.TransformerFactory();



//---------------------------------------------------------------------- xml related utility methods
/*
 * loadXml(sUrl) - returns oDomDoc. parses xml from the url into DOM document object.
 * loadXmlString() - returns oDomDoc. parses xml from the contents of the string into DOM document object.
 * loadXsl()
 * transform()
 */
com.ibm.portal.xslt.ie = {};
com.ibm.portal.xslt.gecko = {};

com.ibm.portal.xslt.getXmlHttpRequest = function() {
	var oXml = null;
	if (typeof ActiveXObject != "undefined") {
		oXml = new ActiveXObject("Microsoft.XMLHTTP");
	} else {
		oXml = new XMLHttpRequest();
	}
  return oXml;
}

com.ibm.portal.xslt.loadXml = function(sUrl) {
	if (typeof ActiveXObject != "undefined") 
		return com.ibm.portal.xslt.ie.loadXml(sUrl);
	else 
		return com.ibm.portal.xslt.gecko.loadXml(sUrl);
}

com.ibm.portal.xslt.loadXmlString = function(sXml) {
	if (typeof ActiveXObject != "undefined") 
		return com.ibm.portal.xslt.ie.loadXmlString(sXml);
	else 
		return com.ibm.portal.xslt.gecko.loadXmlString(sXml);
}

com.ibm.portal.xslt.loadXsl = function(sUrl) {
	if (typeof ActiveXObject != "undefined") 
		return com.ibm.portal.xslt.ie.loadXsl(sUrl);
	else 
		return com.ibm.portal.xslt.gecko.loadXsl(sUrl);
}

com.ibm.portal.xslt.transform = function (xml, xsl, sXslMode, aXslParams, bReturnString)
{
   ibm.portal.debug.entry( "transform", [xml,xsl,sXslMode,aXslParams,bReturnString] );  
   if (typeof ActiveXObject != "undefined") {
    return com.ibm.portal.xslt.ie.transform(xml, xsl, sXslMode, aXslParams,bReturnString);
  }
  else 
    return com.ibm.portal.xslt.gecko.transform(xml, xsl, sXslMode, aXslParams,bReturnString);
}

com.ibm.portal.xslt.transformAndUpdate = function (/*HTMLElement*/nodeToUpdate, /*XMLDocument*/xml, /*XMLDocument*/xsl, /*String?*/sXslMode, /*Map*/aXslParams) {
	ibm.portal.debug.entry( "transformAndUpdate", [ nodeToUpdate, xml, xsl, sXslMode, aXslParams ]);
	
	if ( typeof ActiveXObject != "undefined" ) {
		var resultStr = com.ibm.portal.xslt.transform(xml, xsl, sXslMode, aXslParams, true);
		nodeToUpdate.innerHTML = dojo.string.trim(resultStr);
	} else {
		var doc = nodeToUpdate.ownerDocument ? nodeToUpdate.ownerDocument : document;
		var frag = com.ibm.portal.xslt.gecko._transformToFragment(xml, xsl, sXslMode, aXslParams, doc);
		nodeToUpdate.innerHTML = "";
		nodeToUpdate.appendChild(frag);
	}
	
	ibm.portal.debug.exit( "transformAndUpdate" );
}

//---------------------------------------------------------------------- IE xml related utility methods

com.ibm.portal.xslt.ie.XSLT_PROG_IDS = [
		"Msxml2.XSLTemplate.6.0",
		"Msxml2.XSLTemplate.5.0",
		"Msxml2.XSLTemplate.4.0",
		"MSXML2.XSLTemplate.3.0",
		"MSXML2.XSLTemplate"
	];
	
com.ibm.portal.xslt.ie.DOM_PROG_IDS = [
		"Msxml2.DOMDocument.6.0",
		"Msxml2.DOMDocument.5.0",
		"Msxml2.DOMDocument.4.0",
		"MSXML2.DOMDocument",
		"MSXML.DOMDocument",
		"Microsoft.XMLDOM"
	];
	
com.ibm.portal.xslt.ie.FTDOM_PROG_IDS = [
		"Msxml2.FreeThreadedDOMDocument.6.0",
		"Msxml2.FreeThreadedDOMDocument.5.0",
		"Msxml2.FreeThreadedDOMDocument.4.0",
		"MSXML2.FreeThreadedDOMDocument",
		"MSXML.FreeThreadedDOMDocument",
		"Microsoft.FreeThreadedXMLDOM"
	];
	
com.ibm.portal.xslt.ie._getMSXMLImpl = function(array) {
	// This iterates through the passed in array of Strings which represent ProgIDs for
	// MSXML objects, and attempts to find the first available one and returns an instance
	// to the caller
	for(var i = 0; i < array.length; i++) {
		try {
			var impl = new ActiveXObject(array[i]);
			if(impl) return impl;
		}
		catch(err){}
	}
	throw new Error("No MSXML implementation exists");
}

com.ibm.portal.xslt.ie.loadXml = function(sUrl) {
	//var oXmlDoc = new ActiveXObject("MSXML2.DOMDocument");
	var oXmlDoc = this._getMSXMLImpl(this.DOM_PROG_IDS);
	oXmlDoc.async=0;
	oXmlDoc.resolveExternals = 0;
  	if(!oXmlDoc.load(sUrl))
  	{
  		//Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xml file " + sUrl);
  	}
	return oXmlDoc;

}

com.ibm.portal.xslt.ie.loadXmlString = function(sXml) {
	var oXmlDoc = this._getMSXMLImpl(this.DOM_PROG_IDS);
	oXmlDoc.async=0;
	oXmlDoc.resolveExternals = 0;
  	if(!oXmlDoc.loadXML(sXml))
  	{
  	    //Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xml string " + sXml); 
  	}
	return oXmlDoc;
}

com.ibm.portal.xslt.ie.loadXsl = function(sUrl) {
	//we need to use MSXML2.FreeThreadedDOMDocument interface in order to support 
	//mode and parameters in XSL transformation.
	var oXslDoc = this._getMSXMLImpl(this.FTDOM_PROG_IDS);
	oXslDoc.async=0;
	oXslDoc.resolveExternals = 0;
  	if(!oXslDoc.load(sUrl))
  	{
  		//Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xsl file " + sUrl);
  	}	
	return oXslDoc;
}

com.ibm.portal.xslt.ie.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	var oXml = xmlDoc;
	var oXsl = xsl;

     try {		
		if(!oXsl.documentElement) oXsl = this.loadXsl(xsl);
	 }
	 catch(e) {
		var sMsg = e.message;
		throw new Error(""+sMsg, ""+sMsg);
	 }
	//create the xsl processor and apply the transformation
	var oXslt = this._getMSXMLImpl(this.XSLT_PROG_IDS);
	oXslt.stylesheet = oXsl;
	var oXslProc = oXslt.createProcessor();
	oXslProc.input = oXml;
	
	//set paramaters if any
	if(aXslParams) {
		for(var p in aXslParams) {
			oXslProc.addParameter(p, aXslParams[p]);
		}
	}
	if (sXslMode) oXslProc.addParameter("mode", sXslMode);
	
	if (bReturnString) {
		if(!oXslProc.transform()) {
			//Callers should catch this and can substitute their own error message
  			throw new Error("Error transforming xml doc " + oXml); 
  		}
  		return oXslProc.output;
	} else {
		var oHtmlDoc = this._getMSXMLImpl(this.DOM_PROG_IDS);
		oHtmlDoc.async = false;
		oHtmlDoc.validateOnParse = false;
		oXml.transformNodeToObject(oXsl,oHtmlDoc);
		return oHtmlDoc;	
	}
}

//---------------------------------------------------------------------- GECKO xml related utility methods

com.ibm.portal.xslt.gecko.loadXml = function(sUrl) {
    //var oXmlResponse = NG.ServerRequest.postRequest(sUrl);
//    if (oXmlResponse) return xmlLoadString(oXmlResponse.responseText);
//    else return null;
	var oDomDoc = null;
	if(dojo.isSafari) {
		var xhr = new XMLHttpRequest();
		xhr.open( "GET", sUrl, false );
		xhr.send( null );
		  
		if ( xhr.status == 200 ) {
			oDomDoc = xhr.responseXML;
		}
	}
	else {
		oDomDoc = document.implementation.createDocument('','',null);
		oDomDoc.async = 0; // this is the important part
		oDomDoc.load(sUrl);
	} 
	return oDomDoc;
}

com.ibm.portal.xslt.gecko.loadXmlString = function(sXml) {
    var parser = new DOMParser();
    try { oXmlDoc = parser.parseFromString(sXml, "text/xml"); }
    catch (exc) {
	    //Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xml string " + sXml); 
    }
	return oXmlDoc;
}

com.ibm.portal.xslt.gecko.loadXsl = function(sUrl) {
	//This is done through createDocument because of anchor(#) we have in portal url. 
	//Do not change the code without testing the case.
	var oDomDoc = null;
	if(dojo.isSafari) {
		var xhr = new XMLHttpRequest();
		xhr.open( "GET", sUrl, false );
		xhr.send( null );
		  
		if ( xhr.status == 200 ) {
			oDomDoc = xhr.responseXML;
		}
	}
	else {
		oDomDoc = document.implementation.createDocument('','',null); 
		oDomDoc.async = 0; // this is the important part
		oDomDoc.load(sUrl);
	}
	
	return oDomDoc;
}

com.ibm.portal.xslt.gecko._getXSLTProc = function(xmlDoc, xsl, sXslMode, aXslParams) {
	var xslDoc = xsl;
	if(!xslDoc.documentElement) 
	{
		xslDoc = this.loadXsl(xsl);
	}        
    var proc = new XSLTProcessor();
    proc.importStylesheet(xslDoc); 

	
	//set parameters if any
	if(aXslParams) {
		for(var p in aXslParams)  {
			proc.setParameter(null, p, aXslParams[p]);
		}
	}
	
	if (sXslMode) proc.setParameter(null, "mode", sXslMode);
	
	return proc;
}

com.ibm.portal.xslt.gecko._transformToFragment = function(xmlDoc, xsl, sXslMode, aXslParams, doc) {
	var proc = com.ibm.portal.xslt.gecko._getXSLTProc(xmlDoc, xsl, sXslMode, aXslParams);
	
	var resultDoc = null;
	resultDoc = proc.transformToFragment(xmlDoc, doc);
	proc.clearParameters();
	return resultDoc;
}

com.ibm.portal.xslt.gecko.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	try {
		var resultDoc = null;
		if (!bReturnString) {
			var proc = com.ibm.portal.xslt.gecko._getXSLTProc(xmlDoc, xsl, sXslMode, aXslParams);
			resultDoc = proc.transformToDocument(xmlDoc);
			return resultDoc;
		}
		else {
			resultDoc = com.ibm.portal.xslt.gecko._transformToFragment(xmlDoc, xsl, sXslMode, aXslParams, document);
		}

		var serializer = new XMLSerializer();
		// need to trim newlines and spaces from beginning and end so that the root node always has the same
		// number of child nodes regardless of the browser implementation
		// this should be lossless of real expected content
		var resultStr = dojo.string.trim(serializer.serializeToString(resultDoc));
		if(dojo.isOpera && resultDoc.firstChild && resultDoc.firstChild.nodeName == 'result') {
			var start = resultStr.indexOf("<result>") + 8;
			var end = resultStr.lastIndexOf("</result>");
			resultStr = dojo.string.trim(resultStr.substring(start, end));
		}
		return resultStr;    
    }
    catch (exc)
    {
        //alert("error transforming document: "+exc)
		//Callers should catch this and can substitute their own error message
  		throw new Error("Error transforming xml doc " + exc);
	}
}

/* This method sets the content of a layer within the HTML page
 * to the result of transforming the xml parameter by the xsl
 * parameter. The xml and xsl parameters may be of any form
 * supported by the transformXml() method. The layer parameter
 * may be either a DOM object or the name of a DOM object that
 * can be found using the findObject() method.
 */

com.ibm.portal.xslt.setLayerContentByXml = function (layer, xml, xsl, xslparam,bReturnString) {
    var result = com.ibm.portal.xslt.transform(xml,xsl,null,xslparam,bReturnString);
    if (layer.innerHTML) layer.innerHTML = result;
    else {
        var obj = document.getElementById(layer);
        obj.innerHTML=result;
    }
}

/* ***************************************************************** */
/*                                                                                                                                    */
/* IBM Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */
dojo.provide( "ibm.portal.xml.xslt" );
dojo.require( "com.ibm.portal.xslt" );

//---------------------------------------------------------------------- xml related utility methods
/*
 * loadXml(sUrl) - returns oDomDoc. parses xml from the url into DOM document object.
 * loadXmlString() - returns oDomDoc. parses xml from the contents of the string into DOM document object.
 * loadXsl()
 * transform()
 */
ibm.portal.xml.xslt.ie = {};
ibm.portal.xml.xslt.gecko = {};

ibm.portal.xml.xslt.getXmlHttpRequest = function() {
	return com.ibm.portal.xslt.getXmlHttpRequest();
}

ibm.portal.xml.xslt.loadXml = function(sUrl) {
	return com.ibm.portal.xslt.loadXml(sUrl);
}

ibm.portal.xml.xslt.loadXmlString = function(sXml) {
	return com.ibm.portal.xslt.loadXmlString(sXml);
}

ibm.portal.xml.xslt.loadXsl = function(sUrl) {
	return com.ibm.portal.xslt.loadXsl(sUrl);
}

ibm.portal.xml.xslt.transform = function (xml, xsl, sXslMode, aXslParams, bReturnString)
{
   ibm.portal.debug.entry( "transform", [xml,xsl,sXslMode,aXslParams,bReturnString] );  
   return com.ibm.portal.xslt.transform(xml, xsl, sXslMode, aXslParams, bReturnString );
}

ibm.portal.xml.xslt.transformAndUpdate = function (/*HTMLElement*/nodeToUpdate, /*XMLDocument*/xml, /*XMLDocument*/xsl, /*String?*/sXslMode, /*Map*/aXslParams) {
	ibm.portal.debug.entry( "transformAndUpdate", [ nodeToUpdate, xml, xsl, sXslMode, aXslParams ]);
	com.ibm.portal.xslt.transformAndUpdate(nodeToUpdate, xml, xsl, sXslMode, aXslParams);	
	ibm.portal.debug.exit( "transformAndUpdate" );
}

//---------------------------------------------------------------------- IE xml related utility methods

ibm.portal.xml.xslt.ie.loadXml = function(sUrl) {
	return com.ibm.portal.xslt.ie.loadXml(sUrl);
}

ibm.portal.xml.xslt.ie.loadXmlString = function(sXml) {
	return com.ibm.portal.xslt.ie.loadXmlString(sXml);
}

ibm.portal.xml.xslt.ie.loadXsl = function(sUrl) {
	return com.ibm.portal.xslt.ie.loadXsl(sUrl);
}

ibm.portal.xml.xslt.ie.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	return com.ibm.portal.xslt.ie.transform(xmlDoc, xsl, sXslMode, aXslParams,bReturnString);
}

//---------------------------------------------------------------------- GECKO xml related utility methods

ibm.portal.xml.xslt.gecko.loadXml = function(sUrl) {
    return com.ibm.portal.xslt.gecko.loadXml(sUrl);
}

ibm.portal.xml.xslt.gecko.loadXmlString = function(sXml) {
    return com.ibm.portal.xslt.gecko.loadXmlString(sXml);
}

ibm.portal.xml.xslt.gecko.loadXsl = function(sUrl) {
	return com.ibm.portal.xslt.gecko.loadXsl(sUrl);
}

ibm.portal.xml.xslt.gecko.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	return com.ibm.portal.xslt.gecko.transform(xmlDoc, xsl, sXslMode, aXslParams,bReturnString);
}

/* This method sets the content of a layer within the HTML page
 * to the result of transforming the xml parameter by the xsl
 * parameter. The xml and xsl parameters may be of any form
 * supported by the transformXml() method. The layer parameter
 * may be either a DOM object or the name of a DOM object that
 * can be found using the findObject() method.
 */

ibm.portal.xml.xslt.setLayerContentByXml = function (layer, xml, xsl, xslparam,bReturnString) {
    com.ibm.portal.xslt.setLayerContentByXml(layer, xml, xsl, xslparam,bReturnString);
}

if(!dojo._hasResource["com.ibm.portal.state"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.state"] = true;
dojo.provide("com.ibm.portal.state");








dojo.declare("com.ibm.portal.state.StateManager", null, {

    /**
     * summary:
     *     Constructor which initializes our internal properties.
     */
    constructor: function(/*String*/ contentHandlerURL) {
        this.stateDOM = null;
        this.stateNode = null;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
        this.serializationManager = new com.ibm.portal.state.SerializationManager(contentHandlerURL);
    },

    /**
     * summary:
     *     Gets the current navigational state that can be read or modified
     *     using the offered state accessors.
     */
    getState: function() {
        return this.stateDOM; // dojo.dom.DOMDocument - DOM representing the navigational state
    },
	
    /**
     * summary:
     *     Creates a new state DOM. If a state is provided, this method will create a
     *     deep copy of the given state. Otherwise a new empty state DOM is created.
     *     
     * state: dojo.dom.DOMDocument - State that should be cloned. May be NULL.
     * xslDoc: dojo.dom.DOMDocument - XSL transformation which should be applied to the state copy. May be NULL.
     * xslParams: Map - Map of XSL parameters. May be NULL.
     */
    newState: function(/*dojo.dom.DOMDocument*/ state, /*dojo.dom.DOMDocument*/ xslDoc, /*Map*/ xslParams) {
        var resultState = null;
        // just create a new document if state is null
        if (state == null) {
            resultState = dojox.data.dom.createDocument();
        } else {
            // just clone the given state if no XSL transformation has been passed in
            if (xslDoc == null) {
                resultState = dojox.data.dom.createDocument(dojox.data.dom.innerXML(state));
            } else {
                // apply transformation; pass in XSL parameters
                var xslt = com.ibm.portal.xslt;
                var results = xslt.transform(state, xslDoc, null, xslParams, true);
                resultState = dojox.data.dom.createDocument(results);
            }
        }
        return resultState; /*dojo.dom.DOMDocument*/
    },
    
    /**
     * summary:
     *     Sets the current navigational state that can be read or modified
     *     using the offered state accessors.
     *     
     * state: dojo.dom.DOMDocument - the navigational state represented as a DOM.
     */
    reset: function(/*dojo.dom.DOMDocument*/ stateDOM) {        
        // remember the state
        this.stateDOM = stateDOM;
        // make sure that at least the "<state type='navigational'>" element exists
        this.stateNode = this._getStateNode(stateDOM);
    },
	
    /**
     * summary:
     *     Returns a SerializationManager which cam be used to serialize /deserialize
     *     state. 
     */
    getSerializationManager: function() {
        return this.serializationManager;
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows for reading and writing navigational
     *     state of a portlet. This includes portlet mode, window state, and render
     *     parameters.
     *     
     * portletWindowID: String - Unique identifier of the portlet window. Must not
     *     be NULL. 
     * state: dojo.dom.DOMDocument - the navigational state the requested portlet
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used.  
     */
    newPortletAccessor: function(/*String*/ portletWindowID, /*dojo.dom.DOMDocument*/ state) {
        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }
        
        // get the portlet node
        var expr = "state:portlet[@id='" + portletWindowID + "']";
        var portletNode = this._getSpecificStateNode("portlet", expr, stNode, stDOM);
        portletNode.setAttribute("id", portletWindowID);
        
        return new com.ibm.portal.state.PortletAccessor(portletNode, stDOM);
        // com.ibm.portal.state.PortletAccessor - Accessor to read and write portlet-specific state.
    },
	
    /**
     * summary:
     *     Gets a state accessor that allows for reading the set of portlet IDs that are
     *     part of the state DOM.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested portlet
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used.  
     */
    newPortletListAccessor: function(/*dojo.dom.DOMDocument*/ state) {
        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        return new com.ibm.portal.state.PortletListAccessor(stNode, stDOM);
        // com.ibm.portal.state.PortletListAccessor - Accessor to read portlet IDs.
    },    
    
    /**
     * summary:
     *     Gets a state accessor that allows for reading and writing page selection
     *     state information. This includes the currently selected page as well as label
     *     mappings.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested selection
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newSelectionAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the selection node
        var selectionNode = this._getSpecificStateNode("selection", "state:selection", stNode, stDOM);

        return new com.ibm.portal.state.SelectionAccessor(selectionNode, stDOM);
        // com.ibm.portal.state.SelectionAccessor - Accessor to read and write page selection state.
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a portlet window
     *     is in solo state or to set a portlet window to solo state.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested solo state
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newSoloStateAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the solo state information
        var soloNode = this._getSpecificStateNode("solo", "state:solo", stNode, stDOM);

        return new com.ibm.portal.state.SoloStateAccessor(soloNode, stDOM);
        // com.ibm.portal.state.SoloStateAccessor - Accessor to read and write solo state.
    },
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a theme-template
     * 	   has been specified and, if not, set one.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested theme-template
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newThemeTemplateAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the theme-template information
        var templateNode = this._getSpecificStateNode("theme-template", "state:theme-template", stNode, stDOM);

        return new com.ibm.portal.state.ThemeTemplateAccessor(templateNode, stDOM);
        // com.ibm.portal.state.ThemeTemplateAccessor - Accessor to read and write theme-template state.
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a locale
     * 	   has been specified and, if not, set one.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested locale
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newLocaleAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the locale information
        var localeNode = this._getSpecificStateNode("locale", "state:locale", stNode, stDOM);

        return new com.ibm.portal.state.LocaleAccessor(localeNode, stDOM);
        // com.ibm.portal.state.LocaleAccessor - Accessor to read and write locale information.
    },
    
    /**
     * summary:
     *     Internal method that resolves the <state type='navigational' /> node which is
     *     the root node of navigational state information. If not already present, this
     *     method will create the state node.
     *     
     * aStateDOM: dojo.dom.DOMDocument - the navigational state document that should be inspected.
     *    Must not be NULL. 
     */
    _getStateNode: function(aStateDOM) {
        var expr = "state:root/state:state[@type='navigational']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, aStateDOM, this.ns);
        var stateNode = null;         
        if (nodes == null || nodes.length <= 0) {
            //var rootNode = aStateDOM.firstChild;
            var rootNode = this._firstChild(aStateDOM);
            if (rootNode == null) {
                // create the root element
                rootNode = this._createElement(aStateDOM, "root");
                this._prependChild(rootNode, aStateDOM);
            } 
            //stateNode = rootNode.firstChild;
            stateNode = this._firstChild(rootNode);
            if (stateNode == null) {
                // create the state element
                stateNode = this._createElement(aStateDOM, "state");
                this._prependChild(stateNode, rootNode);
            }
            // add the 'type' attribute
            stateNode.setAttribute("type", "navigational");
        } else {
            // in most cases we will end up here
            stateNode = nodes[0];
        }
        return stateNode; 
    },
   
    /**
     * summary:
     *     Internal method that resolves a specific state node with the given node name
     *     which is located as a child of the given <state type='navigational'> node.
     *     If the node does not exist, this method will create a new node. 
     *     
     * aNodeName: String - the name of the node that should be resolved. Must not be NULL.
     * aPathExpression: String - XPath expression that addresses the node to be resolved
     *     relative to the given state node.
     * aStateNode: Element - DOM element representing the <state type='navigational'> node.
     * aStateDOM: dojo.dom.DOMDocument - the navigational state document that should be
     *     inspected. Must not be NULL. 
     */
    _getSpecificStateNode: function(aNodeName, aPathExpression, aStateNode, aStateDOM) {
        // evaluate the expression starting from the state node
        var nodes = com.ibm.portal.xpath.evaluateXPath(aPathExpression, aStateNode, this.ns);
        var node;
        if (nodes == null || nodes.length <= 0) {
            // create the state element
            node = this._createElement(aStateDOM, aNodeName);
            this._prependChild(node, aStateNode);
        } else {
            node = nodes[0];
        }
        return node;
    },
    _prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
	var firstChild = this._firstChild(parent);
        (firstChild != null) ? parent.insertBefore(node, firstChild) : parent.appendChild(node);
    },
    _firstChild: function(par) {
        if (par == null)
            return null;
        var res = par.firstChild;
        while (res) {
            if (!this._isIgnorable(res))
                return res;
            res = res.nextSibling;
        }
        return null;
    },
    _isIgnorable: function(nod) {
	var t = nod.nodeType;
	return t == 8 || t == 4 || t == 7 || (t == 3 && isAllWS(nod));
    },
    _isAllWS: function (nod) {
        return !(/[^\t\n\r ]/.test(nod.data));
    },
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
    
});


dojo.declare("com.ibm.portal.state.PortletAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this acccessor with the
     *     state DOM and the DOM node that keeps the state information
     *     of the respective portlet window in its subtree.
     */
    constructor: function (/*Element*/ portletNode, 
                           /*dojo.dom.DOMDocument*/ stateDOM) {

        this.portletNode = portletNode;
        this.stateDOM = stateDOM;
        this.parameters = new com.ibm.portal.state.Parameters(portletNode, stateDOM);
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };

        this.xsltURL = dojo.moduleUrl( "com", "ibm/portal/state/");
    },

    /**
     * summary:
     *     Gets the current portlet mode of the portlet window this accessor
     *     operates on.
     */
    getPortletMode: function() {
        
        var expr = "state:portlet-mode";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        var portletMode = ibm.portal.portlet.PortletMode.VIEW; 
        if (nodes != null && nodes.length > 0) {
            var txtNode = nodes[0].firstChild;
            if (txtNode != null) {
                portletMode = txtNode.nodeValue;
            }
        }
        return portletMode; // ibm.portal.portlet.PortletMode - The current portlet mode.
    },
	
    /**
     * summary:
     *     Gets the current window state of the portlet window this accessor
     *     operates on.
     */
    getWindowState: function() {
        var expr = "state:window-state";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        var windowState = ibm.portal.portlet.WindowState.NORMAL; 
        if (nodes != null && nodes.length > 0) {
            var txtNode = nodes[0].firstChild;
            if (txtNode != null) {
                windowState = txtNode.nodeValue;
            }
        }
        return windowState; // ibm.portal.portlet.WindowState - The current window state.
    },

    /**
     * summary:
     *     Gets the current render parameters of the portlet window this accessor
     *     operates on. The returned Parameters object allows for querying, adding,
     *     modifying and removing parameters.
     */
    getRenderParameters: function() {
        return this.parameters; // com.ibm.portal.state.Parameters - The current render parameters. 
    },

    /**
     * summary:
     *     Sets the portlet mode of the portlet window this accessor operates on
     *     by changing the navigational state DOM accordingly.
     *     
     * portletMode: ibm.portal.portlet.PortletMode - the portlet mode to be set.
     *     Must not be NULL. 
     */
    setPortletMode: function(/*ibm.portal.portlet.PortletMode*/ portletMode) {
        var expr = "state:portlet-mode";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        if (nodes == null || nodes.length <= 0) {
            var portletModeNode = this._createElement(this.stateDOM, "portlet-mode");
            this._prependChild(portletModeNode, this.portletNode);
			var txtNode = this.stateDOM.createTextNode(portletMode);
            this._prependChild(txtNode, portletModeNode);
        } else {
            nodes[0].firstChild.nodeValue = portletMode;
        }
    },

    /**
     * summary:
     *     Sets the window state of the portlet window this accessor operates on
     *     by changing the navigational state DOM accordingly.
     *     
     * windowState: ibm.portal.portlet.WindowState - the window state to be set.
     *     Must not be NULL. 
     */
    setWindowState: function(/*ibm.portal.portlet.WindowState*/ windowState) {
        var expr = "state:window-state";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        if (nodes == null || nodes.length <= 0) {
            var windowStateNode = this._createElement(this.stateDOM, "window-state");
            this._prependChild(windowStateNode, this.portletNode);
            var txtNode = this.stateDOM.createTextNode(windowState)
            this._prependChild(txtNode, windowStateNode);
        } else {
            nodes[0].firstChild.nodeValue = windowState;
        }
    },
    
    /**
     * summary:
     *     Gets a dojo.dom.DOMDocument which contains only the state of the portlet
     *     this accessor operates on. The returned DOMDocument still complies with the
     *     state.DTD i.e. it contains the complete path to the portlet node that
     *     contains the portlet's state information. 
     */
    getPortletState: function() {
        // TODO implement this function using XSLT
        
        /*var xslt = com.ibm.portal.xslt;
        // load the xsl file
        var uri = this.xsltURL + "SelectPortletState.xsl";
        var xslDoc = xslt.loadXsl(uri);
	    // apply the transformation
	    if (xslDoc.documentElement == null) { alert("xslDoc is null"); }
    	var results = xslt.transform(this.stateDOM, xslDoc, null, {"id" : this.portletNode.getAttribute("id")}, true);
        return dojo.dom.createDocumentFromText(results);*/
        
        // get a new portlet accessor operating on a new empty state
        var newDOM = dojox.data.dom.createDocument();
        var portletAcc = com.ibm.portal.state.STATE_MANAGER.newPortletAccessor(this.portletNode.getAttribute("id"), newDOM);
        portletAcc.setPortletMode(this.getPortletMode());
        portletAcc.setWindowState(this.getWindowState());
        var paramMap = this.getRenderParameters().getMap();
        if (paramMap.length > 0) {
            portletAcc.getRenderParameters().putAll(paramMap);
        }
        return newDOM; /* dojo.dom.DOMDocument */
    },

    /**
     * summary:
     *     Sets the complete state of the portlet this accessor operates on.
     *     Ignores all state information in the given state that is not
     *     related to the portlet this accessor operates on. 
     *     
     *  state: dojo.dom.DOMDocument - The state DOM that contains state for the
     *      portlet this accessor operates on. Must not be NULL.
     *  merge: boolean - Flag that indicates whether the given state of the portlet
     *      should be merged into the state of the accessor or if it should be replaced.
     *      May be NULL which defaults to false, i.e. the current state of the portlet
     *      is replaced with the state retrieved from the given DOM. 
     */
    setPortletState: function(/* dojo.dom.DOMDocument */ state, /* boolean */ merge) {
        // TODO implement this function using XSLT
        
        // get a new portlet accessor operating on a new empty state
        var portletAcc = com.ibm.portal.state.STATE_MANAGER.newPortletAccessor(this.portletNode.getAttribute("id"), state);
        this.setPortletMode(portletAcc.getPortletMode());
        this.setWindowState(portletAcc.getWindowState());
        var paramMap = portletAcc.getRenderParameters().getMap();
        // clear parameters only if we should replace them
        if (merge == null || merge == false) {
            this.getRenderParameters().clear();
        }
        if (paramMap.length > 0) {
            this.getRenderParameters().putAll(paramMap);
        }
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
    
});	



dojo.declare("com.ibm.portal.state.Parameters", null, {

    /**
     * summary:
     *     Constructor which initializes this Parameters object with the
     *     state DOM and the DOM node that keeps the parameters information.
     */
    constructor: function(/*Element*/ baseNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.baseNode = baseNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary: 
     *     Returns a read-only parameter map represented as a Object[].
     *     Returns an empty map if no parameters are available.
     */
    getMap: function() {
        var names = this.getNames();
        var map = new Array(names.length);
        for (var i = 0; i < names.length; i++) {
            var name = names[i];
            map[i] = {name: name, values : this.getValues(name)};
        }
        return map; // Object[] {name: String, values: String[]}
    },

    /**
     * summary:
     *     Returns a String[] containing the names of all parameters that have an
     *     associated value. Returns an empty array if no parameters are available.
     */
    getNames: function() {
        var expr = "state:parameters/state:param";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var names = new Array();
        if (nodes != null && nodes.length > 0) {
            var nodeCount = nodes.length;
            for (var i = 0; i < nodeCount; i++) {
                names[i] = nodes[i].getAttribute("name");
            }
        }
        return names;
    },

    /**
     * summary:
     *     Returns the String value associated with the parameter with the given name.
     *     
     * description:
     *     If there are one or more parameter values associated with the given name 
     * 	   it returns the first associated value.
     *
     * name: String - Name of the parameter for which the associated value is to
     *     be returned. Must not be NULL.
     */
    getValue: function(/*String*/ name) {
        var values = this.getValues(name);
        var firstValue = null;
        if (values != null && values.length > 0) {
            firstValue = values[0];
        }
		return firstValue;
    },

    /**
     * summary:
     *     Returns the String[] representing the values that are associated with
     *     the parameter with the given name.
     *
     * name: String - Name of the parameter for which the associated values are to
     *     be returned. Must not be NULL.
     */
    getValues: function(/*String*/ name) {
        var expr = "state:parameters/state:param[@name='" + name + "']/state:value";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var values = null;
        if (nodes != null && nodes.length > 0) {
            values = new Array(nodes.length);
            var nodeCount = nodes.length;
            for (var i = 0; i < nodeCount; i++) {
                // get the text value
                var txtNode = nodes[i].firstChild;
                if (txtNode != null) {
                    values[i] = txtNode.nodeValue;
                }
            }
        }
        return values;
    },

    /**
     * summary:
     *     Removes the parameter with the given name.
     *
     * name: String - Name of the parameter that should be removed. Must not be NULL.
     */
    remove: function (/*String*/ name) {
        var expr = "state:parameters/state:param[@name='" + name + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        if (nodes != null) {
			var toRemove = nodes[0];
			if(toRemove && toRemove.parentNode){
				toRemove.parentNode.removeChild(toRemove);
			}
		} 
    },

    /**
     * summary:
     *     Adds all parameters that are contained in the given map. Overrides the values
     *     of parameters that already exist.
     *
     * map: Object[] {name: String, values: String[]} - Two dimensional object array
     *     representing the map of parameters that should be added. Must not be NULL.
     */
    putAll: function(/*Object[] {name: String, values: String[]}*/ map) {
        if (map != null && map.length > 0) {
            for (var i = map.length - 1; i >= 0; i--) {
                var paramName = map[i].name;
                var paramValues = map[i].values;
                this.setValues(paramName, paramValues);
            }
        }
    },

    /**
     * summary:
     *     Sets a simple String value for the parameter with the given name. Overrides
     *     the value of the parameter if it already exists.
     *
     * name: String - The name of the parameter. Must not be NULL.
     * value: String - The value of the parameter that should be set. Must not be NULL. 
     */
    setValue: function (/*String*/ name, /*String*/ value) {
        this.setValues(name, new Array(value));
    },

    /**
     * summary:
     *     Sets a String[] representing the value of the parameter with the given name.
     *     Overrides the value of the parameter if it already exists.
     *
     * name: String - The name of the parameter. Must not be NULL.
     * value: String[] - The value of the parameter that should be set. Must not be NULL. 
     */
    setValues: function(/*String*/ name, /*String[]*/ values) {
        var expr = "state:parameters/state:param[@name='" + name + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var paramNode;
        if (nodes == null || nodes.length == 0) {
            // create the parameters node
            var paramsNode = null;// this._getFirstChildWithTag(this.baseNode, "state:parameters"); // do we really need to use the NS prefix here? bug??
            if (paramsNode == null) {
                paramsNode = this._createElement(this.stateDOM, "parameters");
                this._prependChild(paramsNode, this.baseNode);
            }
            // create the param node
            paramNode = this._createElement(this.stateDOM, "param");
            paramNode.setAttribute("name", name);
            this._prependChild(paramNode, paramsNode);
        } else {
            paramNode = nodes[0];
			// remove all children of paramNode
			dojox.data.dom.removeChildren(paramNode);
        }
        // putAll sometimes passes in null for values
        if (values != null) {
            for (var i = values.length - 1; i >= 0; i--) {
                var valueNode = this._createElement(this.stateDOM, "value");
                this._prependChild(valueNode, paramNode);
                var value = values[i];
                // make sure that we do not encode something like "undefined" into a value node 
                if (value != null) {
                    var txtNode = this.stateDOM.createTextNode(value);
                    this._prependChild(txtNode, valueNode);
                }
            }
        }
    },

    /**
     * summary:
     *     Removes all existing parameters.
     */
    clear: function() {
        var expr = "state:parameters";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        if (nodes != null) {
			var toRemove = nodes[0];
			if(toRemove && toRemove.parentNode){
				toRemove.parentNode.removeChild(toRemove);
			}
        } 
    },
	_getFirstChildWithTag: function(parentNode, tagName){
		// validate arguments
		if(!parentNode || !tagName) return null;
		var node = parentNode.firstChild;
		// iterate over all children
		while(node){
			// if an element node has a tag equal to tagname, return it
			if( node.nodeType == 1 && node.tagName && node.tagName.toLowerCase() == tagName.toLowerCase()){
				return node;
			}
			// otherwise, move on to the next sibling
			node = node.nextSibling;
		}
		// if we got here, we didn't find a match
		return null;
	},	
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
	
});	


dojo.declare("com.ibm.portal.state.PortletListAccessor", null, {
   
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node representing the root of navigational state
     *     information in the state DOM.
     */
    constructor: function(/*Element*/stateNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.stateNode = stateNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Returns an array of Strings representing the IDs of the portlet windows
     *     that are part of the state DOM this accessor operates on. Returns null
     *     the state DOM does not contain any portlet-related state information.
     */
    getPortlets: function() {
        // get all portlet nodes
        var expr = "state:portlet";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.stateNode, this.ns);
        var portletIDs = null;
        if (nodes != null && nodes.length > 0) {
            portletIDs = new Array(nodes.length);
            for (var i=0; i < nodes.length; i++) {
               var node = nodes[i];
               portletIDs[i] = node.getAttribute("id");  
            }
        }
        return portletIDs; /*String[]*/
    }
    
});


dojo.declare("com.ibm.portal.state.SelectionAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the selection information.
     */
    constructor: function(/*Element*/selectionNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.selectionNode = selectionNode;
        this.stateDOM = stateDOM;
        this.parameters = new com.ibm.portal.state.Parameters(this.selectionNode, stateDOM);
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Returns a String representing the unique identifier of the
     *     navigation node that represents the currently selected page.
     */
    getPageSelection: function() {
        return this.selectionNode.getAttribute("selection-node"); // String
    },
	
    /**
     * summary:
     *     Returns a String representing the unique identifier of the
     *     selected fragment. Typically the fragment is a layout node
     *     that is part of the layout model of the currently selected page.
     */
    getFragmentSelection: function() {
        var params = this.getParameters();
        var fragmentParams = params.getValues("frg");
        var fragmentID = null;
        if (fragmentParams != null && fragmentParams.length > 0) {
            fragmentID = fragmentParams[0];
            if (fragmentParams.length > 1) {
                if (fragmentID == "pw") {
                    fragmentID = fragmentParams[1];
                }
            }    
        }
        return fragmentID; // String
    },
    
    /**
     * summary:
     *     Returns a String representing the unique ID of the navigation node
     *     that represents the page that should be selected instead of the
     *     label with the given identifier. Returns null if there is no
     *     mapping available for the given label ID.
     *     
     * srcID:
     *     String representing the ID of the label that should be resolved.
     *     Must not be NULL.
     */
    getMapping: function(/*String*/ srcID) {
        var expr = "state:mapping[@src='" + srcID + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
        var targetID = null;
        if (nodes != null && nodes.length > 0) {
            var mappingNode = nodes[0];
            targetID = mappingNode.getAttribute("dst");
        }
        return targetID; // String or null
    },

    /**
     * summary:
     *     Returns a com.ibm.portal.state.Parameters object which contains
     *     selection parameters. Does not return NULL.
     * 
     */
    getParameters: function() {
        return this.parameters; // com.ibm.portal.state.Parameters
    },
    
    /**
     * summary:
     *     Changes the selection according to the given navigation node ID.
     *     
     * navigationNodeID:
     *     String representing the identifier of the page or label that should
     *     be selected. Must not be NULL.
     */
    setPageSelection: function(/*String*/ navigationNodeID) {
        this.selectionNode.setAttribute("selection-node", navigationNodeID);
    },

    /**
     * summary:
     *     Changes the fragment selection according to the given layout node ID.
     *     
     * layoutNodeID:
     *     String representing the identifier of the layout node that represents
     *     the oage fragment that should be selected. Must not be NULL.
     * isPortletWindow:
     *     Flag which indicates whether or not the layout node represents a
     *     portlet window. May be NULL to indicate that the layout node is a
     *     portlet window.
     */
    setFragmentSelection: function(/*String*/ layoutNodeID, /*boolean*/ isPortletWindow) {
        var params = this.getParameters();
        if (isPortletWindow == null || isPortletWindow == true) {
            var values = new Array(2);
            values[0] = layoutNodeID;
            values[1] = "pw";
            params.setValues("frg", values);
        } else {
            params.setValue("frg", layoutNodeID);
        }
    },

    /**
     * summary:
     *     Sets a label mapping that specifies the page that should be
     *     selected instead of the label. The label is identified via the
     *     given source ID, the page via the target ID. If the given targetID
     *     is null, the method will completely remove the mapping from the state.
     *     
     * srcID:
     *     ID of the label that should be mapped to a page. Must not be NULL.
     * targetID:
     *     ID of the page the label should be mapped to. May be NULL..
     */
    setMapping: function(/*String*/ srcID, /*String*/ targetID ) {
        if (targetID != null) {
            var expr = "state:mapping[@src='" + srcID + "']";
            var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
            var mappingNode;
            if (nodes != null && nodes.length > 0) {
                mappingNode = nodes[0];
            } else {
                mappingNode = this._createElement(this.stateDOM, "mapping");
                this._prependChild(mappingNode, this.selectionNode);
                mappingNode.setAttribute("src", srcID);
            }
            mappingNode.setAttribute("dst", targetID);
        } else {
            // this is equivalent with removing the mapping
            this.removeMapping(srcID);
        }
    },

    /**
     * summary:
     *     Removes the label mapping with the given source ID.
     *     
     * srcID:
     *     ID of the label that should be removed. Must not be NULL.
     */
    removeMapping: function(/*String*/ srcID) {
        var expr = "state:mapping[@src='" + srcID + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
        var removed = false;
        if (nodes != null && nodes.length > 0) {
            // actually there should be only one such node
            // but clean up if we ran into an inconsistent state 
            for (var i=0; i < nodes.length; i++) {
	   			var toRemove = nodes[i];
				if(toRemove && toRemove.parentNode){
					toRemove.parentNode.removeChild(toRemove);
				}

            }
            removed = true;
        }
        return removed;
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    },
    
    /**
     * @deprecated: use getPageSelection instead
     */
    getSelection: function() {
        return this.getPageSelection(); // String
    },

    /**
     * @deprecated: use setPageSelection instead
     */
    setSelection: function(/*String*/ navigationNodeID) {
        this.setPageSelection(navigationNodeID);
    }
    
});	


dojo.declare("com.ibm.portal.state.SoloStateAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the solo state information.
     */
    constructor: function(/*Element*/soloNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.soloNode = soloNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Sets the portlet window with the given ID into solo state.
     *     
     * portletWindowID:
     *     String representing the identifier of the portlet window which should
     *     be set to solo state. To entirely delete the solo state information from
     *     the state DOM, pass in NULL.
     */
    setSoloPortlet: function(/*String*/ portletWindowID) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.soloNode);
         if (portletWindowID != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(portletWindowID)
             this._prependChild(txtNode, this.soloNode);
         }
    },

    /**
     * summary:
     *     Returns the ID of the portlet window which is currently in solo state.
     *     Returns NULL if no such portlet window exists.
     */
    getSoloPortlet: function() {
        var txtNode = this.soloNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },

    /**
     * summary:
     *     Sets the page the portal should navigate to after leaving
     *     Solo state.
     * 
     * pageID:
     *     ID of the return page. Must not be NULL.
     */ 
    setReturnSelection: function(/*String*/ pageID) {
        this.soloNode.setAttribute("return-selection", pageID);
    },

    /**
     * summary:
     *     Returns the ID of the page the portal navigates to
     *     after leaving Solo state.
     */
    getReturnSelection: function() {
        return this.soloNode.getAttribute("return-selection");
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}   


});

dojo.declare("com.ibm.portal.state.ThemeTemplateAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the theme-template state information.
     */
    constructor: function(/*Element*/themeTemplateNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.themeTemplateNode = themeTemplateNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Sets the theme-template to the given value.
     *     
     * themeTemplate:
     *     String representing the theme-template which should be set. To entirely delete the theme-template
     *     information from the state DOM, pass in NULL.
     */
    setThemeTemplate: function(/*String*/ themeTemplate) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.themeTemplateNode);
         if (themeTemplate != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(themeTemplate)
             this._prependChild(txtNode, this.themeTemplateNode);
         }
    },

    /**
     * summary:
     *     Returns the name of the current theme-template.
     *     Returns NULL if no such value exists.
     */
    getThemeTemplate: function() {
        var txtNode = this.themeTemplateNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}
	
});

dojo.declare("com.ibm.portal.state.LocaleAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the locale information.
     */
    constructor: function(/*Element*/localeNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.localeNode = localeNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Sets the locale to the given value.
     *     
     * themeTemplate:
     *     String representing the locale which should be set. To entirely delete the locale
     *     information from the state DOM, pass in NULL.
     */
    setLocale: function(/*String*/ locale) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.localeNode);
         if (locale != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(locale)
             this._prependChild(txtNode, this.localeNode);
         }
    },

    /**
     * summary:
     *     Returns the name of the current locale in the state.
     *     Returns NULL if no such value exists.
     */
    getLocale: function() {
        var txtNode = this.localeNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },

	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}
	
});


/**
 * The serialization manager is able to serialize /deserialize state documents. 
 */
dojo.declare("com.ibm.portal.state.SerializationManager", null, {
    
    /**
     * Scheme of the uri that addresses the state REST service.
     */
    STATE_URI_SCHEME: "state",
    STATE_URI_POST: "state:encode",
    
    /**
     * The download mode
     */
    DOWNLOAD_MODE: "download",
    
    STATUS_UNDEFINED: 0,
    STATUS_OK: 1,
    STATUS_ERROR: 2,
    
    STATE_NS_URI: "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state",

    /**
     * Maximum length the serialized state must not exceed.
     * The actual URL length limit is 2048 but we need to take into account the base URL incl. query params.
     */
    STATE_THRESHOLD: 1024,
    
    /**
     * summary:
     *     Constructor which initializes the serialization manager with a URL that
     *     can be used to access the state REST service of portal.
     */
    constructor: function(/*String*/ stateServiceURL) {
        this.serviceURL = stateServiceURL;
    },
    
    /**
     * summary:
     *     Serializes the given state DOM into a form that can be included into URLs.
     *     If a callback function is provided, the method will execute asynchronously.
     *     Otherwise it will execute synchronously and return a
     *     {status: STATUS_OK /STATUS_ERROR /STATUS_UNDEFINED, state: dojo.dom.DOMDocument, url: String} object. 
     *     
     * state: dojo.dom.DOMDocument - The state DOM that should be serialized. Must not
     *     be NULL.
     * validate: boolean - Optional flag specifying whether the server should validate the state or not.
     *     Defaults to false. 
     * callbackfn: function - The callback function which should be called in case of
     *     internal status changes. May be NULL to indcate that the method should execute
     *     synchronously.
     */
    serialize: function(/*dojo.dom.DOMDocument*/ state, /*boolean*/ validate, /*Function*/ callbackfn) {
         ibm.portal.debug.entry( "SerializationManager.serialize", [ dojox.data.dom.innerXML( state ), validate, callbackfn ] );
         
         // get the serialized state
         var stateXML = dojox.data.dom.innerXML(state);
         // apply ASCII encoding to state XML
         // because we want to use it as a scheme-specific part in a URI or as part of a POST body
		 //RES: this used to call dojo.string.encodeAscii(), which also included special handling for unicode characters
		 // if escape() is not sufficient, see the _encodeAscii function defined in this object (though it doesn't work yet :)
         var encodedStateXML = escape(stateXML);
         //var encodedStateXML = dojo.string.encodeAscii(stateXML);
         //var encodedStateXML = this._encodeAscii(stateXML);
         var mType = this._getMimeType();
         var result = null;
         var me = this;
         ibm.portal.debug.text( "Mime type for response: " + mType );
         ibm.portal.debug.text( "Length of encoded state XML is: " + encodedStateXML.length );
         ibm.portal.debug.text( "Encoded state XML is: " + encodedStateXML );
         var digest = com.ibm.portal.services.PortalRestServiceConfig.digest;
         ibm.portal.debug.text( "Digest: " + digest );
         // check if we need to send a GET or a POST based on the length of the serialized state
         if (encodedStateXML.length <= this.STATE_THRESHOLD) {
             // GET
             // create the uri
             var payload = this.STATE_URI_SCHEME + ":" + encodedStateXML;
             var params;
             validate = (validate != null && validate == true);
             if (validate == true) {
                 if (digest != null) {
                     params = {"uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "preprocessors": "true", "digest": digest};
                 } else {
                     params = {"uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "preprocessors": "true"};
                 }
             } else {
                 if (digest != null) {
                     params = {"uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "digest": digest};
                 } else {
                     params = {"uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI};
                 }
             }
             
             ibm.portal.debug.text( "Doing a GET request: { url: \"" + this.serviceURL + "\", sync: " + ((callbackfn)?false:true) + ", content: " + params + ", handleAs: " + mType + ", transport: XMLHTTPRequest" );
             ibm.portal.debug.text( "Parameters: uri=\"" + params.uri + "\" mode=\"" + params.mode + "\" xmlns=\"" + params.xmlns + "\"" );
              
             dojo.xhrGet({
                 url: this.serviceURL,
                 sync: (callbackfn)?false:true,
                 content: params,
                 handleAs: mType,
                 handle: function(response, ioArgs) {
                      ibm.portal.debug.text( "Response: " + response );
                      result = me._handleSerializationResponse.call(me, response, callbackfn, state, validate);
                      return response;
                 },
                 transport: "XMLHTTPTransport"
             });
         } else {
             ibm.portal.debug.text( "Doing a POST request." );
             // tentative fix: content handler framework cannot deal with UTF-16 at the moment
             if (dojo.isIE) {
                 var idx = stateXML.indexOf("UTF-16");
                 if (idx >= 0) {
                     stateXML = stateXML.replace(/UTF-16/, "UTF-8");
                 }
             }
             var url = this.serviceURL + "?uri=" + this.STATE_URI_POST + "&xmlns=" + this.STATE_NS_URI;
             if (digest != null) {
                 url += "&digest=" + digest;
             }
             // POST
             dojo.rawXhrPost({
                 url: url,
                 sync: (callbackfn)?false:true,
                 postData: stateXML,
                 handleAs: mType,
                 headers: {"Content-Type":"text/xml"},
                 handle: function(response, ioArgs) {
                     result = me._handleSerializationResponse.call(me, response, callbackfn, state, validate);
                     return response;
                 },
                 transport: "XMLHTTPTransport"
             });
         }
         
         ibm.portal.debug.exit( "SerializationManager.serialize", result );
         return result;
    },
    
    /**
     * summary:
     *     Deserializes the state from the given URL. If a callback function is provided,
     *     the method will execute asynchronously. Otherwise it will execute synchronously
     *     and return a
     *     {status: STATUS_OK /STATUS_ERROR /STATE_UNDEFINED, url: String, state: dojo.dom.DOMDocument}
     *     object. 
     *     
     * url: String - String representing the URL the state DOM should be retrieved from.
     *     Must not be NULL.
     * callbackfn: function - The callback function which should be called in case of
     *     internal status changes. May be NULL to indcate that the method should execute
     *     synchronously.
     */
    deserialize: function(/*String*/ url, /*Function*/ callbackfn) {
        var payload = this.STATE_URI_SCHEME + ":" + url;
        var result = null;
        var mType = this._getMimeType();
        var me = this;
        var digest = com.ibm.portal.services.PortalRestServiceConfig.digest;
        ibm.portal.debug.text( "Digest: " + digest );
        var params;
        if (digest != null) {
            params = {"uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "digest" : digest};
        } else {
            params = {"uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI};
        }
        dojo.xhrGet({
             url: this.serviceURL,
             sync: (callbackfn)?false:true,
             content: params,
             handleAs: mType,
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if (type == "load") {
                      var stateDOM = me._getResponseXML(response);
                      //If we get a parser error, then valid XML was not returned by 
                      //the serialization service and we want to return an empty xml
                      //document in this case.
                      if ( stateDOM.documentElement.nodeName == "parsererror" ) {
                          stateDOM = dojox.data.dom.createDocument();
                      }
                      if (callbackfn) {
                          // TODO use this.STATUS_OK here
                          callbackfn(1, url, stateDOM);
                      } else {
                          // TODO use this.STATUS_OK here
                          result = {"status" : 1, "input" : me.serviceURL, "url" : me.serviceURL, "returnObject" : stateDOM, "state" : stateDOM};
                      }
                  } else if (type == "error") {
                      if (callbackfn) {
                          // TODO use this.STATUS_ERROR here
                          callbackfn(2, url, null);
                      } else {
                          // TODO use this.STATUS_ERROR here
                          result = {"status" : 2, "input" : me.serviceURL, "url" : me.serviceURL, "returnObject" : null, "state" : null};
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return result;
    },
    

    _handleSerializationResponse: function(response, callbackfn, state, validate) {
        var result = null;
        var type = ( response instanceof Error ) ? "error" : "load";
        if (type == "load") {
            var responseXML = this._getResponseXML(response);
            // get the serialized state from response XML
            var linkExpr = "atom:entry/atom:link";
            var ns = {"atom" : "http://www.w3.org/2005/Atom", "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state"};
            var serializedState = null;
            var linkNodes = com.ibm.portal.xpath.evaluateXPath(linkExpr, responseXML, ns);
            if (linkNodes != null && linkNodes.length > 0) {
                // take the fist node found and read its href attribute
                serializedState = linkNodes[0].getAttribute("href");
            }
            var resultState = state;
            // check if the state potentially changed
            if (validate == true) {
                // get the new state from the response XML
                var stateExpr = "atom:entry/atom:content/state:root";
                var stateNodes = com.ibm.portal.xpath.evaluateXPath(stateExpr, responseXML, ns);
                if (stateNodes != null && stateNodes.length > 0) {
                    var newState = dojox.data.dom.innerXML(stateNodes[0]); 
                    resultState = dojox.data.dom.createDocument(newState);
                }
            }
            if (callbackfn) {
                // TODO use this.STATUS_OK here
                callbackfn(1, resultState, serializedState);
            } else {
                // TODO use this.STATUS_OK here
                result = {"status" : 1, "input" : resultState, "state" : resultState, "returnObject" : serializedState, "url" : serializedState};
            }
        } else if (type == "error") {                      
            if (callbackfn) {
                // TODO use this.STATUS_ERROR here
                callbackfn(this.STATUS_ERROR, state, null);
            } else {
                // TODO use this.STATUS_ERROR here
                result = {"status" : this.STATUS_ERROR, "input" : state, "state" : state, "returnObject" : null, "url" : null};
            }
        }
        return result;
    },


    _getMimeType: function() {
        var mtype = "xml";
        // IE does not work with "text/xml"
        if (dojo.isIE) {
             mtype = "text";
        }
        return mtype;
    },
    
    _getResponseXML: function(data) {
        var responseDOM = data;
        if (dojo.isIE) {
            responseDOM = dojox.data.dom.createDocument(data);
        }
        return responseDOM;
    },
	_encodeAscii: function(str) {
		var ret = str;
		if(dojo.isString(ret)) {
			var escapedStr = escape(ret);
			// \u#### patterns will be replaced with %u#### patterns by the escape() function
			var unicodeRegx = /%u([A-F0-9][A-F0-9][A-F0-9][A-F0-9])/i;
			var unicodeVal = null;
			// now find all %u#### patterns and replace them with an 
			while((unicodeVal = escapedStr.match(unicodeRegx))) {
				ret += escapedStr.substring(0, unicodeVal.index) + escape(Number("0x"+unicodeVal[1]));
				escapedStr = escapedStr.substring(unicodeVal.index + unicodeVal[0].length);
			}
			ret += escapedStr;
			ret = ret.replace(/\+/g, "%2B");
		}
		return ret;
	}	
    
    
});

/**
 * Component that manages state vary expressions on an ID basis.
 */
dojo.declare("com.ibm.portal.navigation.controller.StateVaryManager", null, {
    
				/**
				 * summary:
				 *     Constructor which initializes the state vary manager with a URL
				 */
				constructor: function() {
					this._expr = new Array();
				},

				setExpressions: function (/*String*/ id, /*Array*/ expressions) {
					var bucket = this._findBucket(id);
					if (bucket == null) {
						// create a new empty bucket
						bucket = { "id": id, "expr": null };
						// update the cache
						this._expr.push(bucket);
					}
					// add /replace the expressions
					bucket.expr = expressions;
				},

				getExpressions: function(/*String*/ id) {
					var result = null;
					var bucket = this._findBucket(id);
					if (bucket != null) {
						result = bucket.expr;
					}
					return result;
				},

				_findBucket: function(/*String*/ id) {
					var bucket = null;
					for (i=0; i<this._expr.length; i++) {
						var temp = this._expr[i];
						if (temp.id == id) {
							bucket = temp;
							break;
						}
					}
			      	return bucket;
				}
    
});

com.ibm.portal.state.STATE_MANAGER = new com.ibm.portal.state.StateManager(); 	
com.ibm.portal.state.STATE_MANAGER.reset(dojox.data.dom.createDocument()); // workaround


}

dojo.provide( "com.ibm.portal.debug" );

//To satisfy dojo.require
//TODO: convert all ibm.portal.debug statements to com.ibm.portal.debug stmts
dojo.provide( "ibm.portal.debug" );

ibm.portal.debug.text = function( str, functionName ) {
	window.console.log(str);
} 

ibm.portal.debug.entry = function ( functionName, args ) {
	var debugStr = functionName + " --> entry; { ";
	
	if ( args && args.length > 0 )
	{
		for ( arg in args ) {
			debugStr = debugStr + args[arg] + " ";
		}
	} 
	
	debugStr = debugStr + " } ";
	
	ibm.portal.debug.text( debugStr, functionName );
}

ibm.portal.debug.exit = function( functionName, retValue ) {
	var debugStr = functionName + " --> exit;"
	if ( typeof( retValue ) != "undefined" )
	{
		debugStr = debugStr +" { " + retValue + " } ";
	}		
	ibm.portal.debug.text( debugStr, functionName );		
}

ibm.portal.debug.escapeXmlForHTMLDisplay = function( string ) {
	string = string.replace( /</g, "&lt;" );
	string = string.replace( />/g, "&gt;" );
	
	return string;
}

dojo.provide( "com.ibm.portal.EventBroker" );

dojo.require( "com.ibm.portal.debug" );

dojo.declare( "com.ibm.portal.Event",
			  null,
			  {
			  	constructor: function ( /*String*/uniqueEventId ) {
				  	this.eventName = uniqueEventId;
				  	this._listeners = new Array();
				},
			  	fire: function ( /*Object?*/ parameters ) {
					// summary: Fires the event to the listeners.
					// parameters: any parameters that are to be passed to the listeners
					ibm.portal.debug.text( "Firing event: " + this.eventName + " with parameters: " );
					////dojo.debugShallow( parameters );
					//dojo.event.topic.publish( this.eventName, parameters );
					dojo.publish( this.eventName, [parameters] );
				},
				register: function ( /*Function or Object*/listener, /*Function or String*/listenerFunc ) {
					// summary: Registers a listener with an event.
					// listener: the listener function to be called when an event is fired.
					
					if ( !listenerFunc ) {
						//dojo.event.topic.subscribe( this.eventName, listener );
						return dojo.subscribe( this.eventName, null, listener ); // Handle
					}
					else {
						//dojo.event.topic.subscribe( this.eventName, listener, listenerFunc );
						return dojo.subscribe( this.eventName, listener, listenerFunc ); // Handle
					}	
				},
				unregister: function( /*Handle*/handle ) { 
					// summary: Unregisters a listener from an event. The parameters must match
					//		exactly.
					//dojo.event.topic.unsubscribe( this.eventName, listener, listenerFunc );
					dojo.unsubscribe( handle );
				},
				cancel: function ( /*Object?*/ parameters ) { 
					// summary: Cancels an event.
					// parameters: any parameters to be passed to the listeners
					dojo.publish( this.id + "/cancel" );	
				} 
			  }
);

dojo.declare( "com.ibm.portal.EventBroker",
			  null,
			  {
			  	// summary: All events should be instance variables of an Event.	
				
				//required events
				startPage: new com.ibm.portal.Event( "portal/StartPage" ),
				endPage: new com.ibm.portal.Event( "portal/EndPage" ),
				startFragment: new com.ibm.portal.Event( "portal/StartFragment" ),
				endFragment: new com.ibm.portal.Event( "portal/EndFragment" ),
				fragmentUpdated: new com.ibm.portal.Event( "portal/FragmentUpdated" ),
				startRequest: new com.ibm.portal.Event( "portal/StartRequest" ),
				endRequest: new com.ibm.portal.Event( "portal/EndRequest" ),
				cancelAll: new com.ibm.portal.Event( "portal/CancelAll" ),
				stateChanged: new com.ibm.portal.Event( "portal/StateChanged" ),
				startScriptHandling: new com.ibm.portal.Event( "portal/StartScriptHandling" ),
				endScriptHandling: new com.ibm.portal.Event( "portal/EndScriptHandling" ),
                javascriptCleanup: new com.ibm.portal.Event( "portal/JavascriptCleanup" ),
                beforeSnapShot: new com.ibm.portal.Event( "portal/BeforeSnapShot" ),
                afterSnapShot: new com.ibm.portal.Event( "portal/AfterSnapShot" ),
                restorePointUpdated: new com.ibm.portal.Event( "portal/RestorePointUpdated" ),
                clearRestorePoint: new com.ibm.portal.Event( "portal/ClearRestorePoint" ),
				stopEvent: new com.ibm.portal.Event( "portal/StopEvent" ),
				redirect: new com.ibm.portal.Event( "portal/Redirect" )
			  }
);

com.ibm.portal.EVENT_BROKER = new com.ibm.portal.EventBroker();

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.dnd._copyKey = navigator.appVersion.indexOf("Macintosh") < 0 ? "ctrlKey" : "metaKey";

dojo.dnd.getCopyKeyState = function(e) {
	// summary: abstracts away the difference between selection on Mac and PC,
	//	and returns the state of the "copy" key to be pressed.
	// e: Event: mouse event
	return e[dojo.dnd._copyKey];	// Boolean
};

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary: returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary: returns true, if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

dojo.provide( "com.ibm.portal.utilities" );

com.ibm.portal.utilities = {
	findPortletIdByElement: function ( /*DOMElement*/ element ) {
		// summary: Finds the portlet ID which the given element belongs to
		// description: Finds the portlet ID to which the give DOM element belongs
		//		by walking up the parents of the given element until the ancestor is 
		//		found with one of the defined class names.
		// element: the DOM element to find the portlet ID of
		
		ibm.portal.debug.entry( "findPortletID", [ element ] );
		var id = "";
		var current = element.parentNode;
		while ( current && id.length == 0 ) {
			ibm.portal.debug.text( "examining element " + current.tagName + "; class=" + current.className,"findPortletID" );
		
			if ( current.className && (current.className.match(/\bwpsPortletBody\b/) || current.className.match(/\bwpsPortletBodyInlineMode\b/)) ) {
				id = current.id;
				var indexOf = id.indexOf( "_mode" );
				if ( indexOf >= 0 )  {
					id = id.substring( 0, indexOf );
				}
			}
			current = current.parentNode;	
		}
		
		if ( id.indexOf( "portletActions_" ) >= 0 ) {
			id = id.substring( "portletActions_".length );
		}
		
		ibm.portal.debug.exit( "findPortletID", [ id ] );
		
		return id;
	},
	findFormByElement: function ( /*DOMElement*/ element) {
		var current = element;
		while(current) {
			if(current.tagName && current.tagName.toLowerCase() == "form") break;
			current = current.parentNode;
		}
		return current;
	},
	encodeURI: function ( /*String*/uri ) {
		// summary: Encodes a model uri for use in an URL.
		// uri: the unencoded model uri to encode
		// returns: the encoded uri ready for use in an URL
		ibm.portal.debug.entry( "encodeURI", [ uri ] );
		
		var encodedURI = uri;
		var index = uri.lastIndexOf( ":" );
		
		while ( index >= 0 ) {
			var scheme = encodedURI.substring(0, index );
			var part = encodedURI.substring( index + 1  );
			
			encodedURI = scheme + ":" + encodeURIComponent( part );
			index = scheme.lastIndexOf( ":" );
		}
		
		encodedURI = encodeURIComponent( encodedURI );
		
		ibm.portal.debug.exit( "encodeURI", [ encodedURI ] );
		return encodedURI; // String
	},
	decodeURI: function ( /*String*/uri ) { 
		// summary: Decodes a model uri.
		// uri: the encode model uri
		// returns: the decoded model uri
		ibm.portal.debug.entry( "decodeURI", [ uri ] );
		
		var decodedURI = decodeURIComponent( uri );
		var index = decodedURI.indexOf( ":" );
		
		while ( index >= 0 ) {
			var scheme = decodedURI.substring(0, index );
			var part = decodedURI.substring( index + 1 );
			
			decodedURI = scheme + ":" + decodeURIComponent( part );
			index = decodedURI.indexOf( ":", index+1 );
		}
		
		ibm.portal.debug.exit( "decodeURI", [ decodedURI ] );
		return decodedURI;	//String
	},
	getSelectionNodeId: function ( /*String*/fullPortletId ) {
		// summary: Extracts the selection node object id from the full portlet id returned
		//		by the layout model feed.
		ibm.portal.debug.entry( "getSelectionNodeId", [ fullPortletId ] );
		
		// this split will break the full portlet id into the control with prefixes at index [0]
		// and the selected node will be at index [1]
		var selNodeId = fullPortletId.split('@oid:');
				
		ibm.portal.debug.exit( "getSelectionNodeId", [ selNodeId[1] ] );
		return selNodeId[1];
	},
	getControlId: function ( /*String*/fullPortletId ) {
		// summary: Extracts the control object id from the full portlet id returned
		//		by the feed.
		ibm.portal.debug.entry( "_getControlId", [ fullPortletId ] );
		
		// this first split will get rid of the page oid
		var controlParts = fullPortletId.split('@oid:');
		
		// this second split will leave the feed prefix (i.e. lm) at index [0]
		// and index [1] will have the control oid
		var controlId = controlParts[0].split("oid:");
		
		ibm.portal.debug.exit( "getControlId", [ controlId[1] ] );
		return controlId[1];
	},
	overwriteProperty: function ( /*Object*/obj, /*String*/propertyName, /*Object*/newValue, /*Boolean?*/onlyOnce ) {
		// summary: Overwrites the given property.
		ibm.portal.debug.entry( "overwriteProperty", [ obj, propertyName, newValue, onlyOnce ] );
		
		if ( !obj["_overwritten_"] ) {
			obj["_overwritten_"] = new Object();
		}
		
		//In case onlyOnce is undefined.
		if ( !onlyOnce ) { onlyOnce = false; }
		
		var skipOverwrite = (onlyOnce &&  (obj["_overwritten_"][propertyName] != null) );
		if ( !skipOverwrite ) {
			if ( obj["_overwritten_"][propertyName] == null ) {
                obj["_overwritten_"][propertyName] = obj[propertyName];
            }
            else {
                obj["_overwritten_"][propertyName] = null;
            }    
			obj[propertyName] = newValue;
			ibm.portal.debug.text( "Property overwrite successful!" );
		}
		
		ibm.portal.debug.exit( "overwriteProperty");	
	},
	restoreProperty: function ( /*Object*/obj, /*String*/propertyName ) {
		// summary: Restores an overwritten property.
        // returns: the overwritten value for the property
		ibm.portal.debug.entry( "utilities.restoreProperty", [ obj, propertyName ] );
		var retVal = obj[propertyName];

        if ( obj["_overwritten_"] != null ) {
            ibm.portal.debug.text( "overwritten property value: " + obj["_overwritten_"] ); 
            obj[propertyName] = obj["_overwritten_"][propertyName];
            obj["_overwritten_"][propertyName] = null;
        }
        else {
            obj[propertyName] = null;
        }
        
        ibm.portal.debug.exit( "utilities.restoreProperty", retVal );    
        return retVal;
	},
	getOverwrittenProperty: function ( /*Object*/obj, /*String*/propertyName ) {
		// summary: Access an overwritten property.
		if ( obj["_overwritten_"] ) {
			return obj["_overwritten_"][propertyName];
		}
		else {
			return null;
		}	
	},
	setOverwrittenProperty: function ( /*Object*/obj, /*String*/propertyName, /*Object*/value ) {
		// summary: Sets the overwritten property directly; bypassing the overwriting process.
		ibm.portal.debug.entry( "utilities.setOverwrittenProperty", [ obj, propertyName, value ] ); 
		
		if ( !obj["_overwritten_"] ) {
			obj["_overwritten_"] = new Object();
		}
		
		obj["_overwritten_"][propertyName] = value;
		
		ibm.portal.debug.exit( "utilities.setOverwrittenProperty" );
	},
	callOverwrittenFunction: function ( /*Object*/thisObj, /*String*/propertyName, /*Object[]?*/args ) {
		ibm.portal.debug.entry( "utilities.callOverwrittenFunction", [ thisObj, propertyName, args ]);
		//IE doesn't like it if we pass in an undefined args object to apply.
		var retVal = null;
		var oldProp = this.getOverwrittenProperty( thisObj, propertyName );
		ibm.portal.debug.text( "Overwritten property: " + oldProp );
		ibm.portal.debug.text( "old property's apply function: " + oldProp.apply );
		
		if ( args ) {
			retVal = oldProp.apply( thisObj, args );
		}
		else {
			retVal = oldProp.apply( thisObj );
		}
		
		ibm.portal.debug.exit( "utilities.callOverwrittenFunction", retVal );
		return retVal;
	},
	isExternalUrl: function ( /*String*/urlString ) {
		// summary: Determines if the given url points to an external server.
		// urlString: the url in string form
		// returns: true if the url points to an external server
		ibm.portal.debug.entry( "isExternalUrl", [ urlString ] );
		
		var host = window.location.host;
		var protocol = window.location.protocol;
		var urlStringNoQuery = urlString.split( "?" )[0];
		var retVal = !(urlStringNoQuery.indexOf("://") < 0 || (urlStringNoQuery.indexOf(protocol) == 0 && urlStringNoQuery.indexOf(host) == protocol.length+2) );
		
		ibm.portal.debug.text( "urlStringNoQuery.indexOf(\"://\") = " + urlStringNoQuery.indexOf("://") );
		ibm.portal.debug.text( "urlStringNoQuery.indexOf(protocol) = " + urlStringNoQuery.indexOf(protocol) );
		
		ibm.portal.debug.exit( "isExternalUrl", retVal );
		return retVal;
	},
	isJavascriptUrl: function ( /*String*/urlString ) {
		// summary: Determines if the given url is a javascript url.
		// description: Checks to see if the url begins with the javascript: protocol
		// urlString: the url in string form
		// returns: true if the url is actually a javascript call.
		ibm.portal.debug.entry( "isJavascriptUrl", [ urlString ] );
		var url = com.ibm.portal.utilities.string.trim( urlString.toLowerCase() );
		var retVal = ( url.indexOf( "javascript:" ) == 0 );
		ibm.portal.debug.exit( "isJavascriptUrl", retVal );
		return retVal;
	},
	isPortalUrl: function ( /*String*/urlString ) {
		// summary: Determines if a given url is a url to the portal servlet.
		// description: Checks to see if the url begins with the path to the portal servlet.
		// urlString: the url to check in string form
		// returns: true if the url is a portal url
		ibm.portal.debug.entry( "utilities.isPortalUrl", [ urlString ] );
		
		var retVal = ( urlString.indexOf( ibmPortalConfig["portalURI"] ) >= 0 );
		
		ibm.portal.debug.exit( "utilities.isPortalUrl", retVal );
		return retVal;
	},	

	addExternalNode: function ( /*DOMDocument*/ doc, /*Element*/ node ) {
		var newNode = null;
		if(doc.importNode) newNode = doc.importNode(node, true);
		else newNode = node;
		doc.appendChild( newNode );
	},
    decodeXML: function ( /*String*/xmlText ) {
		// summary: decodes xml sent by multi-part form 
		ibm.portal.debug.entry( "decodeXML", [ xmlText ] );

		var newXmlText1 = xmlText.replace(/&amp;/g, "&");
		var newXmlText2 = newXmlText1.replace(/&amp;/g, "&");
		newXmlText1 = newXmlText2.replace(/&#039;/g, "\'");
		newXmlText2 = newXmlText1.replace(/&#034;/g, '\"');
		
		newXmlText2 = newXmlText2.replace( /&lt;/g, "<" );
		newXmlText2 = newXmlText2.replace( /&gt;/g, ">" );

		ibm.portal.debug.exit( "decodeXML", [ newXmlText2 ] );
		return newXmlText2;
	},
	eventHandlerToString: function ( eventHandler ) {
		var eventHandlerStr = eventHandler.toString();
		var startIndex = eventHandlerStr.indexOf("{");
		var endIndex = eventHandlerStr.lastIndexOf("}");
		onclickStr = eventHandlerStr.substring(startIndex+1, endIndex);
		
		return onclickStr;
	},
	_waitingForScript: false,
	_isWaitingForScript: function() { return com.ibm.portal.utilities._waitingForScript; },
	stopWaitingForScript: function() { com.ibm.portal.utilities._waitingForScript = false; },

	waitFor: function(testFunction, pollInterval, callback, args) {
		var interval = setInterval(function() {
			if (testFunction())
			{
				clearInterval(interval);
				
				if ( !args ) {
					callback();
				}
				else {
					callback(args);
				}
			}
		},
		pollInterval);
	},

	waitForScript: function( callback, args ) {
		com.ibm.portal.utilities._waitingForScript = true;
		com.ibm.portal.utilities.waitFor(function()
		{
			return (!com.ibm.portal.utilities._isWaitingForScript());
		},
		500,
		callback,
		args);
	}
}

com.ibm.portal.utilities.string = {
	findNext: function ( /*String*/ string, /*Array*/searchStrings, /*int?*/from ) {
		// summary: Finds the next index of any of the given search strings.
		// description: Looks for each search string in the given string and returns the index 
		//		of the first one encountered. If none of the search strings are discovered, 
		//		-1 is returned. 
		// string: the string to search
		// searchStrings: the array of strings to look for in the string
		// from: the index to start looking from - the length of the search string is always added 
		//		from index before searching for the next instance
		ibm.portal.debug.entry( "string.findNext", [ string, searchStrings ] );
		
		var lowValueNotZero = -1;
		
		for ( var i = 0; i < searchStrings.length; i++ ) {
			var fromIndex = null;
			if ( from ) {
				fromIndex = from + searchStrings[i].length;
			}
			var index = string.indexOf( searchStrings[i], fromIndex );
				
			if ( index > -1 && ( index < lowValueNotZero || lowValueNotZero == -1 ) ) {
				lowValueNotZero = index;
			}
		}
		
		ibm.portal.debug.exit( "string.findNext", [ lowValueNotZero ] );
		return lowValueNotZero;		
	},
	contains: function ( /*String*/ stringToSearch, /*String*/stringToLookFor ) {
		// summary: Determines if the stringToSearch contains the stringToLookFor
		ibm.portal.debug.entry( "string.contains", [ stringToSearch, stringToLookFor ] );
		
		var retVal = false;
		
		if ( stringToSearch != null && stringToLookFor != null ) {
			retVal = ( stringToSearch.indexOf( stringToLookFor ) != -1 );		
		}
		
		ibm.portal.debug.exit( "string.contains", [ retVal ] );
		return retVal;
	},
	strip: function ( /*String*/string, /*String*/stringToStrip ) {
		// summary: Strips out the given string from the original string if it contains
		//		the stringToStrip.
		// string: the original string
		// stringToStrip: the string to remove from the original string
		// returns: the original string without any instances of the stringToStrip
		ibm.portal.debug.entry( "string.strip", [ string, stringToStrip ] );
		
		var retVal = string.replace( new RegExp( stringToStrip, "g" ), "" );
				
		ibm.portal.debug.exit ( "string.strip", [ retVal ]);
		return retVal;
	},
	properCase: function ( /*String*/string ) {
		// summary: Changes the first character of the string to Upper Case and all remaining characters
		//		to Lower Case.  Note that this does not split the string into words and capitalize
		//		each word, but only acts on the entire string as a whole.
		// string: The original string to change
		// returns: The original string with the first character in Upper Case and all other characters
		//		in Lower Case.
		if(string == null || string.length < 1) return "";
		ibm.portal.debug.entry("string.properCase", [string] );
		var retVal = string.charAt(0).toUpperCase();
		if(string.length > 1) {
			retVal += string.substring(1).toLowerCase();
		}
		ibm.portal.debug.exit("string.properCase", [retVal] );
		return retVal;
	},
	trim: function ( /*String*/ string ) {
		// summary: Removes leading and trailing spaces from a string.
		// string: the string to trim
		// returns: the original string minus any leading or trailing spaces
		ibm.portal.debug.entry( "string.trim", [ string ] );
		var retVal = string;
		
		retVal = retVal.replace( /^\s+/, '' );
		retVal = retVal.replace( /\s+$/, '' );
		
		ibm.portal.debug.exit( "string.trim", retVal );
		return retVal;
	}
}

dojo.declare( "com.ibm.portal.utilities.HttpUrl", 
			  null,
			  {
			  	constructor: function ( /*String*/urlString ) {
					this.scheme = "http://";
				  	this.server = this._extractServer( urlString );
					this.port = this._extractPort( urlString );
					this.path = this._extractPath( urlString );
					this.query = this._extractQuery( urlString );
					this.anchor = "";
				},
			  	addParameter: function ( /*String*/name, /*String*/value ) {
					this.query += "&" + name + "=" + value;
				},
				toString: function () {
					var str = "";
					if ( this.server != "" ) {
						str += this.scheme + this.server;
					} 
					if ( this.port != "" ) {
						str += ":" + this.port;
					}
					
					str += "/" + this.path;
					
					if ( this.query != "" ) {
						str += "?" + this.query;
					}
					if ( this.anchor != "" ) {
						str += "#" + this.anchor;
					}
					
					return str;
				},
				_extractServer: function (/*String*/urlString ) {
					var indexOfScheme = urlString.indexOf( this.scheme );
					var retVal = "";
					if ( indexOfScheme == 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfScheme + this.scheme.length );
						var serverAndPort = urlString.substring( indexOfScheme+ this.scheme.length, indexOfSlash );
						retVal = serverAndPort.split( ":" )[0];
					}	
					
					return retVal; 
				},
				_extractPort: function (/*String*/urlString ) {
					var indexOfServer = urlString.indexOf( this.server );
					var retVal = "";
					if ( indexOfServer >= 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfServer );
						var serverAndPort = urlString.substring( indexOfServer, indexOfSlash );
						var serverAndPortParts = serverAndPort.split( ":" );
						if ( serverAndPortParts.length > 1 ) {
							retVal = serverAndPortParts[1];
						}
					}	
					
					return retVal; 
				},
				_extractPath: function ( /*String*/urlString ) {
					var indexOfServer = urlString.indexOf( this.server );
					var retVal = "";
					if ( indexOfServer >= 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfServer );
						var indexOfQuery = urlString.indexOf( "?" );
						var indexOfHash = urlString.lastIndexOf( "#" );
						if ( indexOfQuery >= 0 ) {
							retVal = urlString.substring( indexOfSlash + 1, indexOfQuery );
						}
						else {
							if ( indexOfHash >= 0 ) {
								retVal = urlString.substring( indexOfSlash + 1, indexOfHash );
							}
							else {
								retVal = urlString.substring( indexOfSlash + 1 );
							}		
						}
					}	
					
					return retVal;
				},
				_extractQuery: function ( /*String*/urlString ) {
					var retVal = "";
					var urlParts = urlString.split( "?" );
					
					if ( urlParts.length > 1 ) {
						retVal = urlParts[1].split( "#" )[0];
					}	
					
					return retVal;
				},
				_extractAnchor: function ( /*String*/urlString ) {
					var retVal = "";
					var urlParts = urlString.split( "#" );
					
					if ( urlParts.length > 1 ) {
						retVal = urlParts[urlParts.length - 1];
					}	
					
					return retVal;
				}
					
			  }
);

dojo.provide( "com.ibm.portal.utilities.html" );

dojo.require( "com.ibm.portal.utilities" );

com.ibm.portal.utilities.html = {
	createAnchor: function ( /*HTMLElement*/parent, /*String*/href, 
									     /*String*/id, /*String*/displayText, 
										 /*String*/className ) {
					
		// summary: Creates an anchor element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created anchor
		// href: the value of the href attribute
		// id: the id of the anchor element
		// displayText: the inner text of the anchor
		// className: the CSS style name to apply
		// returns: the created anchor object
		ibm.portal.debug.entry( "SkinRenderer.createAnchor", [ parent, href, id, displayText, className ] );
		
		//Create and set the attributes for the anchor.
		var anchor = document.createElement( "A" );
		anchor.href = href;
		if ( id ) { anchor.id = id; }
		if ( className ) { anchor.className = className; }
		if ( displayText ) { anchor.appendChild( document.createTextNode( displayText ) ); }
												 	
		//Append the anchor to the parent.
		parent.appendChild( anchor );
		
		ibm.portal.debug.exit( "SkinRenderer.createAnchor", [ anchor ]);
		return anchor; //HTMLElement																
	},
	createButton: function ( /*HTMLElement*/parent, /*String*/href, 
						     /*String*/id, /*String?*/displayText, 
							 /*String*/className ) {
		
		// summary: Creates a button element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created anchor
		// href: the value of the href attribute
		// id: the id of the anchor element
		// displayText: the inner text of the anchor
		// className: the CSS style name to apply
		// returns: the created anchor object
		ibm.portal.debug.entry( "SkinRenderer.createButton", [ parent, href, id, displayText, className ] );
		
		//Create and set the attributes for the anchor.
		var button = document.createElement( "BUTTON" );
		if ( href ) { button.href = href; }
		if ( id ) { button.id = id; }
		if ( className ) { button.className = className; }
		if ( displayText ) {
			button.appendChild( document.createTextNode( displayText ) );
		}	
												 	
		//Append the anchor to the parent.
		parent.appendChild( button );
		
		ibm.portal.debug.exit( "SkinRenderer.createButton", [ button ]);
		return button; //HTMLElement																
	},
	createImage: function ( /*HTMLElement*/parent, /*String*/src,
							/*String*/id, /*String*/altText,
							/*String*/className ) {
								
		// summary: Creates an img element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created image
		// src: the url to the image
		// id: the id of the created image element
		// altText: the alternate text for the image
		// className: the CSS style name to apply.
		ibm.portal.debug.entry( "SkinRenderer.createImage", [ parent, src, id, altText, className ] );
		
		//Create and set attributes for our image.
		var img = document.createElement( "IMG" );
		img.src = src;
		if ( id ) { img.id = id; }
		if ( altText ) { 
			img.alt = altText; img.setAttribute("title", altText); 
			//set the title for the button ("tooltip")
			if( parent.nodeName == "BUTTON") {
				parent.setAttribute("title", altText);
			}
		}
		if ( className ) { img.className = className; }
		
		//Append the image to the parent.
		parent.appendChild( img );
		
		ibm.portal.debug.exit( "SkinRenderer.createImage", [ img ] );
		return img; //HTMLElement	
	},
	createImageAnchor: function ( /*HTMLElement*/parent, /*String*/src,
							/*String*/id, /*String*/altText,
							/*String*/className ) {
								
		// summary: Creates an img element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created image
		// src: the url to the image
		// id: the id of the created image element
		// altText: the alternate text for the image
		// className: the CSS style name to apply.
		ibm.portal.debug.entry( "SkinRenderer.createImageAnchor", [ parent, src, id, altText, className ] );
		
		var anchor = com.ibm.portal.utilities.html.createAnchor( parent, "javascript:void(0);" );
		
		//Create and set attributes for our image.
		var img = document.createElement( "IMG" );
		img.src = src;
		if ( id ) { img.id = id; }
		if ( altText ) { img.alt = altText; img.title = altText; }
		if ( className ) { img.className = className; }
		
		//Append the image to the parent.
		anchor.appendChild( img );
		
		ibm.portal.debug.exit( "SkinRenderer.createImageAnchor", [ img ] );
		return anchor; //HTMLElement	
	},
	createTemporaryMarkupDiv: function ( /*String*/ markupStr ) {
		// summary: Creates a div for putting our markup into.  Returns a map with these properties:
		//		{ 	
		// 			node: the new div node,
		//			objects: a map of the extracted OBJECTS keyed by the unique id of the placeholder node in the div node
		//		}
		// description: Necessary because IE doesn't like it when you set innerHTML on a div and the first
		//		element is a script element. For some reason, any script elements up until the first actual
		//		markup element will not be inserted into the DOM. Our workaround is to create a hidden <p>
		//		element which will always be the first element in the markup string.  In IE, this will also
		//		extract all OBJECT elements and put them into the objects property map on the return object
		//		to later be put back into the DOM once the div in the node property of the return object
		//		has been placed into the DOM.  The works around an issue where using innerHTML or outerHTML
		//		of OBJECT elements doesn't retrieve their children in IE.
		// markupStr: the markup string.
		ibm.portal.debug.entry( "html.createTemporaryMarkupDiv" );
		
		var markupMap = {markup: markupStr, objects: {}};
		// fix for inserting OBJECT elements with contents in IE
		if(dojo.isIE) {
			markupMap = com.ibm.portal.utilities.html.extractObjectElementsFromString(markupStr);
		}
		
		var div = document.createElement( "DIV" );
		div.innerHTML = "<p style='display: none;'>&nbsp;</p>" + markupMap.markup;
		
		ibm.portal.debug.exit( "html.createTemporaryMarkupDiv", [ div ] );
		return {node: div, objects: markupMap.objects};
	},
	extractObjectElementsFromString: function ( /*String*/ markupStr ) {
		// summary: Extracts all OBJECT elements from the markupStr and returns an object with the following
		//		properties -
		//		{
		//			markup: the modified markup with uniquely identifiable placeholder nodes in place of the removed
		//				OBJECT elements,
		//			objects: a map of the extracted OBJECTS keyed by the unique id of the placeholder node in the new
		//				markup string
		//		}
		// markupStr: the markup string
		var objElems = {};
		var openRegExp = /<object/gi;
		var closeRegExp = /<\/object>/gi;
		var markup = markupStr;
		var match = null;
		try {
			match = openRegExp.exec(markup);
			if(match && match.index > -1) {
				var start = match.index;
				var buf;
				var end;
				var middle;
				var id;
				while(start > -1) {
					buf = markup.substring(0, start);
					end = markup.indexOf(">", start);
					if(markup.charAt(end - 1) == "/") {
						// we encountered an <OBJECT .... /> tag, so just leave it since it has no contents
						openRegExp.lastIndex = end;
						match = openRegExp.exec(markup);
						if(match) {
							start = match.index;
							continue;
						}
						else break;	// we didn't find anything after the last match
					}
					closeRegExp.lastIndex = start;
					match = closeRegExp.exec(markup);
					if(match) end = match.index;
					else break; // bad markup, OBJECT not closed, just let it go
					
					middle = markup.substring(start, end + 9);
					id = dojo.dnd.getUniqueId();
					markup = buf + "<div id='" + id + "'></div>" + markup.substring(end + 9);
					objElems[id] = middle;
					openRegExp.lastIndex = 0;	// start at the beginning since the markup string has changed
					match = openRegExp.exec(markup);
					
					if(match) start = match.index;
					else break;	// no more matches
				}
			}
			markupStr = markup;
		}
		catch(e) {
			// if we get a failure during the processing of the markup, then the markupStr will still be the same
			// that means we should clear out the objElems and just let the markup pass through as-is
			objElems = {};
		}
		return {markup: markupStr, objects: objElems};
	},
	replaceObjectElementsInMarkup: function ( /*Map*/ objects ) {
		// summary: For every key in the objects argument map, this function looks up a node in the DOM whose id
		// 		equals the key and replaces the outerHTML of that node with the string contents of the property
		//		value of the objects map with that key.  It's useful for putting markup from various nodes back 
		// 		into the DOM that was extracted earlier for some purpose.
		for(var id in objects) {
    		var objContainer = dojo.byId(id);
    		if(objContainer) {
    			objContainer.outerHTML = objects[id];
    		}
    	}
	},
	removeNodesOnCondition: function ( /*DOMElement*/node, /*Function*/condition ) {
		// summary: Walks down this node's descendants and removes all nodes that cause the given
		// condition function to return true, effectively satisfying the condition for removal.
		// Useful for delinting XSL transformations that have created unwanted nodes in the markup.
		if(!condition) condition = function(){return false;};
		if(node && node.childNodes) {
			for(var i = 0; i < node.childNodes.length; i++) {
				if(condition(node.childNodes[i])) {
					var nonNode = node.childNodes[i];
					node.removeChild(nonNode);
					delete nonNode;
					i--; // going back because the childNodes array has been updated
				}
				else {
					this.removeNodesOnCondition(node.childNodes[i], condition);
				}
			}
		}
	},
    getElementsByTagNames: function ( /*HTMLElement*/baseElem /*String 1...N*/ ) {
        // summary: Retrieves elements by tag name for 1 to N tag names.
        // description: Builds a single array with all elements of any tag name specified. A base element
        //        must be specified event if it's just the document object. Any number of tag names can be
        //        searched for, just simply keep appending parameters to the function call.
        // baseElem: the base element to search from
        // tagName1: a tag name
        // tagName2: a tag name
        // ...
        // tagNameN: a tag name
        ibm.portal.debug.entry( "html.getElementsByTagNames", [ baseElem ] );
        
        var retVal = new Array();
        
        for ( var i=1; i < arguments.length; i++ ) {
            var elements = baseElem.getElementsByTagName( arguments[i] );
            ibm.portal.debug.text( 'found ' + elements.length + ' ' + arguments[i] + ' tags.');
            for ( var j=0; j < elements.length; j++ ) {
                retVal.push( elements[j] );
            }    
        }
        
        ibm.portal.debug.exit( "html.getElementsByTagNames", [ retVal ] );
        return retVal;
    },
    getX: function ( /*HTMLElement*/elem ) {
    	// summary: Calculates the X coordinate of the top left corner of the given element. This
    	//		function was stolen from WCL and included here so that we have no dependency on 
    	//		WCL.
    	// elem: the html element to calculate the position of.
    	// returns: an integer with the X position.
    	ibm.portal.debug.entry( "html.getX", [ elem ] );
    	
    	var size = 0;
	    
	    if ( elem != null ) {
	       if ( elem.offsetParent != null ) {
	          size += com.ibm.portal.utilities.html.getX( elem.offsetParent );
	       }
	       if ( elem != null ) {
	          size += elem.offsetLeft;
	       }
	
	    }
	    
	    ibm.portal.debug.exit( "html.getX", [ size ] );
	    return size; // int
    },
    getY: function ( /*HTMLElement*/elem ) {
    	// summary: Calculates the Y coordinate of the top left corner of the given element. This
    	//		function was stolen from WCL and included here so that we have no dependency on 
    	//		WCL.
    	// elem: the html element to calculate the position of.
    	// returns: an integer with the Y position
    	ibm.portal.debug.entry( "html.getY" [ elem ] );
    	
    	var size = 0;

	    if ( elem != null ) {

	       if ( elem.offsetParent != null ) {
	          size += com.ibm.portal.utilities.html.getY( elem.offsetParent );
	       }
	       if ( elem != null ) {
	          size += elem.offsetTop;
	       }
	
 	    }
 	    
 	    ibm.portal.debug.exit( "html.getY", [ size ] );
 	    return size; // int
    },
    convertFormToQuery: function ( /*HTMLElement*/formObj, /*{name:, value:}*/submitTrigger ) {
    	// summary: Converts a given form object to a query string.
    	// description: Iterates through the given form object's "elements" and attempts to
    	//		a name/value pair to the query string. All parameters are encoded using the native
    	//		javascript function "encodeURIComponent". Since a form may contain multiple submit
    	//		inputs, the submit input name which triggered the current submission may be passed 
    	// 		to this function so that the submit input which triggered the submission may be properly
    	//		encoded into the query string. 
    	// formObj: the form object to decode
    	// submitTrigger: the name and value of the input which triggered the submission.
    	// returns: the query string with all name/value pairs that were successfully decoded
		ibm.portal.debug.entry( "html.convertFormToQuery", [ formObj, submitTrigger ] );
		
		//For some reason, formObj.elements didn't seem to always return ALL input elements.
		var elements = this.getElementsByTagNames( formObj, "input", "select", "textarea", "button" );
    	var queryStr = "";
    	var querySep = "&";
    	var queryNameValueSep = "=";
		var processed = 0;
    	for ( var i = 0; i < elements.length; i++ ) {
    		var inputNV = this.convertInputToNameValuePairs( elements[i], submitTrigger );
    		for ( var k = 0; k < inputNV.length; k++ ) {
				var pair = inputNV[k];
				if ( pair.name != "" ) {
					if ( processed != 0 ) {
						queryStr += querySep;
					}	
					queryStr += encodeURIComponent( pair.name );
		    		
					for ( var j = 0; j < pair.values.length; j++ ) {
		    			if ( j == 0 ) {
		    				queryStr += ( queryNameValueSep + encodeURIComponent( pair.values[j] ) ); 
		    			}
		    			else {
		    				queryStr += ( querySep + encodeURIComponent(pair.name) + queryNameValueSep + encodeURIComponent( pair.values[j] ) );
		    			}	
		    		}
					
					processed = processed+1;
				}
			}	
   		
    	}
		
    	ibm.portal.debug.exit( "html.convertFormToQuery", queryStr );
    	return queryStr;
    },
    convertInputToNameValuePairs: function ( /*HTMLElement*/input, /*{name:,value:}*/submitTrigger ) {
    	// summary: Converts a given form element to a name value pair.
    	// description: Given an HTML INPUT element, produces an object representation of the following format:
    	//		{ name: <input name>, values: [ <input value1>, <input value2, ..., <input valueN> ] } where values is likely
    	//		an array with one value. The array is used to cover the case where a single input has multiple values. Submit
    	//		inputs are only included if no submitTrigger is specified OR the submitTrigger variable value matches the 
    	//		submit input's "name" attribute.
    	// input: the HTML INPUT element to convert
    	// submitTrigger: the input which triggered the form submission
    	// returns: a name/value pair { name: "", values: [] }, guaranteed to be not NULL
    	ibm.portal.debug.entry( "html.convertInputToNameValuePairs", [ input, submitTrigger ]);
    	
    	var type = input.type;
    	ibm.portal.debug.text( "Input type is: " + type );
		ibm.portal.debug.text( "Input name is: " + input.name );
    	var name = "";
    	var values = [];
		var retPairs = [];
    	
    	if ( !input.disabled ) {
    		    		
			switch ( type.toLowerCase() ) {
				case "text":
				case "password":
				case "hidden":
					name = input.name; values.push( input.value );
					retPairs.push( { name: name, values: values } );
					break;
				case "reset":
				case "button": 
					if ( !submitTrigger || ( input.name == submitTrigger.name && input.value == submitTrigger.value ) ) {
						name = input.name; values.push( input.value );
						retPairs.push( { name: name, values: values } );				
					}
					break;		
				case "radio":
				case "checkbox": if ( input.checked ) {
						name = input.name;
						values.push( input.value );
					}
					retPairs.push( { name: name, values: values } );
					break;
				case "image": if ( !submitTrigger || input.name == submitTrigger ) {
						name = input.name;
						if ( input.value ) {
							values.push( input.value );
							retPairs.push( { name: name, values: values } );
						}
						
						retPairs.push( { name: name + ".x", values: [ this.getX( input ) ] } );
						retPairs.push( { name: name + ".y", values: [ this.getY( input ) ] } );
					}
					break;
				case "submit": if ( !submitTrigger || ( input.name == submitTrigger.name && input.value == submitTrigger.value ) ) {
						name = input.name;
						if ( input.value ) {
							values.push( input.value );
						}
						retPairs.push( { name: name, values: values } );
					}
					break;
				case "select-one": 
				case "select-multiple": name = input.name;
					for ( var i = 0; i < input.options.length; i++ ) {
						if ( input.options[i].selected ) {
							var value = input.options[i].value ? input.options[i].value : input.options[i].text;
							values.push( value );
						}
					}
					if ( values.length != 0 ) { 
						retPairs.push( { name: name, values: values } );
					}	
					
					break;
				case "file": break;
				default: name = input.name; values.push( input.value ); retPairs.push( { name: name, values: values } );
			}
    	}	
		
		ibm.portal.debug.exit( "html.convertInputToNameValuePairs", retPairs );
		return retPairs;	
    },
		isHidden: function(node) {
    	// summary: determines whether the specified node is hidden or shown.
    	// description: this is a lightweight replacement for the obsolete dojo.html.isHidden function. It checks the element
    	//		only, not its parents, for display:none being set.
    	// input: the dom node
    	// returns: true if the node has display:none style set, false otherwise	
			return dojo.style(node, 'display') == 'none';
		},
		hide: function(node) {
    	// summary:  hides the specified node from the rendered page.
    	// description: this is a replacement for the obsolete dojo.html.hide() function. 
    	// input: the dom node to hide
			dojo.fx.wipeOut({node: node, duration: 5}).play();
		},
		show: function(node) {
    	// summary:  shows the specified node from the rendered page.
    	// description: this is a replacement for the obsolete dojo.html.show) function. 
    	// input: the dom node to show
			dojo.fx.wipeIn({node: node, duration: 5}).play();
		},
		isDescendantOf: function(node, ref) {
			//summary: Checks to see if the given node descends from the reference node.
			//node: node to check
			//ref: the reference node
			// returns: true if the node is a descendant of the given reference node
			var node = node.parentNode;
			var found = false;
			
			while ( node && !found ) {
				if ( node == ref ) {
					found = true;
				}
				node = node.parentNode;
			}
			
			return found;
		}
	
};

dojo.provide( "com.ibm.portal.services.PortalRestServiceRequest" );

dojo.require( "com.ibm.portal.EventBroker" );


dojo.declare( "com.ibm.portal.services.ContentHandlerURL",
			  null,
			  {
		  		// summary: Provides convenience methods to construct URLs to the 
				// 		content handler servlet which serves most of the Portal 
				//		REST services.
				
				constructor: function ( /*String*/uri, /*String or int*/levels, /*String*/verb, /*String?*/params ) {
				  	ibm.portal.debug.entry( "ContentHandlerURL.constructor", [ uri, levels, verb, params ]);
				  	//If no URI is passed in, just return null.
					if ( uri == null ) {
						return null;
					}	
					
					if ( !levels ) { 
						levels = 2;
					}

					// check if we need to enforce a specific locale
					var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
					var locale = state.getLocale();
					if (locale) {
						if (params) {
							params += "&locale=" + locale;
						} else {
							params = "&locale=" + locale;
						}
					}
				  
				  	this.url = "";
					if ( uri.charAt(0) == "?" ) {
						this.url = this._fromQueryString( uri, params );	
					}
					else {
						this.url = this._fromURI( uri, levels, "download", params );
					}
					ibm.portal.debug.exit( "ContentHandlerURL.constructor" );
				},
				_fromQueryString: function ( /*String*/queryString, /*String?*/params  ) {
					// summary: Constructs an url from a given query string.
					// description: Constructs the URL to the content handler servlet using
					//		the given query string. Replaces any '&amp;' characters with '&' 		
					//		since Firefox thinks the 'amp;' is part of the parameter name.
					// queryString: the query string to construct the URL from
					// returns: String - the constructed URL
					ibm.portal.debug.entry( "fromQueryString", [ queryString ] );
				
					var str = ibmPortalConfig["contentHandlerURI"] + queryString;
					str = str.replace(/&amp;/g,"&" );
					if ( params ) {
						str = str + params;
					}
					
					if ( str.indexOf( "rep=compact" ) < 0 && str.indexOf( "rep=full" ) < 0 ) {
						str = str + "&rep=compact";
					}	
				
					ibm.portal.debug.exit( "fromQueryString", [ str ] );
					
					return str; //String	
				},
				_fromURI: function ( /*String*/uri, /*int?*/levels, /*String?*/verb, /*String?*/params ) {
					// summary: Constructs an URL using the given parameters.
					// description: Constructs the URL to the content handler servlet for
					//		the given URI, number of levels to traverse, and the action verb
					//		for the REST service to execute. It is not generally recommended
					//		to request more than 2 levels at a time since this will cause an
					//		expensive tree traversal server-side. The verb is generally expected
					//		to be set to "download".
					// uri: the uri for the resource to request
					// levels: the number of levels to include in the response
					// verb: the action to take (e.g. "view" or "download")
					// returns: String - the constructed URL
					ibm.portal.debug.entry( "ContentHandlerURL._fromURI", [ uri, levels, verb, params ] );
					
					uri = com.ibm.portal.utilities.encodeURI( uri );
					
					var qStr = "?uri=" + uri;
					
					if ( levels ) {
						qStr = qStr + "&levels=" + encodeURIComponent( levels );
					}
					
					if ( verb ) {
						qStr = qStr + "&mode=" + encodeURIComponent( verb );
					}
					if ( params ) {
						qStr = qStr + params;
					}
					
					if ( qStr.indexOf( "rep=compact" ) < 0 && qStr.indexOf( "rep=full" ) < 0 ) {
						qStr = qStr + "&rep=compact";
					}	
					
					return this._fromQueryString( qStr ); //String
				},
				getURI: function() {
					ibm.portal.debug.entry( "ContentHandlerURL.getURI" );
					return com.ibm.portal.utilities.decodeURI( this._extractParamValue( "uri" ) ); //String
				},
				getLevels: function () {
					return this._extractParamValue( "levels" );
				},
				getVerb: function () {
					return this._extractParamValue( "verb" );
				},
				_extractParamValue: function ( /*String*/ paramName ) {
					ibm.portal.debug.entry( "ContentHandlerURL._extractParamValue", [ paramName ] );
					
					var startIndex = this.url.indexOf( paramName );
					var endIndex = this.url.indexOf( "&", startIndex );
					var uriParamStr = this.url.slice( startIndex + paramName.length + 1, endIndex );
					
					ibm.portal.debug.exit( "ContentHandlerURL._extractParamValue", [ uriParamStr ]);
					
					return uriParamStr;
				}
			  }
);

dojo.require( "com.ibm.portal.utilities.html" );

dojo.declare( "com.ibm.portal.services.PortalRestServiceForm",
			  null,
			  {
			  	// summary: Object representation of a form that will be submitted to one of the Portal
			  	//		REST services. Encapsulates all the information about the form necessary to get
			  	//		the form info to the server.
			  	// method: the METHOD of the form (usually GET or POST)
			  	// isMultipart: indicates whether this is a multipart encoded form or not
			  	// encoding: the encoding used to submit the form
			  	// DomId: the ID of the DOM element corresponding to the form
			  	method: "GET",
			  	isMultipart: false,
			  	encoding: "application/x-www-form-urlencoded",
			  	DomId: null,
			  	constructor: function ( /*HTMLElement*/formElem ) {
			  		if ( formElem.getAttributeNode("method" ) ) {
			  			this.method = formElem.getAttributeNode( "method" ).value;	
			  		}
			  		if ( formElem.getAttributeNode( "encType" ) ) {
			  			this.encoding = formElem.getAttributeNode( "encType" ).value;
			  		}
			  		if ( formElem.getAttributeNode( "id" ) ) {	
			  			this.DomId = formElem.getAttributeNode( "id" ).value;
			  		}
			  		else {
			  			DomId = formElem;
			  		}	
			  		this.isMultipart = ( this.encoding == "multipart/form-data");
			  	},
			  	getDOMElement: function () {
			  		// summary: Returns the DOM element for this form.
			  		// returns: the DOM element corresponding to this form
			  		return dojo.byId( this.DomId );
			  	},
			  	submit: function () {
			  		// summary: Submits the form. This is necessary because the native submit function
			  		//		may have been overridden to ensure the form submission was intercepted.
			  		this.getDOMElement().submit();
			  	},
			  	toQuery: function () {
			  		// summary: Translates the form into a query string. Only elements which make sense
			  		//		for a query string are translated (i.e. NOT inputs of type "file").
			  		// returns: a string of name/value pairs suitable for being included as part of a url's
			  		//		query string.
			  		return com.ibm.portal.utilities.html.convertFormToQuery( this.getDOMElement() );
			  	}			  	
			  }
);

dojo.declare( "com.ibm.portal.services.PortalRestServiceRequest",
			  null,
			  {
			  	// summary: Sends a request to one of the Portal REST Services and handles the response.
				// description: Create a new instance of this object for every REST service request that is
				//		sent. The URL is passed in to the constructor. If a form node and form filter is 
				// 		passed into the constructor, the REST service request is handled as a form submit.
				// 		The request can also be executed synchronously if TRUE is passed into the constructor.
				// 		In general, only the model implementations should send these requests.
				
				constructor: function( /*ContentHandlerURL*/feedlocation, /*PortalRestServiceForm?*/form, /*boolean?*/textOnly, /*boolean?*/sync ) {
				  	ibm.portal.debug.entry( "PortalRestServiceRequest.constructor", [ feedlocation, form, textOnly, sync ] );
					this._feedURI = feedlocation.url;
					this._textOnly = textOnly;
					this._sync = sync;
					this._form = form;
					
					//Make sure this isn't undefined.
					if ( !this._sync ) {
							this._sync = false;
					}
					ibm.portal.debug.exit( "PortalRestServiceRequest.constructor" );
			    },
				create: function ( /*ibm.atom.Feed*/feed, /*ibm.atom.Entry*/entry, /*Function*/callbackfn ) {
					// summary: Creates an entry in a feed. 
					// description: Creates an entry in a feed. This operation is not implemented in this
					//		 class. It should be overridden by subclasses and properly implemented if
					// 		 the REST service supports the create operation.
					// feed: the ATOM feed object
					// entry: the ATOM entry to create
					// callbackfn: the function to call when the operation is complete
					dojo.unimplemented( "com.ibm.portal.services.PortalRestServiceRequest.create" );	
				},
				read: function ( /*Function?*/callbackfn, /*Object?*/addtlCallbackFnArguments ) {
					// summary: Read the ATOM feed provided by the REST service. 
					// description: If textOnly is set to true, the ATOM feed is returned as text and 
					//		 passed in as a single argument to the specified callback function. 
					//		 Otherwise, the ATOM feed is parsed into an {@link ibm.atom.Feed} object 
					//		 and passed into the callback function along with the original XML Document 
					//		 Object Model object.
					// callbackfn: the function to call when the operation is complete (only valid if
					//		 sync is false)
					// addtlCallbackFnArguments: any arguments to pass through to the callback function
					ibm.portal.debug.entry( "PortalRestServiceRequest.read", [ callbackfn, addtlCallbackFnArguments ] );
					
					com.ibm.portal.EVENT_BROKER.startRequest.fire( { uri: this._feedURI } );
					
					if ( this._textOnly ) {
						this._retrieveRawFeed( callbackfn, addtlCallbackFnArguments );
					}	
					else {
						this._retrieve( callbackfn, addtlCallbackFnArguments );
					}
					ibm.portal.debug.exit( "PortalRestServiceRequest.read");
				},
				update: function ( /*ibm.atom.Feed*/feed, /*ibm.atom.Entry*/entry, /*Function*/callbackfn ) {
					// summary: Updates an entry in a feed. 
					// description: Updates an entry in a feed. This operation is not implemented in this
					// 		class. It should be overridden by subclasses and properly implemented if the 
					//		REST service supports the update operation.
					// feed: the ATOM feed object
					// entry: the ATOM entry to create
					// callbackfn: the function to call when the operation is complete
					dojo.unimplemented( "com.ibm.portal.services.PortalRestServiceRequest.update" );
				},
				remove: function ( /*ibm.atom.Feed*/feed, /*ibm.atom.Entry*/entry, /*Function*/callbackfn ) {
					// summary: Removes an entry from a feed. 
					// description: Removes an entry from a feed. This operation is not implemented in this
					// 		class. It should be overridden by subclasses and properly implemented if the 
					//		REST service supports the remove operation.
					// feed: the ATOM feed object
					// entry: the ATOM entry to create
					// callbackfn: the function to call when the operation is complete
					dojo.unimplemented( "com.ibm.portal.services.PortalRestServiceRequest.remove" );
				},
				_retrieveRawFeed: function ( callbackfn, callbackargs ) {
					ibm.portal.debug.entry( "_retrieveRawFeed", [callbackfn, callbackargs] );
					var me = this;
					dojo.xhrGet({
						url: this._feedURI,
						load: function( type, data, evt ) {
							callbackfn( data, callbackargs );
							com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } ); 
						},
						sync: this._sync
					});
					
					ibm.portal.debug.exit( "_retrieveRawFeed" );
				},
				_retrieve: function ( callbackfn, callbackargs, formNode, formFilter ) {
					ibm.portal.debug.entry( "_retrieve", [ callbackfn ] );
					
					if ( this._form && this._form.isMultipart ) {
						this._doIframeRequest( callbackfn, callbackargs );
					}
					else {
						this._doXmlHttpRequest( callbackfn, callbackargs );
					}
					
					ibm.portal.debug.exit( "PortalRestServiceRequest._retrieve" );  
				},
				_doIframeRequest: function ( callbackfn, callbackargs ) {
					// summary: Handles a multi-part form submission.
					// description: Multi-part form submissions are not currently feasible using XMLHttpRequests, so
					// 		we need to submit these forms using IFRAMEs. 
					
					ibm.portal.debug.entry( "PortalRestServiceRequest._doIframeRequest", [ callbackfn ] );
					
					var iframe = null;
					var iframeName = dojo.dnd.getUniqueId(); 
					
					if ( dojo.isIE ) { 
						//Any other method of assigning the onload handler in IE seems to fail miserably...
						iframe = document.createElement( "<iframe name='" + iframeName + "' id='"+iframeName + "' src='about:blank' onload='com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handleMultiPartResult(this.id);'></iframe>" );
						//This is necessary so that we actually have the Function object to call when the iframe loads (since we seem to
						//only be able to assign an onload handler using text insertion methods).
						com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER._callbackfns[iframeName]= { fn: callbackfn, args: callbackargs };
						//IE applies an XSLT to an XML result in an IFRAME so we need text only.
						var url = new com.ibm.portal.utilities.HttpUrl( this._feedURI );
						url.addParameter( "ibm.web2.contentType", "text/plain" );
						this._form.getDOMElement().setAttribute( "action", url.toString() );
					}
					else {
						ibm.portal.debug.text( "Creating the iframe... name is: " + iframeName + "; url is: " + this._feedURI );
						iframe = document.createElement( "IFRAME" );
						iframe.setAttribute( "name", iframeName );
						iframe.setAttribute( "id", iframeName );
						var me = this;
						iframe.onload = function() { var xml = window.frames[iframeName].document; callbackfn( "load", xml, null, callbackargs ); com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } ); };
						this._form.getDOMElement().setAttribute( "action", this._feedURI );	
					} 
					//Can't set the iframe's display to 'none' or problems recognizing the iframe will result...
					iframe.style.visibility = "hidden";
					iframe.style.height = "1px";
					iframe.style.width = "1px";
					document.body.appendChild( iframe );
					
					//IE Workaround again... the name isn't always correctly assigned from the above createElement call. 
					if(window.frames[iframeName].name != iframeName) { window.frames[iframeName].name = iframeName; }
					
					ibm.portal.debug.text( "Setting the iframe target attribute to: " + iframeName );
					this._form.getDOMElement().setAttribute( "target", iframeName );
					this._form.submit();
					
					ibm.portal.debug.exit( "PortalRestServiceRequest._doIframeRequest" );
				},
				_doXmlHttpRequest: function ( callbackfn, callbackargs ) {
					// summary: Performs a standard request using an XMLHttpRequest object.
					// description: If an error response code is returned, the Error object is returned instead of the
					// 		xml payload. This means that a check should be done to see if the response is instanceof Error.
					ibm.portal.debug.entry( "PortalRestServiceRequest._doXmlHttpRequest", [ callbackfn, callbackargs ] );
					var content = {};
										
					ibm.portal.debug.text( "Attempting to retrieve: " + this._feedURI + "; synchronously? " + this._sync );
					var me = this;
					var args = {
						url: this._feedURI,
						content: content,
						//headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
						handle: function( response, ioArgs ) {
							ibm.portal.debug.entry( "PortalRestServiceRequest.handle", [ response, ioArgs ] );
							//Get the location of the resource actually returned by the response.
							//All re-directs have been followed at this point.
							var xhr = ioArgs.xhr;
							
							ibm.portal.debug.text( "XHR object: " + xhr ); 
							var config = com.ibm.portal.services.PortalRestServiceConfig;
							var digest = xhr.getResponseHeader( "X-Request-Digest" );
							
							if ( digest ) {
								config.digest = digest;	
							}
							
							if ( xhr.status == 200 ) {
								var data = response;
								var loc = xhr.getResponseHeader( "IBM-Web2-Location");
								
								if ( loc ) {
									//Check to see if we are going from a public resource to a protected resource.
									//This means a successful login has occurred and we need to force a full page 
									//refresh.
									if ( loc.indexOf( ibmPortalConfig["portalProtectedURI"]) >= 0 && me._feedURI.indexOf( ibmPortalConfig["portalPublicURI"] ) >= 0 ) {
										top.location.href = loc;
										return;
									}
								}
								
								var contentType = xhr.getResponseHeader( "Content-Type" );
								// If the HTML content is returned, this is probably another re-direction so we need to
								// force a full-page refresh.
								if ( contentType && contentType.indexOf( "text/html" ) >= 0 ) {
									
									var fullPageRefreshURL = me._feedURI;
									if ( loc ) { fullPageRefreshURL = loc; }

									if (dojo.cookie("WASReqURL") != null) {
										// create URL to current navigational state
										// we cannot use fullPageRefreshURL because it does not encode the entire nav state
										var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
										var fullStateUrl = state.createLinkToCurrentState();
										var cookieStr = "WASReqURL=" + fullStateUrl + "; path=/";
										// does not work with Dojo cookie APIs
										document.cookie=cookieStr;
									}

									//Fire the redirect event so any necessary clean up can occur.
									com.ibm.portal.EVENT_BROKER.redirect.fire( { url: fullPageRefreshURL } ); 
									top.location.href = fullPageRefreshURL;
									return;
								}	
								
								ibm.portal.debug.text( "Read feed: " + me._feedURI );
								
								if ( dojo.isIE ) {
									var doc = dojox.data.dom.createDocument( data );
									callbackfn( "load", doc, xhr, callbackargs );
								}
								else {
									callbackfn( "load", data, xhr, callbackargs );
								}
							}
							else {
								callbackfn( "error", response, null, callbackargs );
							}	
							
							com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } );
							ibm.portal.debug.exit( "PortalRestServiceRequest.handle" );
						},
						sync: this._sync,
						handleAs: "xml"	
					}
					
					var method = "Get";
					
					if ( this._form ) {
						//DOJO expects the content parameter to be of the format created by
						//dojo.queryToObject(). The contents of the content parameter are sent either
						//as part of the query string (method=GET) or as the body of the request (method=POST).
						args.content = dojo.queryToObject( this._form.toQuery() );
						method = com.ibm.portal.utilities.string.properCase( this._form.method );
					}
					
					if ( dojo.isIE ) {
						args.content["ibm.web2.contentType"] = "text/xml";
						args.handleAs = "text"; //temp
					}
					var config = com.ibm.portal.services.PortalRestServiceConfig;
					if ( config.timeout ) {
						args.timeout = config.timeout;
					}
					if ( config.digest ) {
						args.content["digest"] = config.digest;
					}
					
					var xhr = dojo["xhr" + method]( args );
					ibm.portal.debug.exit( "PortalRestServiceRequest._doXmlHttpRequest" );
				},
				toString: function () {
					return this._feedURI;
				}
			  }
);
com.ibm.portal.services.PortalRestServiceConfig = {
	timeout: null,
	digest: null
}

dojo.provide( "com.ibm.portal.services.PortletFragmentService" );

dojo.require( "dojox.data.dom" );

dojo.require( "com.ibm.portal.services.PortalRestServiceRequest" );
dojo.require( "com.ibm.portal.utilities" );
dojo.require( "com.ibm.portal.debug" );
dojo.require( "com.ibm.portal.EventBroker" );

dojo.declare( "com.ibm.portal.services.PortletFragmentURL",
			  null,
			  {
			  	constructor: function ( /*String*/uri ) {
				  	if ( uri.indexOf( "?uri=" ) == 0 ) {
						this.url = ibmPortalConfig["portalURI"] + uri;
						this.url = this.url.replace(/&amp;/g,"&" );
						this.url = this.url.replace( /lm:/, "pm:" );
					}
					else if ( uri.indexOf( "lm:" ) == 0 ) {
						this.url = ibmPortalConfig["portalURI"] + "?uri=fragment:" + uri;
						this.url = this.url.replace( /lm:/, "pm:" );
					}
					else {
						this.url = uri;
					}
				  }
			  }
);

dojo.declare( "com.ibm.portal.services.PortletInfo",
			  null,
			  {
				constructor: function( /*String*/wId, /*String*/pId, /*String*/pMarkup, /*Array*/pModes, /*Array*/wStates, /*Array*/depPortlets, /*Array*/otherPortlets, /*XMLDocument*/stateDOM, /*Array*/ stateVaryExpr, /*PortletMode*/ portletMode, /* WindowState */ windowState ) {
					ibm.portal.debug.entry( "PortletInfo.constructor", [ wId, pId, pMarkup, pModes, wStates, depPortlets, stateDOM ]);
					
					this.windowId = wId;
					this.portletId = pId;
					this.uri = "fragment:pm:oid:" + wId + "@oid:" + pId;
					this.markup = pMarkup;
					this.portletModes = pModes;
					this.windowStates = wStates;
					this.dependentPortlets = depPortlets;
					this.otherPortlets = otherPortlets;
					this.stateVaryExpressions = stateVaryExpr;
					this.updatedState = stateDOM;
					this.currentMode = portletMode;
					this.currentWindowState = windowState;
					
					ibm.portal.debug.exit( "PortletInfo.constructor" );
			    }
			  }
);

dojo.declare( "com.ibm.portal.services.PortletFragmentService",
			  null,
			  {
			  	namespaces: {
		            "xsl" : "http://www.w3.org/1999/XSL/Transform",
		            "thr" : "http://purl.org/syndication/thread/1.0",
		            "atom" : "http://www.w3.org/2005/Atom",
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
		            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
		            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
		            "xsi" : "http://www.w3.org/2001/XMLSchema-instance",
				"state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state",
				"state-vary" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state-vary"
		        },
				_flagPortletUrl: function ( /*String*/ url, /*String*/ portletId ) {
					ibm.portal.debug.entry( "PortletFragmentService._flagPortletUrl", [ url ] );
					
					// Check to see if the uri parameter is already included in url
					var uriIncluded = url.indexOf("uri=fragment:pm:oid:");

					//Create the url and add the necessary parameters.
					var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
					urlObj.addParameter( "ibm.web2.keepRenderMode", "false" );
					
					// If uriIncluded is -1, then add the uri as a parameter
					if(uriIncluded < 0 ){
						//Convert the portlet id to a fragment id.
						portletId = portletId.replace( /lm:/g, "fragment:pm:" );
						urlObj.addParameter( "uri", portletId );
					}
					
					ibm.portal.debug.exit( "PortletFragmentService._flagPortletUrl", [ urlObj.toString() ]);
					return urlObj.toString();
				},
				getPortletInfo: function ( /*String*/portletId, /*String*/ portletUrl, /*Function?*/ callbackFn, /*PortalRestServiceForm?*/form, /*Boolean?*/stateless ) {
					// summary: Retrieves the PortletInfo for the given portlet url.
					// description: Sends a request for the given portlet information, including markup for the current navigational state. If the request does not
					//		complete successfully, a null object will be returned or passed into the callback function. 
					// portletUrl: a portlet URL, should come directly from the Portlet Fragment Feed or be created using the
					//		NavigationalState.newPortletRenderURL() function.
					// callbackFn: OPTIONAL - a callback function to be called when the portlet info is retrieved. the first parameter is the portlet info object
					//		or NULL if an error occurred and the second parameter is the XMLHttpRequest object used to make the request.
					// formNode: OPTIONAL - the form to submit as part of the request OR the query string produced by a "serialized" form
					// formFilter: OPTIONAL - the form filter for elements to encode (see dojo documentation)
					// stateless: OPTIONAL - indicates that this request should not affect the state of the portlet. Setting this to
					//		true cancels the topic publishing for this request only. 
					ibm.portal.debug.entry( "PortletFragmentService.getPortletInfo", [portletId, portletUrl, callbackFn, form, stateless ] );
					
					//Two cases where we don't want this request to continue:
					//1. The url is a hash mark (frequenty used for simply firing onclick events).
					//2. The url is a javascript: url. Usually this would have been discovered by now, but in the off chance
					//		that it isn't, we just want to eval it and return. 
					if ( portletUrl == "#" || portletUrl == window.location.href + "#" ) {
						ibm.portal.debug.text( "Illegal portlet url provided: " + portletUrl );
						ibm.portal.debug.text( "Aborting request." );
						return false;
					}
					
					if ( com.ibm.portal.utilities.isJavascriptUrl( portletUrl )) {
						return eval( portletUrl );
					}
					
					if ( !stateless ) {
						com.ibm.portal.EVENT_BROKER.startFragment.fire( { id: portletId } );
					}
	
					// if the URL is relative (e.g. "?fragment:pm:oid:..."), we need to resolve it
					var tempUrl = portletUrl;
					if(tempUrl.indexOf(top.location.href) == 0) {
						// here we're checking to see if the URL is actually the same as our current
						// url... we then trim that down and see what it really points to, like if it's
						// a relative URL in disguise 
						tempUrl = tempUrl.substring(top.location.href.length);
						while(tempUrl.length > 0 && tempUrl.charAt(0) == "/") {
							tempUrl = tempUrl.substring(1);
						}
					}
					if (tempUrl.indexOf('?') == 0) {
						var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
						portletUrl = state.resolveRelativePortletURL(tempUrl);
					}

					if ( com.ibm.portal.utilities.isExternalUrl( portletUrl ) ) {
						self.location.href = portletUrl;	
					}
					else{
						var url = { url: this._flagPortletUrl( portletUrl, portletId ) }
						var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, form );
						var me = this;
						serviceReq.read(
							function ( /*String*/type, /*XMLDocument*/xmlData, /*XMLHttpRequest*/xhr ) {
								var pInfo = null;
								if ( type == "load" ) {
									pInfo = me.createPortletInfo( xmlData );
								}
								
								if ( xmlData instanceof Error ) {
									pInfo = xmlData;
								}
								
								if ( !stateless ) {
									me._fireEvents( pInfo, portletId, xhr );
								}
								
								if ( callbackFn ) {
									callbackFn( pInfo, xhr );
								}
							}	
						);
					}

					ibm.portal.debug.exit( "PortletFragmentService.getPortletInfo" );	
				},
				readWindowID: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readWindowID", [xmlData] );
					
					var windowIdXPath = "/atom:feed/atom:entry/atom:id";
					var nodes = com.ibm.portal.xpath.evaluateXPath( windowIdXPath, xmlData, this.namespaces );
					var windowId = dojox.data.dom.textContent( nodes[0] );
					
					ibm.portal.debug.exit( "PortletFragmentService.readWindowID", [windowId.substring(4)]);		
					return windowId.substring(4);	
				},
				readPortletID: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readPortletID", [xmlData] );
					
					var portletIdXPath = "/atom:feed/atom:id";
					var nodes = com.ibm.portal.xpath.evaluateXPath( portletIdXPath, xmlData, this.namespaces );
					var portletId = dojox.data.dom.textContent( nodes[0] );
					
					ibm.portal.debug.exit( "PortletFragmentService.readPortletID", [portletId.substring(4)]);
					return portletId.substring(4);
				},
				readMarkup: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readMarkup", [xmlData] );
					var contentsXPath = "/atom:feed/atom:entry/atom:content";
					var nodes = com.ibm.portal.xpath.evaluateXPath(contentsXPath, xmlData, this.namespaces);
					var markup = "";
					
					if (nodes != null && nodes.length > 0) {
			            markup = dojox.data.dom.textContent( nodes[0] );
			        }
					
					ibm.portal.debug.exit( "PortletFragmentService.readMarkup", [markup]);
					return markup;
				},
				readPortletModes: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readPortletModes", [xmlData] );
					var modesXPath = "/atom:feed/atom:entry/atom:link[@portal:rel='portlet-mode']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(modesXPath, xmlData, this.namespaces);
					var modes = new Array();
					
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							modes.push( { "link": nodes[i].getAttribute("href"), "mode": nodes[i].getAttribute("title") } );
			            }
			        }
					ibm.portal.debug.exit( "PortletFragmentService.readPortletModes", [modes]);
					return modes;
				},
				readWindowStates: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readWindowStates", [xmlData] );
					var wStatesXPath = "/atom:feed/atom:entry/atom:link[@portal:rel='window-state']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(wStatesXPath, xmlData, this.namespaces);
					var wStates = new Array();
					
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							wStates.push( { "link": nodes[i].getAttribute("href"), "mode": nodes[i].getAttribute("title") } );
			            }
			        }
					ibm.portal.debug.exit( "PortletFragmentService.readWindowStates", [wStates]);
					return wStates;
				},
				readDependentPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readDependentPortlets", [xmlData] );
					var depPortletsXPath = "/atom:feed/atom:link[@portal:rel='dependent']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(depPortletsXPath, xmlData, this.namespaces );
					var depPortlets = new Array();
					
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							depPortlets.push( { "link": nodes[i].getAttribute("href"), "portlet": nodes[i].getAttribute("title"), "uri": nodes[i].getAttribute("portal:uri") ? nodes[i].getAttribute("portal:uri") :  nodes[i].getAttribute("uri") } );
			            }
			        }
					
					ibm.portal.debug.exit( "PortletFragmentService.readDependentPortlets", [depPortlets]);
					return depPortlets;
				},
				readOtherPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readOtherPortlets", [xmlData] );
					var otherPortletsXPath = "/atom:feed/atom:link[@portal:rel='other']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(otherPortletsXPath, xmlData, this.namespaces );
					var otherPortlets = new Array();
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							otherPortlets.push( { "link": nodes[i].getAttribute("href"), "portlet": nodes[i].getAttribute("title"), "uri": nodes[i].getAttribute("portal:uri") } );
			            }
			        }
					ibm.portal.debug.exit( "PortletFragmentService.readOtherPortlets", [otherPortlets]);
					return otherPortlets;
				},
				readStateVaryExpressions: function (/*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readStateVaryExpressions", [xmlData] );
					var stateVaryXPath = "/atom:feed/atom:entry/state-vary:state-vary/state-vary:expr";
					var nodes = com.ibm.portal.xpath.evaluateXPath(stateVaryXPath, xmlData, this.namespaces );
					var varyExpr = new Array();
					if (nodes != null && nodes.length > 0) {
						var nodeCount = nodes.length;
						for (var i = 0; i < nodeCount; i++) {
							var txtNode = nodes[i].firstChild;
							if (txtNode != null) {
								varyExpr.push( txtNode.nodeValue );
							}
						}
					}
					ibm.portal.debug.exit( "PortletFragmentService.readStateVaryExpressions", [varyExpr]);
					return varyExpr;
				},
				readPortletState: function ( /*XMLDocument*/xmlData ) {
					return this._readPortletState(xmlData);
				},
				_readPortletState: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readPortletState", [xmlData] );
					var stateXPath = "/atom:feed/atom:entry/state:root";
					var nodes = com.ibm.portal.xpath.evaluateXPath(stateXPath, xmlData, this.namespaces);
					var result = null;
					if ( nodes != null && nodes.length > 0 ) {
						var doc = dojox.data.dom.createDocument();
						com.ibm.portal.utilities.addExternalNode(doc, nodes[0]);
						result = doc;
					} else {
						stateXPath = "/atom:feed/state:root";
						nodes = com.ibm.portal.xpath.evaluateXPath(stateXPath,xmlData,this.namespaces);
						if ( nodes != null && nodes.length > 0 ) {
							var doc = dojox.data.dom.createDocument();
							com.ibm.portal.utilities.addExternalNode(doc, nodes[0]);
							result = doc;
						}
					}
					ibm.portal.debug.exit( "PortletFragmentService.readPortletState", [ result ] );
					return result;
				},
				_fireEvents: function ( /*PortletInfo*/pInfo, /*String*/portletId, /*XMLHttpRequest*/xhr ) {
					this._fireGlobalPortletStateChange( pInfo, portletId, xhr );
					//this._fireIndividualPortletStateChange( pInfo );
				},
				_fireGlobalPortletStateChange: function ( /*PortletInfo*/pInfo, /*String*/portletId, /*XMLHttpRequest*/xhr ) {
					//dojo.event.topic.publish( "/portal/PortletFragmentService/portletInfoRetrieved", { portletInfo: pInfo } );
					com.ibm.portal.EVENT_BROKER.endFragment.fire( { portletInfo: pInfo, id: portletId, xhr: xhr } );		
				},
				_fireIndividualPortletStateChange: function ( /*PortletInfo*/pInfo ) {
					//ibm.portal.debug.text( "Firing individual portlet event: /portal/navigation/portlet/" + pInfo.portletId + "@" + pInfo.windowId  );
					//dojo.event.topic.publish( "/portal/PortletFragmentService/portletInfoRetrieved/oid:" + pInfo.windowId + "@oid:" + pInfo.portletId , { portletInfo: pInfo } );	
				},
				createPortletInfo: function( /*XMLDocument*/xmlData ) {
					var windowId = this.readWindowID( xmlData );
					var portletId = this.readPortletID( xmlData ); 
					var markup = this.readMarkup( xmlData );
					var modes = this.readPortletModes( xmlData );
					var wStates = this.readWindowStates( xmlData );
					var depPortlets = this.readDependentPortlets( xmlData );
					var otherPortlets = this.readOtherPortlets( xmlData );
					var newState = this.readPortletState( xmlData );
					var stateVaryExpr = this.readStateVaryExpressions( xmlData );
					// determine the current portlet mode and window state
					var currentState = newState;
					if (currentState == null) {
						currentState = this._readPortletState(xmlData);
					}
					var stateMgr = new com.ibm.portal.state.StateManager();
					var portletAcc = stateMgr.newPortletAccessor( windowId, currentState );
					var mode = portletAcc.getPortletMode();
					var winState = portletAcc.getWindowState();
					return new com.ibm.portal.services.PortletInfo( windowId, portletId, markup, modes, wStates, depPortlets, otherPortlets, newState, stateVaryExpr, mode, winState );
				} 
			  }
);

dojo.declare( "com.ibm.portal.services.IndependentPortletFragmentService",
			  com.ibm.portal.services.PortletFragmentService,
			  {
			  	// summary: Ignores any dependent portlets.
			  	
			  	readDependentPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "DependentPortletFragmentService.readDependentPortlets", [xmlData] );
					var depPortlets = new Array();
				
					ibm.portal.debug.exit( "DependentPortletFragmentService.readDependentPortlets", [depPortlets]);
					return depPortlets;
				},	
				readOtherPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "DependentPortletFragmentService.readOtherPortlets", [xmlData] );
					var otherPortlets = new Array();
				
					ibm.portal.debug.exit( "DependentPortletFragmentService.readOtherPortlets", [otherPortlets]);
					return otherPortlets;
				},
				readPortletState: function ( /*XMLDocument*/xmlData ) {
					return null;
				}
			}
);

if(!dojo._hasResource["ibm.portal.portlet.portlet"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.portal.portlet.portlet"] = true;
dojo.provide("ibm.portal.portlet.portlet")







ibm.portal.portlet._SafeToExecute = false;

if ( window.addEventListener ) {
    window.addEventListener( "load", function () { ibm.portal.portlet._SafeToExecute = true; }, false );
}
else if ( window.attachEvent ) {
    window.attachEvent( "onload", function () { ibm.portal.portlet._SafeToExecute = true; } );
}

dojo.declare("ibm.portal.portlet.PortletWindow", null, {

    STATUS_UNDEFINED: 0,
    STATUS_OK: 1,
    STATUS_ERROR: 2,
    
    constructor: function (/*String*/ windowid) {
        /**
         * summary:
        *     Constructor
        *
        * windowid: The window id
        */

        if (windowid==null) return; // special constructor for static initializers

        this.windowID = windowid;
        var element = document.getElementById("com.ibm.wps.web2.portlet.preferences."+this.windowID);
        this.preferenceEditID = element.getAttribute("editid");
        this.preferenceConfigID = element.getAttribute("configid");
        this.preferenceEditDefaultsID = element.getAttribute("editdefaultsid");
        this.pageID = element.getAttribute("pageid");

        this.attributes = new Array();
        this._queuedFuncs = new Array();
        this.portletState = new ibm.portal.portlet.PortletState(windowid);
        
        this.isCSA = false; 
        try {
            this.isCSA = (typeof(document.isCSA) != "undefined" );
        }
        catch (e) {}
        
        //Certain functions aren't safe to be executed until the page has loaded. This will execute any functions
        //which have been queued at the page load event.
        var me = this;
        function executeQueued() {
            for ( var i = 0; i < me._queuedFuncs.length; i++ ) {
                me._queuedFuncs[i]();
            }   
        }
        
        if ( window.addEventListener ) {
            window.addEventListener( "load", function () { if ( !ibm.portal.portlet._SafeToExecute) { ibm.portal.portlet._SafeToExecute = true; } executeQueued(); }, false );
        }
        else if ( window.attachEvent ) {
            window.attachEvent( "onload", function () { if ( !ibm.portal.portlet._SafeToExecute) { ibm.portal.portlet._SafeToExecute = true; } executeQueued(); } );
        }
    },
    
    reportError: function(/*ibm.portal.portlet.Error*/ error) {
    /**
     * summary:
     *     Reports an error to the framework for display purposes
     */
        var code;
        if (error.getErrorCode()==ibm.portal.portlet.Error.ERROR) code = "error";
        else if (error.getErrorCode()==ibm.portal.portlet.Error.INFO) code = "info";
        else if (error.getErrorCode()==ibm.portal.portlet.Error.WARN) code = "warning";
        var statusMessage = { "_type" : code,
                              "_message" : error.getMessage(), 
                              "_details" : error.getDescription() };
        if ( this.isCSA )
        {
            dojo.publish("/portal/status",[{message: statusMessage}]);
        }
        else {
            if ( typeof( console ) != "undefined") {
                if (error.getErrorCode()==ibm.portal.portlet.Error.ERROR) 
                    console.error(statusMessage._message+"\n"+statusMessage._details);
                else if (error.getErrorCode()==ibm.portal.portlet.Error.INFO) 
                    console.info(statusMessage._message+"\n"+statusMessage._details);
                else if (error.getErrorCode()==ibm.portal.portlet.Error.WARN) 
                    console.warn(statusMessage._message+"\n"+statusMessage._details);
            }
            else {
                alert(statusMessage._type.toUpperCase()+"\nMessage: "+statusMessage._message+"\nDetails: "+statusMessage._details);
            }
        }
    },
    getAttribute: function(/*String*/ name) {
    /**
     * summary:
     *     Gets the attribute for the given name scoped by the portlet window
     */
        return this.attributes[name]; // Object, maybe null
    },

    setAttribute: function(/*String*/ name, /*Object*/ value) {
    /**
     * summary:
     *     Sets the attribute for the given name scoped by the portlet window
     */
         var ret = this.attributes[name];
         this.attributes[name] = value;
         return ret; // Object, maybe null
    },

    removeAttribute: function(/*String*/ name) {
    /**
     * summary:
     *     Removes the attribute for the given name scoped by the portlet window
     *     
     */
        this.attributes[name] = null;
    },

    clearAttributes: function() {
    /**
     * summary:
     *     Removes all attributes scoped by the portlet window
     *     
     */
         this.attributes = new Array();
    },

    getPortletState: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the portlet state bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet state.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
        var returnObject = this.portletState;
        var pointer2this = this;
        var result = null;
        if (callbackfn != null) {
            // invoke the callback function
            callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_OK, returnObject);
        } else {
            result = { "portletWindow" : pointer2this, 
                       "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                       "returnObject" : returnObject };
        }
        return result;
    },

    setPortletState: function(/*PortletState*/state, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the portlet state bean (holding the navigational state) scoped
     *     by the window id. The behavior of this function varies depending on
     *     whether or not the surrounding theme is capable of dealing with
     *     client-side state changes. If not, this function will trigger a full
     *     page refresh. Otherwise the provided callback function decides how to
     *     react on the state change.
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes storing the portlet state.
     *     The callback function defines how to update the portlet's markup
     *     to reflect the new navigational state. If no callback function is
     *     provided (or null is passed in) the method executes synchronously,
     *     and in addition refreshes the portlet (default behavior of reacting
     *     to a navigational state change.
     */
        this.portletState = state;
        
        if ( this.isCSA ) {
            // if no callback function is provided, execute the default behavior:
            // refresh the portlet by sending a render request to the server
            if (callbackfn == null) {
                var navState = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                var url = navState.newPortletRenderURL( this.windowID );
                var service = new com.ibm.portal.services.PortletFragmentService();
                service.getPortletInfo( "lm:oid:" + this.windowID + "@oid:"+ this.pageID, url );
            }
        } else {
            var stateManager = new com.ibm.portal.state.StateManager(ibmPortalConfig["contentHandlerURI"]);
            stateManager.reset( state.portletAccessor.stateDOM );
            // serialize the state into a portal URL
            var serializationMgr = stateManager.getSerializationManager();
            var serializationResult = serializationMgr.serialize(stateManager.getState());
            
            // return the URL
            var serializedState = serializationResult["returnObject"];
            
            var url = serializedState;
            window.location.href = url;
        }
        // dispatch
        return this.getPortletState(callbackfn);
    },
    
    _queueUp: function ( /*Function*/funcToQ ) {
        // summary: Certain functions are not safe to execute before the page loads. Any function
        //      passed to this function will be queued up and executed only after the page loads.
        // funcToQ: the function to queue
        this._queuedFuncs.push( funcToQ );
    },
    
    _throwInappropriateRequestError: function ( /*String*/funcName ) {
        throw new Error( "Cannot execute a synchronous call before the page loads! Please use an onload handler to execute this call to \"" + funcName + "\"." );
        return null;
    },
    
    getPortletPreferences: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the portlet preferences bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
        if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.getPortletPreferences( callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "getPortletPreferences" );
            }
        }
        
        var portletMode = this.getPortletState().returnObject.getPortletMode();
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var tempurl = document.getElementById("com.ibm.wps.web2.portlet.root."+this.windowID).innerHTML;
         var idx = tempurl.indexOf("--portletwindowid--");
         var _url = tempurl.replace(/--portletwindowid--/g, this.windowID);
         
         if ( _url.indexOf( "?" ) < 0 ) {
            _url = _url + "?"; 
         } 
         
         _url += "&verb=download&levels=-all&rep=compact&preferences=aggregated";

         this.requestedPreferenceID = "pm:oid:"+this.preferenceEditID; //+"@oid:"+this.pageID;
         if (portletMode==ibm.portal.portlet.PortletMode.CONFIG) 
             this.requestedPreferenceID = "pm:oid:"+this.preferenceConfigID;
         else if (portletMode==ibm.portal.portlet.PortletMode.EDIT_DEFAULTS) 
             this.requestedPreferenceID = "pm:oid:"+this.preferenceEditDefaultsID;
         
         var pointer2this = this;
         var pointer2return = null;
         
         dojo.xhrGet({
             url: _url,
             handleAs: "xml",
             headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
             sync: (callbackfn)?false:true,
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if(type == "load") {
                      var responseXML = response;
                      
                      //IE doesn't populate the responseXML property here because of the content-type of the response. So we need to create the XML
                      //document from text to be able to execute XPath queries.
                      if ( !responseXML || ( typeof( dojox.data.dom.innerXML( response )) == "undefined" ) ) {
                        responseXML = dojox.data.dom.createDocument( ioArgs.xhr.responseText );
                      }
                      
                      var returnObject = 
                          new ibm.portal.portlet.PortletPreferences(pointer2this.windowID,
                                                                        pointer2this.requestedPreferenceID,
                                                                        responseXML );
                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,returnObject);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : returnObject };
                      }
                  }else if(type == "error"){
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    setPortletPreferences: function(/*PortletPreferences*/preferences, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the portlet preferences bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes storing the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.setPortletPreferences( preferences, callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "setPortletPreferences" );
            }
        }
         
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var tempurl = document.getElementById("com.ibm.wps.web2.portlet.root."+this.windowID).innerHTML;
         var idx = tempurl.indexOf("--portletwindowid--");
         var _url = tempurl.replace(/--portletwindowid--/g, this.windowID);
         if ( _url.indexOf( "?" ) < 0 ) {
             _url += "?verb=download";
         }
         else {
             _url += "&verb=download";
         }

         var requestedPreferenceID = preferences.requestedPreferenceID;
         // strip down to necessary data
         var expr = "/atom:feed/atom:entry[atom:id='"+requestedPreferenceID+"']";
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, preferences.xmlData, preferences.ns);
         var entry;
         if (nodes && nodes.length>0) {
             entry = nodes[0];
         }
         else {
             return null; // TODO
         }
         var parentNode = entry.parentNode;
         expr = "/atom:feed/atom:entry";
         nodes = ibm.portal.xml.xpath.evaluateXPath(expr, preferences.xmlData, preferences.ns);
         for (var i=0; i<nodes.length; i++) {
             var node = nodes[i];
             if (node!=entry) {
                 parentNode.removeChild(node);
             }
         }
         
         //alert (dojox.data.dom.innerXML(preferences.xmlData));

         var pointer2this = this;
         var pointer2return = null;


         dojo.rawXhrPut({
             url: _url,
             sync: (callbackfn)?false:true,
             putData: dojox.data.dom.innerXML(preferences.xmlData),
             contentType: "application/xml",
             handleAs: "xml",
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if(type == "load") {
                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,preferences);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : preferences };
                      }
                  }else if(type == "error") {
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    getUserProfile: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the current user profile
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.getUserProfile( callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "getUserProfile" );
            }
        }
         
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var _url = document.getElementById("com.ibm.wps.web2.portlet.user."+this.windowID).innerHTML;

         var pointer2this = this;
         var pointer2return = null;
         dojo.xhrGet({
             url: _url,
             headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
             sync: (callbackfn)?false:true,
             handleAs: "xml",
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if (type == "load") {
                      var responseXML = response;
                      
                      // IE doesn't populate the responseXML property here because of the content-type of the response.
                      // We need to create the XML document from text to be able to execute XPath queries.
                      if ( !responseXML || ( typeof( dojox.data.dom.innerXML( response )) == "undefined" ) ) {
                          responseXML = dojox.data.dom.createDocument( ioArgs.xhr.responseText );
                      }

                      var returnObject = 
                          new ibm.portal.portlet.UserProfile(pointer2this.windowID, responseXML );

                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,returnObject);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : returnObject };
                      }
                  } else if(type == "error"){
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    setUserProfile: function(/*userProfile*/userProfile, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the current user profile
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         //var serializer = new XMLSerializer();
         //resultStr = serializer.serializeToString(userProfile.xmlData);
         //alert(resultStr);
        if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.setUserProfile( userProfile, callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "setUserProfile" );
            }
        }

         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var _url = document.getElementById("com.ibm.wps.web2.portlet.user."+this.windowID).innerHTML;

         var pointer2this = this;
         var pointer2return = null;
         dojo.rawXhrPost({
             url: _url,
             sync: (callbackfn)?false:true,
             postData: dojox.data.dom.innerXML(userProfile.xmlData),
             contentType: "application/xml",
             handleAs: "xml",
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if(type == "load") {
                      if (callbackfn) {
                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,userProfile);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                              "returnObject" : userProfile };
                      }
                  } else if(type == "error"){
                      if (callbackfn) {
                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
                      }
                      else {
                          pointer2return = {"portletWindow" : pointer2this, 
                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
                              "returnObject" : null };
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },
    newXMLPortletRequest: function() {
        // summary: Creates a new XMLPortletRequest object with the current page ID and portlet window ID.
        // returns: a new XMLPortletRequest
        return new ibm.portal.portlet.XMLPortletRequest( this.pageID, this.windowID ); // ibm.portal.portlet.XMLPortletRequest
    }
});


/**
 * summary:
 * The <CODE>PortletPreferences</CODE> interface allows the portlet to store
 * configuration data. It is not the
 * purpose of this interface to replace general purpose databases.
 * 
 * description:
 * There are two different types of preferences:
 * <ul>
 * <li>modifiable preferences - these preferences can be changed by the
 *     portlet in any standard portlet mode (<code>EDIT, HELP, VIEW</code>).
 *     Per default every preference is modifiable.
 * <li>read-only preferences - these preferences cannot be changed by the
 *     portlet in any standard portlet mode, but may be changed by administrative modes.
 *     Preferences are read-only, if the are defined in the
 *     deployment descriptor with <code>read-only</code> set to <code>true</code>,
 *     or if the portlet container restricts write access.
 * </ul>
 * <p>
 * Changes are persisted when the <code>store</code> method is called.
 */

dojo.declare("ibm.portal.portlet.PortletPreferences", null, {
    constructor: function (/*String*/ windowid, 
                           /*String*/ requestedpreferenceid,
                           /*XMLDocument */ data) {
        /** 
        * summary:
        *     Objects of this type should never be called directly but always created
        *     through the PortletManagers method getPortletPreferences.
        *
        * windowid: The window id
        */

        this.windowID = windowid;
        this.requestedPreferenceID = requestedpreferenceid;
        this.xmlData = data;
        this.xsltURL = dojo.moduleUrl( "ibm", "portal/portlet/");
        
        this.ns = {
            "xsl" : "http://www.w3.org/1999/XSL/Transform",
            "thr" : "http://purl.org/syndication/thread/1.0",
            "atom" : "http://www.w3.org/2005/Atom",
            "xhtml" : "http://www.w3.org/1999/xhtml",
            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
            "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
        };

        // performance optimization
        this.internal_reset();
    },

    getMap: function (){
    /**
     * summary: 
     *     Returns a <code>Map</code> of the preferences.
     *     If no preferences exist this method returns an empty <code>Map</code>.
     */
        //ibm.portal.debug.entry( "PortletPreferences.internal_applyXSLT" , [ feed.toString(), dom, args ] );

        if (this.result_getMap) {
            return this.result_getMap;
        }

        //load the xslt
        var xslDoc = ibm.portal.xml.xslt.loadXsl(this.xsltURL + "PortletPreferencesMap.xsl");

        //apply the xslt
        if (xslDoc.documentElement == null) {alert("xslDoc is null");}
        var results = ibm.portal.xml.xslt.transform(this.xmlData, xslDoc, null, {"selectionid":this.requestedPreferenceID}, true);
        if (results == null) {
            this.result_getNames = null;
            return null;
        }

        var prefs = eval( results );
        // "portletPreferences = { \"preferences\": [ { \"name\":\"parallel\" , \"values\": [ \"false\" ] , \"read-only\": \"false\" }, { \"name\":\"test\" , \"values\": [ \"newValue\" ] , \"read-only\": \"false\" } ] };"

        if (prefs)            
            prefs = prefs.preferences;

        this.result_getMap = prefs;

        return this.result_getMap; /* object[] {name: string, values: string[], readonly: boolean}
     *             a <code>Map</code> containing preference names as 
     *             keys and preference values as map values, or an empty <code>Map</code>
     *             if no preference exist.
     */
    },

    getNames: function (){
    /**
     * summary:
     *     Returns all of the keys that have an associated value,
     *     or an empty <code>Enumeration</code> if no keys are available.
     */
         //ibm.portal.debug.entry( "PortletPreferences.internal_applyXSLT" , [ feed.toString(), dom, args ] );
         
         if (this.result_getNames) {
             return this.result_getNames;
         }
         
         //load the xslt
         var xslDoc = ibm.portal.xml.xslt.loadXsl(this.xsltURL + "PortletPreferencesNames.xsl");

         //apply the xslt
         if (xslDoc.documentElement == null) {alert("xslDoc is null");}
         var results = ibm.portal.xml.xslt.transform(this.xmlData, xslDoc, null, {"selectionid":this.requestedPreferenceID}, true);
         if (results == null) {
             this.result_getNames = null;
             return null;
         }

         var prefsNames = eval( results );
         // "portletPreferences = { \"preferences\": [ { \"name\":\"parallel\" , \"values\": [ \"false\" ] , \"read-only\": \"false\" }, { \"name\":\"test\" , \"values\": [ \"newValue\" ] , \"read-only\": \"false\" } ] };"

         if (prefsNames)
             prefsNames = prefsNames.names;

         this.result_getNames = prefsNames;

         return this.result_getNames; /* String[] - an Enumeration of the keys that have an associated value,
         *         or an empty <code>Enumeration</code> if no keys are
        *         available.
        */
    },

    getValue: function( /*String*/ key, /*String*/ def ){
    /**
     * summary:
     *     Returns the first String value associated with the specified key of this preference.
     * description:
     *     If there is one or more preference values associated with the given key 
     *     it returns the first associated value.
     *     If there are no preference values associated with the given key, or the 
     *     backing preference database is unavailable, it returns the given 
     *     default value.
     *
     * key: String - key for which the associated value is to be returned
     * def: String - the value to be returned in the event that there is no 
     *            value available associated with this <code>key</code>.
     */
         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']/base:value";
         // /atom:feed/atom:entry[atom:id='pm:oid:5_J12SV3D00O6JD02ECR217F00E0@oid:6_J12SV3D00O6JD02ECR217F00M6']/atom:content/model:portlet-entity/model:portletpreferences[@name='parallel']
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         var result;
         if (nodes && nodes.length>0) {
             result = nodes[0].getAttribute("value");
         }
         else {
             result = def;
         }

         return result; /* String - the value associated with <code>key</code>, or <code>def</code>
         *         if no value is associated with <code>key</code>, or the backing
         *         store is inaccessible.
         */
    },

    getValues: function( /*String*/ key, /*String[]*/ def ){
    /**
     * summary:
     *     Returns the String array value associated with the specified key in this preference.
     * description:
     *     Returns the specified default if there is no value
     *     associated with the key, or if the backing store is inaccessible.
     *
     *     If the implementation supports <i>stored defaults</i> and such a
     *     default exists and is accessible, it is used in favor of the
     *     specified default.
     *
     * key: String - key for which associated value is to be returned.
     * def: String[] - the value to be returned in the event that this
     *        preference node has no value associated with <code>key</code>
     *        or the associated value cannot be interpreted as a String array,
     *        or the backing store is inaccessible.
     */

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']/base:value";
         // /atom:feed/atom:entry[atom:id='pm:oid:5_J12SV3D00O6JD02ECR217F00E0@oid:6_J12SV3D00O6JD02ECR217F00M6']/atom:content/model:portlet-entity/model:portletpreferences[@name='parallel']
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);
         var result;
         if (nodes && nodes.length>0) {
             result = new Array();
             for (var i=0; i<nodes.length; i++) {
                 result[i] = nodes[i].getAttribute("value");
             }
         }
         else {
             result = def;
         }
        
         return result; /* String[] - the String array value associated with
         *         <code>key</code>, or <code>def</code> if the
         *         associated value does not exist.
         */
    },

    isReadOnly: function( /*String*/ key ){
    /**
     * summary:
     *     Returns true, if the value of this key cannot be modified by the user.
     * description:
     *     Modifiable preferences can be changed by the
     *     portlet in any standard portlet mode (<code>EDIT, HELP, VIEW</code>). 
     *     Per default every preference is modifiable.
     * 
     *     Read-only preferences cannot be changed by the
     *     portlet in any standard portlet mode, but inside of custom modes
     *     it may be allowed changing them.
     *     Preferences are read-only, if they are defined in the 
     *     deployment descriptor with <code>read-only</code> set to <code>true</code>,
     *     or if the portlet container restricts write access.
     *
     * key: String - key for which the associated value is to be returned
     */

         var id = this.requestedPreferenceID;
         var expr = "/atom:feed/atom:entry[atom:id='"+id+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         
         var result = false;
         if (nodes && nodes.length>0) {
             var temp = nodes[0].getAttribute("read-only");
             if (temp != null) {
                 // convert to a boolean
                 if ( temp == "true" ) {
                    result = true;
                 }
             }
         }
         return result; // boolean - false, if the value of this key can be changed, or if the key is not known
    },

    reset: function ( /*String*/ key ){
    /**
     * summary:
     *     Resets or removes the value associated with the specified key.
     * description:
     *     If this implementation supports stored defaults, and there is such
     *     a default for the specified preference, the given key will be 
     *     reset to the stored default.
     *
     *     If there is no default available the key will be removed.
     *
     * key: String - key to reset
     */
         this.internal_reset();

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);
         if (nodes && nodes.length>0) {
             var parent = nodes[0].parentNode;
             parent.removeChild(nodes[0]);
         }

    },

    setValue: function ( /*String*/ key, /*String*/ value ){
    /**
     * summary:
     *     Associates the specified String value with the specified key in this preference.
     * description:
     *     The key cannot be <code>null</code>, but <code>null</code> values
     *     for the value parameter are allowed.
     *
     * key: String - key with which the specified value is to be associated.
     * value: String - value to be associated with the specified key.
     */
         var values = new Array();
         values[0] = value;
         this.setValues(key,values);
    },

    setValues: function( /*String*/ key, /*String[]*/ values ){
    /**
     * summary:
     *     Associates the specified String array value with the specified key in this preference.
     * description:
     *     The key cannot be <code>null</code>, but <code>null</code> values
     *     in the values parameter are allowed.
     *
     * key: String - key with which the  value is to be associated
     * values: String[] - values to be associated with key
     */
         this.internal_reset();

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);

         var prefNode = null;
         if (nodes && nodes.length>0) {
             prefNode = nodes[0];
             for (var i=prefNode.childNodes.length-1; i>=0; i--) {
                 prefNode.removeChild(prefNode.childNodes[i]);
             }
         }
         else {
             var expr2 = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*";
             var nodes2 = ibm.portal.xml.xpath.evaluateXPath(expr2, this.xmlData, this.ns);
             if (dojo.isIE) {
                 prefNode = this.xmlData.createNode(1, "model:portletpreferences", this.ns.model); // 1 = NODE_ELEMENT
             }
             else {
                 prefNode = this.xmlData.createElementNS(this.ns.model,"model:portletpreferences");
             }
             prefNode.setAttribute("name",key);
             prefNode.setAttribute("read-only","false");
             nodes2[0].appendChild(prefNode);
         }
         for (var i=0; i<values.length; i++) {
             var newNode;
             if (dojo.isIE) {
                 newNode = this.xmlData.createNode(1, "base:value", this.ns.base); // 1 = NODE_ELEMENT
                 var attribute = this.xmlData.createNode(2, "xsi:type",this.ns.xsi ); // 1 = NODE_ATTRIBUTE
                 attribute.nodeValue = "String";
                 newNode.setAttributeNode(attribute);
             }
             else {
                 newNode = this.xmlData.createElementNS(this.ns.base,"base:value");
                 newNode.setAttributeNS(this.ns.xsi,"xsi:type","String");
             }
             newNode.setAttribute("value",values[i]);
             prefNode.appendChild(newNode);
         }
    },

    internal_reset: function() {
         this.result_getMap = null;
         this.result_getNames = null;
    },

    clone:function()
    {
        var resultStr = dojox.data.dom.innerXML(this.xmlData);
        var xmlDataCopy = dojox.data.dom.createDocument( resultStr ); 

        return new ibm.portal.portlet.PortletPreferences(this.windowID, 
                                                             this.requestedPreferenceID,
                                                             xmlDataCopy);
    }


});















dojo.declare("ibm.portal.portlet.PortletMode", null, {
    VIEW: "view",
    EDIT: "edit",
    EDIT_DEFAULTS: "edit_defaults",
    HELP: "help",
    CONFIG: "config"    
});

dojo.declare("ibm.portal.portlet.WindowState", null, {
    NORMAL: "normal",
    MINIMIZED: "minimized",
    MAXIMIZED: "maximized"
});

dojo.declare("ibm.portal.portlet.PortletState", null, {
    constructor: function (/*String or XMLDocument*/ windowid, /*String?*/portletWindowId ) {
        /**
         * summary:
         *     Objects of this type should never be called directly but always created
         *     through the PortletManagers method getPortletState.
         *
         * windowid: The window id
         */
        var stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
        if ( dojo.isString( windowid ) ) {
            var stateDOM = this._getExistingState( windowid, stateManager.getSerializationManager() );
            stateManager.reset( stateDOM );
        }
        else {
            stateManager.reset( windowid );
            windowid = portletWindowId; 
        }
        
        // get a state accessor that allows for reading and writing portlet state
        this.portletAccessor = stateManager.newPortletAccessor(windowid);
        // also get the Parameters object for performance reasons
        this.renderParameters = this.portletAccessor.getRenderParameters();
    },
    _isCSA: function () {
        var retVal = false;
        try {
            retVal = (typeof(document.isCSA )!= "undefined" );
        }
        catch (e) {
            //do nothing here... something was not defined so we aren't in CSA
        }
        
        return retVal;
    },
    _getExistingState: function ( /*String*/windowid, serializationMgr ) {
        var retVal = null;
        if ( this._isCSA() ) {
            // CSA has access to the current state
            retVal = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState().stateDOM;
        } else if (serializationMgr != null) {
            // get the current state from the server
            var result = serializationMgr.deserialize( location.href );
            retVal = result.returnObject;
        } else {
            // fall back to an "empty" state
            retVal = dojox.data.dom.createDocument();
        }
        return retVal;
    },
    getPortletMode:function() {
    /**
     * summary:
     *     Returns the portlet mode.
     */
        return this.portletAccessor.getPortletMode(); /*ibm.portal.portlet.PortletMode*/
    },

    setPortletMode:function(/*ibm.portal.portlet.PortletMode*/ portletMode) {
    /**
     * summary:
     *     Sets the portlet mode.
     */
        this.portletAccessor.setPortletMode(portletMode);
        return portletMode;
    },

    getWindowState:function() {
    /**
     * summary:
     *     Returns the window state.
     */
        return this.portletAccessor.getWindowState(); /*ibm.portal.portlet.WindowState*/
    },
    
    setWindowState:function(/*ibm.portal.portlet.WindowState*/windowState) {
    /**
     * summary:
     *     Sets the window state.
     */
        this.portletAccessor.setWindowState(windowState);
        return windowState;
    },

    getParameterNames: function() {
    /**
     * summary:
     *     Returns a String[] containing the names of all parameters that have an
     *     associated value. Returns an empty array if no parameters are available.
     */
        return this.renderParameters.getNames(); // String[]
    },

    getParameterValue: function(/*String*/ name) {
    /**
     * summary:
     *     Returns the String value associated with the render parameter with the given name.
     *     
     * description:
     *     If there are one or more render parameter values associated with the given name 
     *     it returns the first associated value.
     *
     * name: String - Name of the render parameter for which the associated value is to
     *     be returned. Must not be NULL.
     */
        return this.renderParameters.getValue(name); // String
    },

    getParameterValues: function(/*String*/ name) {
    /**
     * summary:
     *     Returns the String[] representing the values that are associated with
     *     the render parameter with the given name.
     *
     * name: String - Name of the render parameter for which the associated values
     *     are to be returned. Must not be NULL.
     */
        return this.renderParameters.getValues(name); // String[]
    },

    getParameterMap: function() {
    /**
     * summary:
     *     Returns an unmodfiable map of render parameters.
     */
        return this.renderParameters.getMap(); /* object[] {name: string, values: string[]} */
    },

    setParameterValue: function(/*String*/ name, /*String*/ value) {
    /**
     * summary:
     *     Sets a simple String value for the render parameter with the given name. Overrides
     *     the value of the render parameter if it already exists.
     *
     * name: String - The name of the render parameter. Must not be NULL.
     * value: String - The value of the render parameter that should be set. Must not be NULL. 
     */
        this.renderParameters.setValue(name, value);
        return value; // String
    },
    
    setParameterValues: function(/*String*/ name, /*String[]*/ values) {
    /**
     * summary:
     *     Sets a String[] representing the value of the render parameter with the given name.
     *     Overrides the value of the render parameter if it already exists.
     *
     * name: String - The name of the render parameter. Must not be NULL.
     * value: String[] - The value of the render parameter that should be set. Must not be NULL. 
     */
        this.renderParameters.setValues(name, values);
        return values; // String[]
    },
    
    setParameterMap:function(/*object[] {name: string, values: string[]}*/map, /*boolean*/replace) {
    /**
     * summary:
     *     Adds all render parameters that are contained in the given map. Overrides the values
     *     of parameters that already exist. Returns the resulting map.
     *
     * map: Object[] {name: String, values: String[]} - Two dimensional object array
     *     representing the map of parameters that should be added. Must not be NULL.
     * replace: boolean - Optional argument that indicates whether the given map should
     *     completely replace the existing parameter map.
     */
        // check if we need to delete the existing parameters
        if (replace == true) {
            this.renderParameters.clear();
        }
        this.renderParameters.putAll(map);
        return this.renderParameters.getMap(); /* object[] {name: string, values: string[]} */
    },
    
    removeParameter:function(/*String*/name) {
    /**
     * summary:
     *     Removes the render parameter with the given name.
     *
     * name: String - Name of the render parameter that should be removed. Must not be NULL.
     */
        this.renderParameters.remove(name);
    }
    
});







dojo.require( "com.ibm.portal.services.PortletFragmentService" );

dojo.declare("ibm.portal.portlet.XMLPortletRequest", null, {
     // summary: Provides XMLHttpRequest functionality for addressing portlets.
     // description: Wrappers and exposes all standard properties on an XMLHttpRequest, as well
     //         as, exposing an additional property called onportletstateready. This new property
     //         can be assigned a function that will be called when the portlet state is ready. If
     //         a handler is set for onportletstateready, the portlet state will not be updated 
     //         globally. The XMLPortletRequest will only work for URLs which point to the portlet
     //         fragment feed.
     // 
     // onportletstateready: Function - handler for intercepting global portlet state updates, the 
     //         PortletState object is the lone parameter passed into this function when called.
     // onreadystatechange: Function - handler for the readyState property changing
     // readyState: int - indicates the current state of the request
     // responseText: String - the text of the response body (only available when readyState == 4)
     // responseXML: XMLDocument - the xml document in the response body
     // status: int - the HTTP status code received in the response
     // statusText: String - the String corresponding to the HTTP status code
     // 
    onreadystatechange: null,     /*Function*/
    readyState: 0,            /* int */
    responseText: null,           /* String */
    responseXML: null,            /* Document */
    status: null,                 /* int */
    statusText: null,             /* String */
    onportletstateready: null,     /* Function(ibm.portal.portlet.PortletState) */
    _location: null,              /* String - stores the location of the request */
    
    constructor: function ( /*String*/page, /*String*/window ) {
        // summary: Constructor for the XMLPortletRequest object. 
        // description: A correctly initialized instance of this object can be obtained from the portlet window object.  The only
        //      case where this constructor should be called directly is when sending an XPR to a different portlet than the
        //      one represented by the portletWindow object. 
        // page: the page's ObjectID
        // window: the porltet window's ObjectID
        this.pageID = page;
        this.windowID = window;
    },
    _getXHR: function () {
         // summary: Creates the XMLHttpRequest object if necessary and returns it
         // returns: the XMLHttpRequest instance associated with this XMLPortletRequest
        if ( !this._xhr ) { 
            this._xhr = this._createXHR();
        }
        
        return this._xhr; //XMLHttpRequest
    },
    _createXHR: function () {
        // summary: Cross-browser creation of XMLHttpRequest or it's equivalent.
        // returns: a new instance of an XMLHttpRequest or it's equivalent
        var retVal = null;
        
        if ( typeof( XMLHttpRequest ) != "undefined" ) {
            retVal = new XMLHttpRequest();  
        }
        else {
            retVal = new ActiveXObject( "Microsoft.XMLHTTP" );
        }
        
        return retVal; //XMLHttpRequest
    },
    _onreadystatechangehandler: function () {
        // summary: Wrappers any user-set onreadystatechange handlers and processes the 
        //      response from the Portlet Fragment feed.
        // description: Intercepts the underlying XMLHttpRequest readyState changes and
        //      extracts the necessary information from the Portlet Fragment feed to 
        //      make the XMLPortletRequest object ready for consumption by the developer.
        var xhr = this._getXHR();
        
        //Copy over common properties that may have changed.
        this.readyState = xhr.readyState;
        
        //Extract the response portlet state and any dependent portlets
        if ( this.readyState == 4 ) {
            //Can't copy these common properties over until readyState is 4 because of IE
            this.responseText = xhr.responseText;
            this.responseXML = xhr.responseXML;
            this.status = xhr.status;
            this.statusText = xhr.statusText;
            //Use our service object for convenience here. We're just using to parse the
            //ATOM Feed response, not for actually retrieving any data because we need
            //direct access to the XMLHttpRequest object in this use case.
            var service = new com.ibm.portal.services.PortletFragmentService();
            this.responseText = service.readMarkup( xhr.responseXML );
            //Will need to set this based on the atom:content type attribute, but since we
            //aren't supporting any other markup types but HTML out of the box, this should
            //never be set.
            this.responseXML = null;
            
            this._handleDependentPortlets( service.readDependentPortlets( xhr.responseXML ) );
            var doGlobalStateUpdate = true;
            
            if ( this.onportletstateready != null ) {
                var xmlState = service.readPortletState( xhr.responseXML );
                var service = new com.ibm.portal.services.PortletFragmentService();  
                var windowId = service.readWindowID( xhr.responseXML );
                var portletState = new ibm.portal.portlet.PortletState( xmlState, windowId );
                
                //allow developer to handle portlet state
                doGlobalStateUpdate = this.onportletstateready( portletState );
            }
            
            if ( doGlobalStateUpdate && this._isCSA() )
            {
                //update global state
                var portletInfo = service.createPortletInfo( xhr.responseXML );
                service._fireGlobalPortletStateChange( portletInfo );
            }   
            
        }
        
        //Finally call the user's onreadystate change handler.
        if ( this.onreadystatechange != null ) {
            this.onreadystatechange();
        }       
    },
    _handleDependentPortlets: function ( /*Array*/portlets ) {
        // summary: Trigger reloads for all dependent portlets.
        // description: Reloads any portlets whose content has changed as a result of
        //      this XMLPortletRequest. This generally affects C2A portlets. If we are 
        //      not in Client-Side Aggregation, this will trigger a full page refresh.
        if ( this._isCSA() ) {
            var portletService = new com.ibm.portal.services.PortletFragmentService();
            for ( var i = 0; i < portlets.length; i++ ) {
                var portletid = portlets[i].uri;
                portletid = portletid.replace( /fragment:pm:/g, "lm:" );
                com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.getFragment( portletid ).setLoading();
                portletService.getPortletInfo( portletid, portlets[i].link );
            }   
        }
        else {
            if ( portlets.length > 0 ) {
                //Until we have Differential Page Rendering on the server side, we have no 
                //choice but to do a full page refresh, but we must point to the page
                //so the action isn't submitted twice and all portlets are rendered with fresh
                //parameters 
                
                window.location.href = this._newPageURL();
            }   
        }
    },
    _isCSA: function () {
        var retVal = false;
        try {
            retVal = (typeof(document.isCSA )!= "undefined" );
        }
        catch (e) {
            //do nothing here... something was not defined so we aren't in CSA
        }
        
        return retVal;
    },
    _flag: function ( /*String*/ uriToFlag ) {
        // summary: Flags a URL with the appropriate url parameters to retrieve a portlet fragment.
        var id = "lm:oid:" + this.windowID + "@oid:" + this.pageID;
        var service = new com.ibm.portal.services.PortletFragmentService();
        return service._flagPortletUrl( uriToFlag, id );
    },
    _newPageURL: function ( ) {
        // summary: Creates a new page url based on the portlet window id and page id.
        // description: Creates a new page url to the page containing the portlet that 
        // originated this XMLPortletRequest object.
        
        // get a copy of the state which contains the state of our portlet only    
        var stateManager = new com.ibm.portal.state.StateManager(ibmPortalConfig["contentHandlerURI"]);
        var stateDOM = dojox.data.dom.createDocument();
        stateManager.reset( stateDOM );
            
        var portletState = stateManager.newPortletAccessor( this.windowID ).getPortletState();
        
        var selectionAcc = stateManager.newSelectionAccessor(portletState);
        selectionAcc.setPageSelection(this.pageID);
        
        // serialize the state into a portal URL
        var serializationMgr = stateManager.getSerializationManager();
        var serializationResult = serializationMgr.serialize(portletState);
        
        // return the URL
        var serializedState = serializationResult["returnObject"];
        
        var url = serializedState; 
        
        return url; // String
    },
    open: function( /*String*/ method, /*String*/ uri ) {
        // @see XMLHttpRequest
        this.open( method, uri, false );  
    },
    open: function( /*String*/ method, /*String*/ uri,  /*boolean*/ async ) {
        // @see XMLHttpRequest
        var xhr = this._getXHR();
        var me = this;
        this._location = uri;
        xhr.onreadystatechange = function () { me._onreadystatechangehandler(); };
        xhr.open( method, this._flag(uri), async );
    },
    setRequestHeader: function( /*String*/ header, /*String*/ value ) {
        // @see XMLHttpRequest
        this._getXHR().setRequestHeader( header, value );
    },
    send: function( /*String or Document*/ data ) {
        // @see XMLHttpRequest
        this._getXHR().send( data );   
    },
    abort: function( ) {
        // @see XMLHttpRequest
        this._getXHR().abort();
    },
    getAllResponseHeaders: function( ) {
        // @see XMLHttpRequest
        return this._getXHR().getAllResponseHeaders();
    },
    getResponseHeader: function( /*String*/ header ) {
        // @see XMLHttpRequest
        return this._getXHR().getResponseHeader( header );
    }

});

















dojo.declare("ibm.portal.portlet.UserProfile", null, {

    constructor: function (/*String*/ windowid,
                           /*XMLDocument */ data) {
    /**
     * summary:
     *     Objects of this type should never be called directly but always created
     *     through the PortletWindows method getUser.
     *
     * windowid: The window id
     */
        this.windowID = windowid;
        this.xmlData = data;

        this.ns = {
            "xsl" : "http://www.w3.org/1999/XSL/Transform",
            "atom" : "http://www.w3.org/2005/Atom",
            "xhtml" : "http://www.w3.org/1999/xhtml",
            "xsi" : "http://www.w3.org/2001/XMLSchema-instance",
            "um" : "http://www.ibm.com/xmlns/prod/websphere/um.xsd"
        };
    },

    getAttribute:function(/*String*/ name) {
    /**
     * summary:
     *     Returns the value of the given user attribute name as defined by the P3P Spec.
     * description:
     *     The portlet must define the user information attribute it is interested in
     *     inside the user-attribute section of the deployment descriptor. 
     *     If an attribute is not supported by the current runtime system or if it is supported
     *     but not defined for a particular user, then this method will return null
     * 
     * name: String - the name of the P3P attribute to return
     */
         var expr = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']/um:attributeValue";
         //var expr = "/um:profile[@type='user']/um:attribute[@name='"+name+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         

         var result = null;
         if (nodes && nodes.length>0) {
             if (nodes[0].textContent) {
                 result = nodes[0].textContent;
             }
             else {
                 result = nodes[0].text;
             }
         }

         return result; /* String */

    },

    setAttribute:function(/*String*/ name, /*String*/ value) {
         var expr = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']/um:attributeValue";
         //var expr = "/um:profile[@type='user']/um:attribute[@name='"+name+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         var previousText = null;
         if (nodes && nodes.length>0) {
             if (nodes[0].textContent) {
                 previousText = nodes[0].textContent;
                 nodes[0].textContent = value;
             }
             else {
                 previousText = nodes[0].text;
                 nodes[0].text = value;
             }
         }
         else {
             // TODO             
             var expr1 = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']";
             var nodes1 = ibm.portal.xml.xpath.evaluateXPath(expr1, this.xmlData, this.ns);
             var attrNode = null;
             if (nodes1 && nodes1.length>0) {
                 attrNode = nodes1[0];
             }
             else {
                 var expr2 = "/atom:entry/atom:content/um:profile[@type='user']";
                 var nodes2 = ibm.portal.xml.xpath.evaluateXPath(expr2, this.xmlData, this.ns);
                 
                 if (dojo.isIE) {
                     attrNode = this.xmlData.createNode(1, "um:attribute", this.ns.um); // 1 = NODE_ELEMENT
                 }
                 else {
                     attrNode = this.xmlData.createElementNS(this.ns.um, "um:attribute");
                 }
                 attrNode.setAttribute("type","xs:string");
                 attrNode.setAttribute("multiValued","false");
                 attrNode.setAttribute("name",name);
                 nodes2[0].appendChild(attrNode);
             }
             var newNode;
             if (dojo.isIE) {
                 newNode = this.xmlData.createNode(1, "um:attributeValue", this.ns.um); // 1 = NODE_ELEMENT
                 newNode.text = value;
             }
             else {
                 newNode = this.xmlData.createElementNS(this.ns.um, "um:attributeValue");
                 newNode.textContent = value;
             }
             attrNode.appendChild(newNode);
         }

         return previousText;
    },

    clone:function()
    {
        var resultStr = dojox.data.dom.innerXML(this.xmlData);
        var xmlDataCopy = dojox.data.dom.createDocument( resultStr ); 

        return new ibm.portal.portlet.UserProfile(this.windowID,
                                                      xmlDataCopy);
    }
    
});







dojo.declare("ibm.portal.portlet.Error", null, {

    INFO: 0,
    WARN: 1,
    ERROR: 2,
    
    constructor: function (/*String*/ errorCode, /*String*/ message, /*String*/ description) {
        this.errorCode = errorCode;
        this.message = message;
        this.description = description;
    },
    
    getErrorCode: function() {
    /**
     * summary:
     *     Returns the error code
     */
        return this.errorCode;
    },
    
    getMessage: function() {
    /**
     * summary:
     *     Returns the message
     */
        return this.message;
    },
    
    getDescription: function() {
    /**
     * summary:
     *     Returns the description
     */
        return this.description;
    }

});


{ // create static initializers
var com_ibm_portal_portlet_portletwindow = new ibm.portal.portlet.PortletWindow();
ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED = com_ibm_portal_portlet_portletwindow.STATUS_UNDEFINED;
ibm.portal.portlet.PortletWindow.STATUS_OK = com_ibm_portal_portlet_portletwindow.STATUS_OK;
ibm.portal.portlet.PortletWindow.STATUS_ERROR = com_ibm_portal_portlet_portletwindow.STATUS_ERROR;
com_ibm_portal_portlet_portletwindow = null;
var com_ibm_portal_portlet_portletmode = new ibm.portal.portlet.PortletMode();
ibm.portal.portlet.PortletMode.VIEW = com_ibm_portal_portlet_portletmode.VIEW;
ibm.portal.portlet.PortletMode.EDIT = com_ibm_portal_portlet_portletmode.EDIT;
ibm.portal.portlet.PortletMode.EDIT_DEFAULTS = com_ibm_portal_portlet_portletmode.EDIT_DEFAULTS;
ibm.portal.portlet.PortletMode.HELP = com_ibm_portal_portlet_portletmode.HELP;
ibm.portal.portlet.PortletMode.CONFIG = com_ibm_portal_portlet_portletmode.CONFIG;
com_ibm_portal_portlet_portletmode = null;
var com_ibm_portal_portlet_windowstate = new ibm.portal.portlet.WindowState();
ibm.portal.portlet.WindowState.NORMAL = com_ibm_portal_portlet_windowstate.NORMAL;
ibm.portal.portlet.WindowState.MINIMIZED = com_ibm_portal_portlet_windowstate.MINIMIZED;
ibm.portal.portlet.WindowState.MAXIMIZED = com_ibm_portal_portlet_windowstate.MAXIMIZED;
com_ibm_portal_portlet_windowstate = null;
var com_ibm_portal_portlet_error = new ibm.portal.portlet.Error();
ibm.portal.portlet.Error.INFO = com_ibm_portal_portlet_error.INFO;
ibm.portal.portlet.Error.WARN = com_ibm_portal_portlet_error.WARN;
ibm.portal.portlet.Error.ERROR = com_ibm_portal_portlet_error.ERROR;
com_ibm_portal_portlet_error = null;
}


}

